package edu.rochester.cs.quagents3.client;
import java.io.Writer;
import java.util.Scanner;

/**
 * Class which represents a BatchRangefinder query. 
 */
public class BatchRangefinder extends ProtocolZero{
	private static final String opcode = "rb";
	
	private final int type,distance,width,height;
	private final double rot,azi,sample_data_yaw,sample_data_pitch;
	
	private boolean formatted; //formatted iff !running and formatData() called
	
	//global index references
	private static final int DISTANCE = 0;
	private static final int ENTITY = 1;
	private double[][] data;
	
	public BatchRangefinder(int type, int distance, double rot, double azi,
			int width, int height, double sample_data_yaw, double sample_data_pitch, priority_t priority){
		super(true,CommandType.BATCHRANGEFINDER,priority);
		this.type = type;
		this.distance = distance;
		this.width = width;
		this.height = height;
		this.rot = rot;
		this.azi = azi;
		this.sample_data_yaw = sample_data_yaw;
		this.sample_data_pitch = sample_data_pitch;
		formatted = false;
	}
	public BatchRangefinder(int type, int distance, double rot, double azi,
			int width, int height, double sample_data_yaw, double sample_data_pitch, priority_t priority,long period){
		super(true,CommandType.BATCHRANGEFINDER,period,priority);
		this.type = type;
		this.distance = distance;
		this.width = width;
		this.height = height;
		this.rot = rot;
		this.azi = azi;
		this.sample_data_yaw = sample_data_yaw;
		this.sample_data_pitch = sample_data_pitch;
		formatted = false;
	}
	public BatchRangefinder(int type, int distance, double rot, double azi,
			int width, int height, double sample_data_yaw, double sample_data_pitch, priority_t priority,long period, long delay){
		super(true,CommandType.BATCHRANGEFINDER,period,delay,priority);
		this.type = type;
		this.distance = distance;
		this.width = width;
		this.height = height;
		this.rot = rot;
		this.azi = azi;
		this.sample_data_yaw = sample_data_yaw;
		this.sample_data_pitch = sample_data_pitch;
		formatted = false;
	}
	
	
	protected void setOp(){
		op = Client.convertPriority(priority) + " " + opcode + " " + id + " "
		+ type + " " + distance + " " + rot + " " +  azi+ " " +width+ " " + height +
		" "+ sample_data_yaw+ " " + sample_data_pitch;
	}
	
	/**
	 * For efficiency, BatchRangefinder data is initially stored without formatting.
	 * In order to access and utilize data, the program must first call formatData(), which formats
	 * and stores the data in relevant structures. This is called automatically only when the user first attempts to access
	 * structure data.
	 * NOTE: This may take a long time!
	 */
	//we don't want to worry about parsing huge string at readin, use this instead
	//note: careful when this is called, may take a while to complete!
	private void formatData(){
		paramscan = new Scanner(datastring);
		int indicies = paramscan.nextInt();
		data = new double[2][indicies]; //this might be very large
		for(int i=0; i<indicies; i++){
			data[DISTANCE][i] = paramscan.nextDouble();
			data[ENTITY][i] = paramscan.nextInt();
		}
		formatted = true;
	}
	
	/**
	 * Returns entire formatted data set.<br/>
	 * NOTE: This is very large!
	 * @return Entity-distance tuples for all points in the data set. 
	 */
	//let the user decide how to use
	public double[][] getData(){
		if(!formatted)
			formatData();
		return data;
	}
	
	/**
	 * Given entity id, find distance to that entity, if found in the data set.
	 * @param entityid Entity to find the distance to.
	 * @return If entity is within data set, distance to that entity. Else, -1.
	 */
	public double entityDistance(int entityid){
		for(int i=0; i<data[ENTITY].length; i++)
			if(data[ENTITY][i]==entityid)
				return data[DISTANCE][i];
		return -1; //entity not found
	}
	
	public String getDatastring(){
		return datastring;
	}
	
	/**
	 * Allows access to parameter.
	 * @return Type of rangefinder requested.
	 */
	public int getType(){return type;}
	
	/**
	 * Allows access to parameter.
	 * @return Distance of rangefinder requested.
	 */
	public int getDistance(){return distance;}
	
	/**
	 * Allows access to parameter.
	 * @return Width of bitmap requested.
	 */
	public int getWidth(){return width;}
	
	/**
	 * Allows access to parameter.
	 * @return Height of bitmap requested.
	 */
	public int getHeight(){return height;}
	
	/**
	 * Allows access to parameter.
	 * @return Initial rotational angle specified.
	 */
	public double getRotationalAngle(){return rot;}
	
	/**
	 * Allows access to parameter.
	 * @return Initial azimuthal angle specified.
	 */
	public double getAzimuthalAngle(){return azi;}
	
	/**
	 * Allows access to parameter.
	 * @return Yaw differential specified.
	 */
	public double getSampleDataYaw(){return sample_data_yaw;}
	
	/**
	 * Allows access to parameter.
	 * @return Pitch differential specified.
	 */
	public double getSampleDeltaPitch(){return this.sample_data_pitch;}
	
	//we'll worry about parsing data later -- for now just store string
	public void setData(String params) {
		datastring = params;
	}
}
