package edu.rochester.cs.quagents3.client;
import java.io.Writer;
import java.util.Scanner;

/**
 * Class which represents a CanSee query.
 */
public class CanSee extends ProtocolZero{
	private static final String opcode = "cs";
	
	private final int entityid;
	
	private boolean cansee;
	private double targetdist;
	private double targetyaw;
	private double targetpitch;
	
	public CanSee(int entityid, priority_t priority){
		super(true,CommandType.CANSEE,priority);
		this.entityid = entityid;
	}
	public CanSee(int entityid){
		super(true,CommandType.CANSEE);
		this.entityid = entityid;
	}
	public CanSee(int entityid, priority_t priority,
			long period){
		super(true,CommandType.CANSEE,period,priority);
		this.entityid = entityid;
	}
	public CanSee(int entityid,
			long period){
		super(true,CommandType.CANSEE,period);
		this.entityid = entityid;
	}
	public CanSee(int entityid, priority_t priority,
			long period,long delay){
		super(true,CommandType.CANSEE,period,delay,priority);
		this.entityid = entityid;
	}
	public CanSee(int entityid,
			long period,long delay){
		super(true,CommandType.CANSEE,period,delay);
		this.entityid = entityid;
	}
	
	/**
	 * Allows access to parameter.
	 * @return Entity ID requested.
	 */
	public int getEntityID(){return entityid;}
	
	/**
	 * Returns the distance to target. Will return 0 if target cannot be seen.
	 * @return Distance to target entity. 
	 */
	public double getDistanceToTarget(){
		return targetdist;
	}
	
	/**
	 * Returns the yaw to target. Will return 0 if target cannot be seen.
	 * @return Yaw to target entity. 
	 */
	public double getYawToTarget(){
		return targetyaw;
	}
	
	/**
	 * Returns the pitch to target. Will return 0 if target cannot be seen.
	 * @return Pitch to target entity. 
	 */
	public double getPitchToTarget(){
		return targetpitch;
	}
	
	/**
	 * If Quagent can see target entity.
	 * @return True if Quagent can see entity, false if Quagent cannot.
	 */
	public boolean canSeeTarget(){;
		return cansee;
	}
	
	public void setData(String params) {
		datastring = params;
		paramscan = new Scanner(params);
		String s = paramscan.next();
		if(s.equals("t")) cansee = true;
		else if(s.equals("f")) cansee=false;
		else System.err.println("protocol error in CanSee setData");
		targetdist = paramscan.nextDouble();
		targetyaw = paramscan.nextDouble();
		targetpitch = paramscan.nextDouble();
	}
	
	protected void setOp(){
		op = Client.convertPriority(priority) + " " + opcode + " " + id+
		" " + entityid;
	}
	
}
