package edu.rochester.cs.quagents3.client;

public class CartesianSensor2D extends ProtocolOne{

	private final Rangefinder forward;
	private final Rangefinder backward;
	private final Rangefinder right;
	private final Rangefinder left;
	
	private boolean forwardterm,backwardterm,rightterm,leftterm;
	
	public CartesianSensor2D(){
		super(CommandType.CARTESIANSENSOR2D);
		forward = new Rangefinder(priority_t.NOW);
		forward.addTerminationListener(new ForwardListener());
		backward = new Rangefinder (180,priority_t.NOW);
		backward.addTerminationListener(new BackwardListener());
		left = new Rangefinder(90,priority_t.NOW);
		left.addTerminationListener(new LeftListener());
		right = new Rangefinder(270,priority_t.NOW);
		right.addTerminationListener(new RightListener());
	}

	public CartesianSensor2D(long period){
		super(CommandType.CARTESIANSENSOR2D,period);
		forward = new Rangefinder(priority_t.NOW);
		forward.addTerminationListener(new ForwardListener());
		backward = new Rangefinder(180,priority_t.NOW);
		backward.addTerminationListener(new BackwardListener());
		left = new Rangefinder(90,priority_t.NOW);
		left.addTerminationListener(new LeftListener());
		right = new Rangefinder(270,priority_t.NOW);
		right.addTerminationListener(new RightListener());
	}
	
	protected void addChildren(){
		children.add(forward);
		children.add(backward);
		children.add(right);
		children.add(left);
	}
	
	public void update() {
		if(!hasTerminated()) waitForTerminate();
		else{run(); waitForTerminate();}
	}

	public void run() {
		if(paused) return;
		if(!running){
			running = true;
			resetBooleans();
			forward.run();
			backward.run();
			left.run();
			right.run();
		}
	}
	
	private void resetBooleans(){
		forwardterm=backwardterm=rightterm=leftterm=false;
	}
	
	public double getForwardDistance(){
		return forward.getEntityRange();
	}
	
	public double getBackwardDistance(){
		return backward.getEntityRange();
	}
	
	public double getLeftDistance(){
		return left.getEntityRange();
	}
	
	public double getRightDistance(){
		return right.getEntityRange();
	}
	
	public int getForwardEntity(){
		return forward.getEntityID();
	}
	
	public int getBackwardEntity(){
		return backward.getEntityID();
	}
	
	public int getLeftEntity(){
		return left.getEntityID();
	}
	
	public int getRightEntity(){
		return right.getEntityID();
	}
	
	
	@Override
	public String toString(){
		String rtn = "";
		rtn += "-------------------------------------------------------\n"
			+// "Quagent: " + executor.getQuagentID() + ",\t"+executor.name+
			"\nCommand: " + id + " - " + whatami +
			"\nTime: " + System.currentTimeMillis() +
			"\nSensor Data\nForward\t\tEntity ID: " +getForwardEntity()+"  "+"Distance: " +getForwardDistance()+
			"\nBackward:\tEntity ID: "+getBackwardEntity()+"  "+"Distance: "+getBackwardDistance()+
			"\nRight:\t\tEntity ID: "+getRightEntity()+"  "+"Distance: "+getRightDistance()+
			"\nLeft:\t\tEntity ID: "+getLeftEntity()+"  "+"Distance: "+getLeftDistance()+
			"\n-------------------------------------------------------";

		return rtn;
	}
	
	private void validateTerminationState(){
		if(forwardterm && backwardterm && rightterm && leftterm)
			this.terminate();
	}
	
	private class ForwardListener implements CommandTerminationListener{
		public void commandTerminated(Command com) {
			forwardterm=true;
			validateTerminationState();
		}
	}
	
	private class BackwardListener implements CommandTerminationListener{
		public void commandTerminated(Command com){
			backwardterm=true;
			validateTerminationState();
		}
	}
	
	private class RightListener implements CommandTerminationListener{
		public void commandTerminated(Command com){
			rightterm=true;
			validateTerminationState();
		}
	}	
	
	private class LeftListener implements CommandTerminationListener{
		public void commandTerminated(Command com){
			leftterm=true;
			validateTerminationState();
		}
	}
	
}
