package edu.rochester.cs.quagents3.client;
import java.awt.Image;
import java.awt.Point;
import java.io.IOException;

import javax.sound.sampled.SourceDataLine;

/**
 * This interface provides a multitude of conviniece functions for using Quagents in Quagents3.
 * Everything in this interface may be accessed through QuagentEntity functions - ClientInf is no more powerful than QuagentEntity.
 */
public interface ClientInf extends QuagentEntity{
    /* ================================================================= *
     * ====================== Live Commands ============================ *
     * ================================================================= */
	/* ----------------------------------------------------------------- *
	 * -------------------------- Actions ------------------------------ *
	 * ----------------------------------------------------------------- */
	
	//------------moveBy-------------
	/**
	 * Moves the Quagent the specified distance in the specified direction.
	 * @param dir The rotational angle, in degrees left of Quagent's current facing to move. 
	 * @param dist The distance to move.
	 * @param speed The speed at which to move. This is a value between 0 and 1.
	 * @param obstacles 1 if the agent should hop over low obstacles, 0 if it should report blocked when it encounters them.
	 * @param priority The command queue priority.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @return MoveBy does not return any data.
	 */
	public MoveBy moveBy(double dir, double dist, double speed, int obstacles, priority_t priority);
	/**
	 * Moves the Quagent the specified distance in the specified direction.
	 * @param dir The rotational angle, in degrees left of Quagent's current facing to move. 
	 * @param dist The distance to move.
	 * @param speed The speed at which to move. This is a value between 0 and 1.
	 * @param obstacles 1 if the agent should hop over low obstacles, 0 if it should report blocked when it encounters them.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @return MoveBy does not return any data.
	 */
	public MoveBy moveBy(double dir, double dist, double speed, int obstacles, priority_t priority, long period);
	

	/**
	 * Moves the Quagent the specified distance in the specified direction.
	 * @param dir The rotational angle, in degrees left of Quagent's current facing to move. 
	 * @param dist The distance to move.
	 * @param speed The speed at which to move. This is a value between 0 and 1.
	 * @param obstacles Determines whether the Quagent automatically attempts to bypass obstacles. 1 is auto bypass, else 0.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveBy command object representing this action.
	 */
	public MoveBy moveBy(double dir, double dist, double speed, int obstacles);
	/**
	 * Moves the Quagent the specified distance in the specified direction.
	 * @param dir The rotational angle, in degrees left of Quagent's current facing to move. 
	 * @param dist The distance to move.
	 * @param speed The speed at which to move. This is a value between 0 and 1.
	 * @param obstacles Determines whether the Quagent automatically attempts to bypass obstacles. 1 is auto bypass, else 0.
	 * @param period The period at which to repeat the action.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveBy command object representing this action.
	 */
	public MoveBy moveBy(double dir, double dist, double speed, int obstacles, long period);
	
	/**
	 * Moves the Quagent the specified distance in the specified direction.
	 * @param dir The rotational angle, in degrees left of Quagent's current facing to move. 
	 * @param dist The distance to move.
	 * @param speed The speed at which to move. This is a value between 0 and 1.
	 * @param priority The command queue priority.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveBy command object representing this action.
	 */
	public MoveBy moveBy(double dir, double dist, double speed, priority_t priority);
	/**
	 * Moves the Quagent the specified distance in the specified direction.
	 * @param dir The rotational angle, in degrees left of Quagent's current facing to move. 
	 * @param dist The distance to move.
	 * @param speed The speed at which to move. This is a value between 0 and 1.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveBy command object representing this action.
	 */
	public MoveBy moveBy(double dir, double dist, double speed, priority_t priority, long period);
	
	/**
	 * Moves the Quagent the specified distance in the specified direction.
	 * @param dir The rotational angle, in degrees left of Quagent's current facing to move. 
	 * @param dist The distance to move.
	 * @param speed The speed at which to move. This is a value between 0 and 1.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveBy command object representing this action.
	 */
	public MoveBy moveBy(double dir, double dist, double speed);
	/**
	 * Moves the Quagent the specified distance in the specified direction.
	 * @param dir The rotational angle, in degrees left of Quagent's current facing to move. 
	 * @param dist The distance to move.
	 * @param speed The speed at which to move. This is a value between 0 and 1.
	 * @param period The period at which to repeat the action.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveBy command object representing this action.
	 */
	public MoveBy moveBy(double dir, double dist, double speed,long period);
	
	/**
	 * Moves the Quagent the specified distance, in the direction of the Quagent's current facing.
	 * @param dist The distance to move.
	 * @param speed The speed at which to move. This is a value between 0 and 1.
	 * @param priority The command queue priority.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveBy command object representing this action.
	 */
	public MoveBy moveBy(double dist, double speed, priority_t priority);
	/**
	 * Moves the Quagent the specified distance, in the direction of the Quagent's current facing.
	 * @param dist The distance to move.
	 * @param speed The speed at which to move. This is a value between 0 and 1.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveBy command object representing this action.
	 */
	public MoveBy moveBy(double dist, double speed, priority_t priority, long period);
	
	/**
	 * Moves the Quagent the specified distance, in the direction of the Quagent's current facing.
	 * @param dist The distance to move.
	 * @param speed The speed at which to move. This is a value between 0 and 1.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveBy command object representing this action.
	 */
	public MoveBy moveBy(double dist, double speed);
	/**
	 * Moves the Quagent the specified distance, in the direction of the Quagent's current facing.
	 * @param dist The distance to move.
	 * @param speed The speed at which to move. This is a value between 0 and 1.
	 * @param period The period at which to repeat the action.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveBy command object representing this action.
	 */
	public MoveBy moveBy(double dist, double speed, long period);
	
	/**
	 * Moves the Quagent the specified distance, in the direction of the Quagent's current facing.
	 * or is blocked by an obstacle.
	 * @param dist The distance to move.
	 * @param priority The command queue priority.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveBy command object representing this action.
	 */
	public MoveBy moveBy(double dist, priority_t priority);

	/**
	 * Moves the Quagent the specified distance, in the direction of the Quagent's current facing.
	 * or is blocked by an obstacle.
	 * @param dist The distance to move.
	 * @param priority The command queue priority
	 * @param period The period at which to repeat the action..
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveBy command object representing this action.
	 */
	public MoveBy moveBy(double dist, priority_t priority, long period);
	
	/**
	 * Moves the Quagent the specified distance, in the direction of the Quagent's current facing.
	 * @param dist The distance to move.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveBy command object representing this action.
	 */
	public MoveBy moveBy(double dist);
	/**
	 * Moves the Quagent the specified distance, in the direction of the Quagent's current facing.
	 * @param dist The distance to move.
	 * @param period The period at which to repeat the action.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveBy command object representing this action.
	 */
	public MoveBy moveBy(double dist, long period);
	
	
	//--------moveTo----------
	/**
	 * Moves the Quagent on a straight line towards the specified point at the specified speed.
	 * @param point The destination point, in the form of {x,y}.
	 * @param speed The speed at which to move. Floating point value between 0 and 1. 
	 * @param obstacles Determines whether the Quagent automatically attempts to bypass obstacles. 1 is auto bypass, else 0.
	 * @param priority The command queue priority.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveTo command object representing this action. 
	 */
	public MoveTo moveTo(double[] point, double speed, int obstacles, priority_t priority);
	/**
	 * Moves the Quagent on a straight line towards the specified point at the specified speed.
	 * @param point The destination point, in the form of {x,y}.
	 * @param speed The speed at which to move. Floating point value between 0 and 1. 
	 * @param obstacles Determines whether the Quagent automatically attempts to bypass obstacles. 1 is auto bypass, else 0.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveTo command object representing this action. 
	 */
	public MoveTo moveTo(double[] point, double speed, int obstacles, priority_t priority, long period);
	
	
	/**
	 * Moves the Quagent on a straight line towards the specified point at the specified speed.
	 * @param point The destination point, in the form of {x,y}.
	 * @param speed The speed at which to move. Floating point value between 0 and 1. 
	 * @param obstacles Determines whether the Quagent automatically attempts to bypass obstacles. 1 is auto bypass, else 0.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveTo command object representing this action. 
	 */
	public MoveTo moveTo(double[] point, double speed, int obstacles);
	/**
	 * Moves the Quagent on a straight line towards the specified point at the specified speed.
	 * @param point The destination point, in the form of {x,y}.
	 * @param speed The speed at which to move. Floating point value between 0 and 1. 
	 * @param obstacles Determines whether the Quagent automatically attempts to bypass obstacles. 1 is auto bypass, else 0.
	 * @param period The period at which to repeat the action.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveTo command object representing this action. 
	 */
	public MoveTo moveTo(double[] point, double speed, int obstacles, long period);
	
	/**
	 * Moves the Quagent on a straight line towards the specified point at the specified speed.
	 * @param point The destination point, in the form of {x,y}.
	 * @param speed The speed at which to move. Floating point value between 0 and 1. 
	 * @param priority The command queue priority.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveTo command object representing this action. 
	 */
	public MoveTo moveTo(double[] point, double speed, priority_t priority);
	/**
	 * Moves the Quagent on a straight line towards the specified point at the specified speed.
	 * @param point The destination point, in the form of {x,y}.
	 * @param speed The speed at which to move. Floating point value between 0 and 1. 
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveTo command object representing this action. 
	 */
	public MoveTo moveTo(double[] point, double speed, priority_t priority, long period);
	
	/**
	 * Moves the Quagent on a straight line towards the specified point at the specified speed.
	 * @param point The destination point, in the form of {x,y}.
	 * @param speed The speed at which to move. Floating point value between 0 and 1. 
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveTo command object representing this action. 
	 */
	public MoveTo moveTo(double[] point, double speed);
	/**
	 * Moves the Quagent on a straight line towards the specified point at the specified speed.
	 * @param point The destination point, in the form of {x,y}.
	 * @param speed The speed at which to move. Floating point value between 0 and 1. 
	 * @param period The period at which to repeat the action.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveTo command object representing this action. 
	 */
	public MoveTo moveTo(double[] point, double speed, long period);
	
	/**
	 * Moves the Quagent on a straight line towards the specified point at DEFAULTSPEED.
	 * @param point The destination point, in the form of {x,y}.
	 * @param priority The command queue priority.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveTo command object representing this action. 
	 */
	public MoveTo moveTo(double[] point, priority_t priority);
	/**
	 * Moves the Quagent on a straight line towards the specified point at DEFAULTSPEED.
	 * @param point The destination point, in the form of {x,y}.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveTo command object representing this action. 
	 */
	public MoveTo moveTo(double[] point, priority_t priority, long period);
	
	/**
	 * Moves the Quagent on a straight line towards the specified point at DEFAULTSPEED.
	 * @param point The destination point, in the form of {x,y}.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have associated response data.
	 * @return MoveTo command object representing this action. 
	 */
	public MoveTo moveTo(double[] point);
	/**
	 * Moves the Quagent on a straight line towards the specified point at DEFAULTSPEED.
	 * @param point The destination point, in the form of {x,y}.
	 * @param period The period at which to repeat the action.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have associated response data.
	 * @return MoveTo command object representing this action. 
	 */
	public MoveTo moveTo(double[] point, long period);
	
	
	//--------moveForever----------
	/**
	 * Moves a Quagent indefinitely in the specified direction at the specified speed.
	 * @param dir The rotational angle, in degrees, left of Quagent's current facing to move.
	 * @param speed The speed at which to move. 
	 * @param obstacles Determines whether the Quagent automatically attempts to bypass obstacles. 1 is auto bypass, else 0.
	 * @param priority The command queue priority.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveForever command object representing this action. 
	 */
	public MoveForever moveForever(double dir, double speed, int obstacles, priority_t priority);
	/**
	 * Moves a Quagent indefinitely in the specified direction at the specified speed.
	 * @param dir The rotational angle, in degrees, left of Quagent's current facing to move.
	 * @param speed The speed at which to move. 
	 * @param obstacles Determines whether the Quagent automatically attempts to bypass obstacles. 1 is auto bypass, else 0.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveForever command object representing this action. 
	 */
	public MoveForever moveForever(double dir, double speed, int obstacles, priority_t priority, long period);
	
	/**
	 * Moves a Quagent indefinitely in the specified direction at the specified speed.
	 * @param dir The rotational angle, in degrees, left of Quagent's current facing to move.
	 * @param speed The speed at which to move. 
	 * @param obstacles Determines whether the Quagent automatically attempts to bypass obstacles. 1 is auto bypass, else 0.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveForever command object representing this action. 
	 */
	public MoveForever moveForever(double dir, double speed, int obstacles);
	/**
	 * Moves a Quagent indefinitely in the specified direction at the specified speed.
	 * @param dir The rotational angle, in degrees, left of Quagent's current facing to move.
	 * @param speed The speed at which to move. 
	 * @param obstacles Determines whether the Quagent automatically attempts to bypass obstacles. 1 is auto bypass, else 0.
	 * @param period The period at which to repeat the action.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveForever command object representing this action. 
	 */
	public MoveForever moveForever(double dir, double speed, int obstacles, long period);
	
	/**
	 * Moves a Quagent indefinitely in the specified direction at the specified speed.
	 * @param dir The rotational angle, in degrees, left of Quagent's current facing to move.
	 * @param speed The speed at which to move. 
	 * @param priority The command queue priority.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveForever command object representing this action. 
	 */
	public MoveForever moveForever(double dir, double speed, priority_t priority);
	/**
	 * Moves a Quagent indefinitely in the specified direction at the specified speed.
	 * @param dir The rotational angle, in degrees, left of Quagent's current facing to move.
	 * @param speed The speed at which to move. 
	 * @param priority The command queue priority
	 * @param period The period at which to repeat the action..
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveForever command object representing this action. 
	 */
	public MoveForever moveForever(double dir, double speed, priority_t priority, long period);
	
	/**
	 * Moves a Quagent indefinitely in the specified direction at the specified speed.
	 * @param dir The rotational angle, in degrees, left of Quagent's current facing to move.
	 * @param speed The speed at which to move. 
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveForever command object representing this action. 
	 */
	public MoveForever moveForever(double dir, double speed);
	/**
	 * Moves a Quagent indefinitely in the specified direction at the specified speed.
	 * @param dir The rotational angle, in degrees, left of Quagent's current facing to move.
	 * @param speed The speed at which to move.
	 * @param period The period at which to repeat the action. 
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveForever command object representing this action. 
	 */
	public MoveForever moveForever(double dir, double speed, long period);
	
	/**
	 * Moves a Quagent indefinitely in the direction of the Quagent's current facing at the specified speed. 
	 * @param speed The speed at which to move. 
	 * @param priority The command queue priority
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveForever command object representing this action. 
	 */
	public MoveForever moveForever(double speed, priority_t priority);
	/**
	 * Moves a Quagent indefinitely in the direction of the Quagent's current facing at the specified speed. 
	 * @param speed The speed at which to move. 
	 * @param priority The command queue priority
	 * @param period The period at which to repeat the action.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveForever command object representing this action. 
	 */
	public MoveForever moveForever(double speed, priority_t priority, long period);
	
	/**
	 * Moves a Quagent indefinitely in the direction of the Quagent's current facing at the specified speed.
	 * @param speed The speed at which to move. Floating point between 0 and 1.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveForever command object representing this action. 
	 */
	public MoveForever moveForever(double speed);
	/**
	 * Moves a Quagent indefinitely in the direction of the Quagent's current facing at the specified speed.
	 * @param speed The speed at which to move. Floating point between 0 and 1.
	 * @param period The period at which to repeat the action.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveForever command object representing this action. 
	 */
	public MoveForever moveForever(double speed, long period);
	
	/**
	 * Moves a Quagent indefinitely in the direction of the Quagent's current facing at DEFAULTSPEED.
	 * @param priority The command queue priority.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveForever command object representing this action. 
	 */
	public MoveForever moveForever(priority_t priority);
	/**
	 * Moves a Quagent indefinitely in the direction of the Quagent's current facing at DEFAULTSPEED.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveForever command object representing this action. 
	 */
	public MoveForever moveForever(priority_t priority, long period);

	/**
	 * Moves a Quagent indefinitely in the direction of the Quagent's current facing at DEFAULTSPEED.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveForever command object representing this action. 
	 */
	public MoveForever moveForever();
	/**
	 * Moves a Quagent indefinitely in the direction of the Quagent's current facing at DEFAULTSPEED.
	 * @param period The period at which to repeat the action.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have associated response data.
	 * @return MoveForever command object representing this action. 
	 */
	public MoveForever moveForever(long period);
	

	//----------MoveFor------------
	/**
	 * Moves the Quagent in the specified direction for the specified duration.
	 * @param dir The rotational angle, in degrees, left of Quagent's current facing to move.
	 * @param time The duration of the movement.
	 * @param speed The speed at which to move. Floating point value between 0 and 1.
	 * @param obstacles Determines whether the Quagent automatically attempts to bypass obstacles. 1 is auto bypass, else 0.
	 * @param priority The command queue priority.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have associated response data.
	 * @return MoveFor command object representing this action.
	 */
	public MoveFor moveFor(double dir, double time, double speed, int obstacles, priority_t priority);
	/**
	 * Moves the Quagent in the specified direction for the specified duration.
	 * @param dir The rotational angle, in degrees, left of Quagent's current facing to move.
	 * @param time The duration of the movement.
	 * @param speed The speed at which to move. Floating point value between 0 and 1.
	 * @param obstacles Determines whether the Quagent automatically attempts to bypass obstacles. 1 is auto bypass, else 0.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have associated response data.
	 * @return MoveFor command object representing this action.
	 */
	public MoveFor moveFor(double dir, double time, double speed, int obstacles, priority_t priority,long period);
	
	/**
	 * Moves the Quagent in the specified direction for the specified duration.
	 * @param dir The rotational angle, in degrees, left of Quagent's current facing to move.
	 * @param time The duration of the movement.
	 * @param speed The speed at which to move. Floating point value between 0 and 1.
	 * @param obstacles Determines whether the Quagent automatically attempts to bypass obstacles. 1 is auto bypass, else 0.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have associated response data.
	 * @return MoveFor command object representing this action.
	 */
	public MoveFor moveFor(double dir, double time, double speed, int obstacles);
	/**
	 * Moves the Quagent in the specified direction for the specified duration.
	 * @param dir The rotational angle, in degrees, left of Quagent's current facing to move.
	 * @param time The duration of the movement.
	 * @param speed The speed at which to move. Floating point value between 0 and 1.
	 * @param obstacles Determines whether the Quagent automatically attempts to bypass obstacles. 1 is auto bypass, else 0.
	 * @param period The period at which to repeat the action.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have associated response data.
	 * @return MoveFor command object representing this action.
	 */
	public MoveFor moveFor(double dir, double time, double speed, int obstacles, long period);
	
	/**
	 * Moves the Quagent in the specified direction for the specified duration.
	 * @param dir The rotational angle, in degrees, left of Quagent's current facing to move.
	 * @param time The duration of the movement.
	 * @param speed The speed at which to move. Floating point value between 0 and 1.
	 * @param priority The command queue priority.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have associated response data.
	 * @return MoveFor command object representing this action.
	 */
	public MoveFor moveFor(double dir, double time, double speed, priority_t priority);
	/**
	 * Moves the Quagent in the specified direction for the specified duration.
	 * @param dir The rotational angle, in degrees, left of Quagent's current facing to move.
	 * @param time The duration of the movement.
	 * @param speed The speed at which to move. Floating point value between 0 and 1.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have associated response data.
	 * @return MoveFor command object representing this action.
	 */
	public MoveFor moveFor(double dir, double time, double speed, priority_t priority, long period);
	
	/**
	 * Moves the Quagent in the specified direction for the specified duration.
	 * @param dir The rotational angle, in degrees, left of Quagent's current facing to move.
	 * @param time The duration of the movement.
	 * @param speed The speed at which to move. Floating point value between 0 and 1.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have associated response data.
	 * @return MoveFor command object representing this action.
	 */
	public MoveFor moveFor(double dir, double time, double speed);
	
	/**
	 * Moves the Quagent in the specified direction for the specified duration.
	 * @param time The duration of the movement.
	 * @param speed The speed at which to move. Floating point value between 0 and 1.
	 * @param priority The command queue priority.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have associated response data.
	 * @return MoveFor command object representing this action.
	 */
	public MoveFor moveFor(double time, double speed, priority_t priority);
	/**
	 * Moves the Quagent in the specified direction for the specified duration.
	 * @param time The duration of the movement.
	 * @param speed The speed at which to move. Floating point value between 0 and 1.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have associated response data.
	 * @return MoveFor command object representing this action.
	 */
	public MoveFor moveFor(double time, double speed, priority_t priority, long period);
	
	/**
	 * Moves the Quagent in the specified direction for the specified duration.
	 * @param time The duration of the movement.
	 * @param speed The speed at which to move. Floating point value between 0 and 1.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have associated response data.
	 * @return MoveFor command object representing this action.
	 */
	public MoveFor moveFor(double time, double speed);
	/**
	 * Moves the Quagent in the specified direction for the specified duration.
	 * @param time The duration of the movement.
	 * @param speed The speed at which to move. Floating point value between 0 and 1.
	 * @param period The period at which to repeat the action.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have associated response data.
	 * @return MoveFor command object representing this action.
	 */
	public MoveFor moveFor(double time, double speed, long period);
	
	/**
	 * Moves the Quagent in the specified direction for the specified duration.
	 * @param time The duration of the movement.
	 * @param priority The command queue priority.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have associated response data.
	 * @return MoveFor command object representing this action.
	 */
	public MoveFor moveFor(double time, priority_t priority);
	/**
	 * Moves the Quagent in the specified direction for the specified duration.
	 * @param time The duration of the movement.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have associated response data.
	 * @return MoveFor command object representing this action.
	 */
	public MoveFor moveFor(double time, priority_t priority, long period);
	
	/**
	 * Moves the Quagent in the specified direction for the specified duration.
	 * @param time The duration of the movement.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have associated response data.
	 * @return MoveFor command object representing this action.
	 */
	public MoveFor moveFor(double time);
	/**
	 * Moves the Quagent in the specified direction for the specified duration.
	 * @param time The duration of the movement.
	 * @param period The period at which to repeat the action.
	 * @reports BLOCKED,POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have associated response data.
	 * @return MoveFor command object representing this action.
	 */
	public MoveFor moveFor(double time, long period);
	
	
	//--------jump-------------
	/**
	 * Makes the Quagent perform a single jump. Movement during this jump will occur in the specified direction.
	 * Users may notice that the bot does not jump very far in a standalone jump: longer jumps are possible but require a running start.
	 * @param dir The rotational angle, in degrees, left of Quagent's current facing to move.
	 * @param speed The speed at which to jump. Floating point value between 0 and 1.
	 * @param priority The command queue priority. 
	 * @reports POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have associated response data.
	 * @return Jump command object representing this action.
	 */
	public Jump jump(double dir, double speed, priority_t priority);
	/**
	 * Makes the Quagent perform a single jump. Movement during this jump will occur in the specified direction.
	 * Users may notice that the bot does not jump very far in a standalone jump: longer jumps are possible but require a running start.
	 * @param dir The rotational angle, in degrees, left of Quagent's current facing to move.
	 * @param speed The speed at which to jump. Floating point value between 0 and 1.
	 * @param priority The command queue priority. 
	 * @param period The period at which to repeat the action.
	 * @reports POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have associated response data.
	 * @return Jump command object representing this action.
	 */
	public Jump jump(double dir, double speed, priority_t priority, long period);
	
	/**
	 * Makes the Quagent perform a single jump. Movement during this jump will occur in the specified direction.
	 * Users may notice that the bot does not jump very far in a standalone jump: longer jumps are possible but require a running start.
	 * @param dir The rotational angle, in degrees, left of Quagent's current facing to move.
	 * @param speed The speed at which to jump. Floating point value between 0 and 1.
	 * @reports POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have associated response data.
	 * @return Jump command object representing this action.
	 */
	public Jump jump(double dir, double speed);
	/**
	 * Makes the Quagent perform a single jump. Movement during this jump will occur in the specified direction.
	 * Users may notice that the bot does not jump very far in a standalone jump: longer jumps are possible but require a running start.
	 * @param dir The rotational angle, in degrees, left of Quagent's current facing to move.
	 * @param speed The speed at which to jump. Floating point value between 0 and 1.
	 * @param period The period at which to repeat the action.
	 * @reports POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have associated response data.
	 * @return Jump command object representing this action.
	 */
	public Jump jump(double dir, double speed, long period);
	
	/**
	 * Makes the Quagent perform a single jump. Movement during this jump will occur in the direction of the Quagent's current facing.
	 * Users may notice that the bot does not jump very far in a standalone jump: longer jumps are possible but require a running start.
	 * @param priority The command queue priority. 	 
	 * @reports POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have associated response data.
	 * @return Jump command object representing this action.
	 */
	public Jump jump(priority_t priority);
	/**
	 * Makes the Quagent perform a single jump. Movement during this jump will occur in the direction of the Quagent's current facing.
	 * Users may notice that the bot does not jump very far in a standalone jump: longer jumps are possible but require a running start.
	 * @param priority The command queue priority. 	 
	 * @param period The period at which to repeat the action.
	 * @reports POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have associated response data.
	 * @return Jump command object representing this action.
	 */
	public Jump jump(priority_t priority, long period);
	
	/**
	 * Makes the Quagent perform a single jump. Movement during this jump will occur in the direction of the Quagent's current facing.
	 * Users may notice that the bot does not jump very far in a standalone jump: longer jumps are possible but require a running start.
	 * @reports POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have associated response data.
	 * @return Jump command object representing this action.
	 */
	public Jump jump();
	/**
	 * Makes the Quagent perform a single jump. Movement during this jump will occur in the direction of the Quagent's current facing.
	 * Users may notice that the bot does not jump very far in a standalone jump: longer jumps are possible but require a running start.
	 * @param period The period at which to repeat the action.
	 * @reports POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have associated response data.
	 * @return Jump command object representing this action.
	 */
	public Jump jump(long period);
	
	
	//------crouch------
	/**
	 * Sets the Quagent's crouch state. When crouching is enabled the Quagent will crouch during all actions.
	 * @param tocrouch True to enable crouching, false to break crouching.
	 * @param priority The command queue priority.
	 * @reports POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have associated response data.
	 * @return Crouch command object representing this action
	 */
	public Crouch crouch(boolean tocrouch, priority_t priority);
	/**
	 * Sets the Quagent's crouch state. When crouching is enabled the Quagent will crouch during all actions.
	 * @param tocrouch True to enable crouching, false to break crouching.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have associated response data.
	 * @return Crouch command object representing this action
	 */
	public Crouch crouch(boolean tocrouch, priority_t priority, long period);
	
	/**
	 * Sets the Quagent's crouch state. When crouching is enabled the Quagent will crouch during all actions.
	 * @param tocrouch True to enable crouching, false to break crouching.
	 * @reports POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have associated response data.
	 * @return Crouch command object representing this action
	 */
	public Crouch crouch(boolean tocrouch);
	/**
	 * Sets the Quagent's crouch state. When crouching is enabled the Quagent will crouch during all actions.
	 * @param tocrouch True to enable crouching, false to break crouching.
	 * @param period The period at which to repeat the action.
	 * @reports POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have associated response data.
	 * @return Crouch command object representing this action
	 */
	public Crouch crouch(boolean tocrouch, long period);
	
	
	//------fireWeapon--------
	/**
	 * This command tells the agent to briefly fire its currently equipped weapon. 
	 * It fires the weapon exactly in the direction it is looking
	 * @param priority The command queue priority.
	 * @reports POPPED,FORGOTTEN,REPLACED,DONE,EMPTY
	 * @data Does not have associated response data.
	 * @return FireWeapon command object representing this action.
	 */
	public FireWeapon fireWeapon(priority_t priority);
	/**
	 * This command tells the agent to briefly fire its currently equipped weapon. 
	 * It fires the weapon exactly in the direction it is looking
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports POPPED,FORGOTTEN,REPLACED,DONE,EMPTY
	 * @data Does not have associated response data.
	 * @return FireWeapon command object representing this action.
	 */
	public FireWeapon fireWeapon(priority_t priority,long period);
	
	/**
	 * This command tells the agent to briefly fire its currently equipped weapon. 
	 * It fires the weapon exactly in the direction it is looking
	 * @reports POPPED,FORGOTTEN,REPLACED,DONE,EMPTY
	 * @data Does not have associated response data.
	 * @return FireWeapon command object representing this action.
	 */
	public FireWeapon fireWeapon();
	/**
	 * This command tells the agent to briefly fire its currently equipped weapon. 
	 * It fires the weapon exactly in the direction it is looking
	 * @param period The period at which to repeat the action.
	 * @reports POPPED,FORGOTTEN,REPLACED,DONE,EMPTY
	 * @data Does not have associated response data.
	 * @return FireWeapon command object representing this action.
	 */
	public FireWeapon fireWeapon(long period);
	
	//-----pickUp-----------
	/**
	 * Quagent picks up any items in its bounding box, and inserts them into its inventory.
	 * @param priority The command queue priority.
	 * @reports
	 * @data Boolean for picked something up, quantity picked up, and name of entity picked up.
	 */
	//public PickUp pickUp(priority_t priority);
	
	//-----switchWeapon---------
	/**
	 * Makes the agent to switch weapons. Switches to the specified weapon, if possible.
	 * For more information, see: enums interface -> weapon enumerated type.
	 * @param wep The weapon to be switched to.
	 * @param priority The command queue priority.
	 * @reports POPPED,FORGOTTEN,REPLACED,DONE,INVALID_WEAPON
	 * @data The ammunition of the weapon being switched to. Only responds with this data
	 * if the weapon is valid.
	 * @return SwitchWeapon command object representing this action.
	 */
	public SwitchWeapon switchWeapon(weapon wep, priority_t priority);
	/**
	 * Makes the agent to switch weapons. Switches to the specified weapon, if possible.
	 * For more information, see: enums interface -> weapon enumerated type.
	 * @param wep The weapon to be switched to.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports POPPED,FORGOTTEN,REPLACED,DONE,INVALID_WEAPON
	 * @data The ammunition of the weapon being switched to. Only responds with this data
	 * if the weapon is valid.
	 * @return SwitchWeapon command object representing this action.
	 */
	public SwitchWeapon switchWeapon(weapon wep, priority_t priority, long period);
	
	/**
	 * Makes the agent to switch weapons. Switches to the specified weapon, if possible.
	 * For more information, see: enums interface -> weapon enumerated type.
	 * @param wep The weapon to be switched to.
	 * @reports POPPED,FORGOTTEN,REPLACED,DONE,INVALID_WEAPON
	 * @data The ammunition of the weapon being switched to. Only responds with this data
	 * if the weapon is valid.
	 * @return SwitchWeapon command object representing this action.
	 */
	public SwitchWeapon switchWeapon(weapon wep);
	/**
	 * Makes the agent to switch weapons. Switches to the specified weapon, if possible.
	 * For more information, see: enums interface -> weapon enumerated type.
	 * @param wep The weapon to be switched to.
	 * @param period The period at which to repeat the action.
	 * @reports POPPED,FORGOTTEN,REPLACED,DONE,INVALID_WEAPON
	 * @data The ammunition of the weapon being switched to. Only responds with this data
	 * if the weapon is valid.
	 * @return SwitchWeapon command object representing this action.
	 */
	public SwitchWeapon switchWeapon(weapon wep, long period);

	
	//------rotate------
	/**
	 * Rotates the agent to the left by the specified rotational and azimuthal angles.
	 * @param rot The rotational angle to rotate by.
	 * @param azi The azimuthal angle to rotate by.
	 * @param priority The command queue priority.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have an associated data response.
	 * @return Rotate command object representing this action.
	 */
	public Rotate rotate(double rot, double azi, priority_t priority);
	/**
	 * Rotates the agent to the left by the specified rotational and azimuthal angles.
	 * @param rot The rotational angle to rotate by.
	 * @param azi The azimuthal angle to rotate by.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have an associated data response.
	 * @return Rotate command object representing this action.
	 */
	public Rotate rotate(double rot, double azi, priority_t priority, long period);
	
	/**
	 * Rotates the agent to the left by the specified rotational and azimuthal angles.
	 * @param rot The rotational angle to rotate by.
	 * @param azi The azimuthal angle to rotate by.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have an associated data response.
	 * @return Rotate command object representing this action.
	 */
	public Rotate rotate(double rot, double azi);
	/**
	 * Rotates the agent to the left by the specified rotational and azimuthal angles.
	 * @param rot The rotational angle to rotate by.
	 * @param azi The azimuthal angle to rotate by.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have an associated data response.
	 * @return Rotate command object representing this action.
	 */
	public Rotate rotate(double rot, double azi, long period);
	
	/**
	 * Rotates the agent to the left by the specified rotational and azimuthal angles.
	 * @param rot The rotational angle to rotate by.
	 * @param priority The command queue priority.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have an associated data response.
	 * @return Rotate command object representing this action.
	 */
	public Rotate rotate(double rot, priority_t priority);
	/**
	 * Rotates the agent to the left by the specified rotational and azimuthal angles.
	 * @param rot The rotational angle to rotate by.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have an associated data response.
	 * @return Rotate command object representing this action.
	 */
	public Rotate rotate(double rot, priority_t priority, long period);
	
	/**
	 * Rotates the agent to the left by the specified rotational and azimuthal angles.
	 * @param rot The rotational angle to rotate by.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have an associated data response.
	 * @return Rotate command object representing this action.
	 */
	public Rotate rotate(double rot);
	/**
	 * Rotates the agent to the left by the specified rotational and azimuthal angles.
	 * @param rot The rotational angle to rotate by.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data Does not have an associated data response.
	 * @return Rotate command object representing this action.
	 */
	public Rotate rotate(double rot, long period);
	
	
	//-----------say---------------
	/**
	 * Quagent says the parameter message aloud. All other quagents within range (determined by <i>volume</i>)
	 * will hear the message as an event. 
	 * @param msg Message to be spoken.
	 * @param volume Range of message.
	 * @param priority The command queue priority
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @events The message, including source entity id and message content.
	 * @return Say command object which represents this command.
	 */
	public Say say(String msg, double volume, priority_t priority);
	
	/**
	 * Quagent says the parameter message aloud. All other quagents within range (determined by <i>volume</i>)
	 * will hear the message as an event. 
	 * @param msg Message to be spoken.
	 * @param volume Range of message.
	 * @param priority The command queue priority
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @events The message, including source entity id and message content.
	 * @return Say command object which represents this command.
	 */
	public Say say(String msg, double volume, priority_t priority, long period);
	
	/**
	 * Quagent says the parameter message aloud. All other quagents within range (determined by <i>volume</i>)
	 * will hear the message as an event. 
	 * @param msg Message to be spoken.
	 * @param volume Range of message.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @events The message, including source entity id and message content.
	 * @return Say command object which represents this command.
	 */
	public Say say(String msg, double volume);
	
	/**
	 * Quagent says the parameter message aloud. All other quagents within range (determined by <i>volume</i>)
	 * will hear the message as an event. 
	 * @param msg Message to be spoken.
	 * @param volume Range of message.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @events The message, including source entity id and message content.
	 * @return Say command object which represents this command.
	 */
	public Say say(String msg, double volume, long period);
	
	/**
	 * Quagent says the parameter message aloud. All other quagents within range (determined by Client.DEFAULTVOLUME)
	 * will hear the message as an event. 
	 * @param msg Message to be spoken.
	 * @param priority The command queue priority
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @events The message, including source entity id and message content.
	 * @return Say command object which represents this command.
	 */
	public Say say(String msg, priority_t priority);
	
	/**
	 * Quagent says the parameter message aloud. All other quagents within range (determined by Client.DEFAULTVOLUME)
	 * will hear the message as an event. 
	 * @param msg Message to be spoken.
	 * @param priority The command queue priority
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @events The message, including source entity id and message content.
	 * @return Say command object which represents this command.
	 */
	public Say say(String msg, priority_t priority, long period);
	
	/**
	 * Quagent says the parameter message aloud. All other quagents within range (determined by Client.DEFAULTVOLUME)
	 * will hear the message as an event. 
	 * @param msg Message to be spoken.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @events The message, including source entity id and message content.
	 * @return Say command object which represents this command.
	 */
	public Say say(String msg);
	
	/**
	 * Quagent says the parameter message aloud. All other quagents within range (determined by Client.DEFAULTVOLUME)
	 * will hear the message as an event. 
	 * @param msg Message to be spoken.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @events The message, including source entity id and message content.
	 * @return Say command object which represents this command.
	 */
	public Say say(String msg, long period);
	
	
	//------------shove---------------
	/**
	 * Shoves the object with <i> strength </i> in the direction of <i> yaw </i> and <i> pitch </i>.
	 * @param strength The power of the shove. <br/> A value of 400 will move an object as fast as a quagent can run.
	 * A value of 500 is sufficient for most tasks. A value of 5000 will generally launch the object from the level.
	 * @param yaw The horizontal direction to launch to object. <br/>
	 * Measured in degrees. Angle is to the left of the Quagent's current facing.
	 * @param pitch The vertical direction to launch to object. <br/>
	 * Measured in degrees. Angle is down from the Quagent's current facing.
	 * @param priority The command queue priority.
	 * @reports DONE, INVALID_TARGET, NOTHING_THERE, POPPED, FORGOTTEN, REPLACED
	 * @data Does not have an associated data response.
	 * @return Shove command object which represents this command.
	 */
	public Shove shove(double strength, double yaw, double pitch, priority_t priority);
	
	/**
	 * Shoves the object with <i> strength </i> in the direction of <i> yaw </i> and <i> pitch </i>.
	 * @param strength The power of the shove. <br/> A value of 400 will move an object as fast as a quagent can run.
	 * A value of 500 is sufficient for most tasks. A value of 5000 will generally launch the object from the level.
	 * @param yaw The horizontal direction to launch to object. <br/>
	 * Measured in degrees. Angle is to the left of the Quagent's current facing.
	 * @param pitch The vertical direction to launch to object. <br/>
	 * Measured in degrees. Angle is down from the Quagent's current facing.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports DONE, INVALID_TARGET, NOTHING_THERE, POPPED, FORGOTTEN, REPLACED
	 * @data Does not have an associated data response.
	 * @return Shove command object which represents this command.
	 */
	public Shove shove(double strength, double yaw, double pitch, priority_t priority, long period);
	
	/**
	 * Shoves the object with <i> strength </i> in the direction of <i> yaw </i> and <i> pitch </i>.
	 * @param strength The power of the shove. <br/> A value of 400 will move an object as fast as a quagent can run.
	 * A value of 500 is sufficient for most tasks. A value of 5000 will generally launch the object from the level.
	 * @param yaw The horizontal direction to launch to object. <br/>
	 * Measured in degrees. Angle is to the left of the Quagent's current facing.
	 * @param pitch The vertical direction to launch to object. <br/>
	 * Measured in degrees. Angle is down from the Quagent's current facing.
	 * @reports DONE, INVALID_TARGET, NOTHING_THERE, POPPED, FORGOTTEN, REPLACED
	 * @data Does not have an associated data response.
	 * @return Shove command object which represents this command.
	 */
	public Shove shove(double strength, double yaw, double pitch);
	
	/**
	 * Shoves the object with <i> strength </i> in the direction of <i> yaw </i> and <i> pitch </i>.
	 * @param strength The power of the shove. <br/> A value of 400 will move an object as fast as a quagent can run.
	 * A value of 500 is sufficient for most tasks. A value of 5000 will generally launch the object from the level.
	 * @param yaw The horizontal direction to launch to object. <br/>
	 * Measured in degrees. Angle is to the left of the Quagent's current facing.
	 * @param pitch The vertical direction to launch to object. <br/>
	 * Measured in degrees. Angle is down from the Quagent's current facing.
	 * @param period The period at which to repeat the action.
	 * @reports DONE, INVALID_TARGET, NOTHING_THERE, POPPED, FORGOTTEN, REPLACED
	 * @data Does not have an associated data response.
	 * @return Shove command object which represents this command.
	 */
	public Shove shove(double strength, double yaw, double pitch, long period);
	
	/**
	 * Shoves the object with <i> strength </i> in the direction of <i> yaw </i> and the Quagent's current vertical facing.
	 * @param strength The power of the shove. <br/> A value of 400 will move an object as fast as a quagent can run.
	 * A value of 500 is sufficient for most tasks. A value of 5000 will generally launch the object from the level.
	 * @param yaw The horizontal direction to launch to object. <br/>
	 * Measured in degrees. Angle is to the left of the Quagent's current facing.
	 * @param priority The command queue priority.
	 * @reports DONE, INVALID_TARGET, NOTHING_THERE, POPPED, FORGOTTEN, REPLACED
	 * @data Does not have an associated data response.
	 * @return Shove command object which represents this command.
	 */
	public Shove shove(double strength, double yaw, priority_t priority);
	
	/**
	 * Shoves the object with <i> strength </i> in the direction of <i> yaw </i> and the Quagent's current vertical facing.
	 * @param strength The power of the shove. <br/> A value of 400 will move an object as fast as a quagent can run.
	 * A value of 500 is sufficient for most tasks. A value of 5000 will generally launch the object from the level.
	 * @param yaw The horizontal direction to launch to object. <br/>
	 * Measured in degrees. Angle is to the left of the Quagent's current facing.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports DONE, INVALID_TARGET, NOTHING_THERE, POPPED, FORGOTTEN, REPLACED
	 * @data Does not have an associated data response.
	 * @return Shove command object which represents this command.
	 */
	public Shove shove(double strength, double yaw, priority_t priority, long period);
	
	/**
	 * Shoves the object with <i> strength </i> in the direction of <i> yaw </i> and the Quagent's current vertical facing.
	 * @param strength The power of the shove. <br/> A value of 400 will move an object as fast as a quagent can run.
	 * A value of 500 is sufficient for most tasks. A value of 5000 will generally launch the object from the level.
	 * @param yaw The horizontal direction to launch to object. <br/>
	 * Measured in degrees. Angle is to the left of the Quagent's current facing.
	 * @reports DONE, INVALID_TARGET, NOTHING_THERE, POPPED, FORGOTTEN, REPLACED
	 * @data Does not have an associated data response.
	 * @return Shove command object which represents this command.
	 */
	public Shove shove(double strength, double yaw);
	
	/**
	 * Shoves the object with <i> strength </i> in the direction of <i> yaw </i> and the Quagent's current vertical facing.
	 * @param strength The power of the shove. <br/> A value of 400 will move an object as fast as a quagent can run.
	 * A value of 500 is sufficient for most tasks. A value of 5000 will generally launch the object from the level.
	 * @param yaw The horizontal direction to launch to object. <br/>
	 * Measured in degrees. Angle is to the left of the Quagent's current facing.
	 * @param period The period at which to repeat the action.
	 * @reports DONE, INVALID_TARGET, NOTHING_THERE, POPPED, FORGOTTEN, REPLACED
	 * @data Does not have an associated data response.
	 * @return Shove command object which represents this command.
	 */
	public Shove shove(double strength, double yaw, long period);
	
	/**
	 * Shoves the object with <i> strength </i> in the direction of the Quagent's current facing.
	 * @param strength The power of the shove. <br/> A value of 400 will move an object as fast as a quagent can run.
	 * A value of 500 is sufficient for most tasks. A value of 5000 will generally launch the object from the level.
	 * @param priority The command queue priority.
	 * @reports DONE, INVALID_TARGET, NOTHING_THERE, POPPED, FORGOTTEN, REPLACED
	 * @data Does not have an associated data response.
	 * @return Shove command object which represents this command.
	 */
	public Shove shove(double strength, priority_t priority);
	
	/**
	 * Shoves the object with <i> strength </i> in the direction of the Quagent's current facing.
	 * @param strength The power of the shove. <br/> A value of 400 will move an object as fast as a quagent can run.
	 * A value of 500 is sufficient for most tasks. A value of 5000 will generally launch the object from the level.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports DONE, INVALID_TARGET, NOTHING_THERE, POPPED, FORGOTTEN, REPLACED
	 * @data Does not have an associated data response.
	 * @return Shove command object which represents this command.
	 */
	public Shove shove(double strength, priority_t priority, long period);
	
	/**
	 * Shoves the object with <i> strength </i> in the direction of the Quagent's current facing.
	 * @param strength The power of the shove. <br/> A value of 400 will move an object as fast as a quagent can run.
	 * A value of 500 is sufficient for most tasks. A value of 5000 will generally launch the object from the level.
	 * @reports DONE, INVALID_TARGET, NOTHING_THERE, POPPED, FORGOTTEN, REPLACED
	 * @data Does not have an associated data response.
	 * @return Shove command object which represents this command.
	 */
	public Shove shove(double strength);
	
	/**
	 * Shoves the object with <i> strength </i> in the direction of the Quagent's current facing.
	 * @param strength The power of the shove. <br/> A value of 400 will move an object as fast as a quagent can run.
	 * A value of 500 is sufficient for most tasks. A value of 5000 will generally launch the object from the level.
	 * @param period The period at which to repeat the action.
	 * @reports DONE, INVALID_TARGET, NOTHING_THERE, POPPED, FORGOTTEN, REPLACED
	 * @data Does not have an associated data response.
	 * @return Shove command object which represents this command.
	 */
	public Shove shove(double strength, long period);
	
	
	
	/* -------------------------------------------------------------------- *
	 * --------------- Command Queue Management (CQM) --------------------- *
	 * -------------------------------------------------------------------- */
	
	//---------forgetAllTasks-----------
	/**
	 * Removes all actions from the server command queue.
	 * @param priority The command queue priority.
	 * @reports DONE,POPPED,FORGOTTED,REPLACED
	 * @data Does not have an associated data response.
	 * @return ForgetAllTasks command object that represents this CQM.
	 */
	public ForgetAllTasks forgetAllTasks(priority_t priority);
	/**
	 * Removes all actions from the server command queue.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTED,REPLACED
	 * @data Does not have an associated data response.
	 * @return ForgetAllTasks command object that represents this CQM.
	 */
	public ForgetAllTasks forgetAllTasks(priority_t priority, long period);

	/**
	 * Removes all actions from the server command queue.
	 * @reports DONE,POPPED,FORGOTTED,REPLACED
	 * @data Does not have an associated data response.
	 * @return ForgetAllTasks command object that represents this CQM.
	 */
	public ForgetAllTasks forgetAllTasks();
	/**
	 * Removes all actions from the server command queue.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTED,REPLACED
	 * @data Does not have an associated data response.
	 * @return ForgetAllTasks command object that represents this CQM.
	 */
	public ForgetAllTasks forgetAllTasks(long period);
	

	//---------pause-------------
	/**
	 * Pauses the execution of actions. To terminate this state, call resume() or pop().
	 * @param priority The command queue priority.
	 * @reports POPPED,FORGOTTEN,REPLACED
	 * @data Does not have an associated data response.
	 * @return Pause command object that represents this CQM.
	 */
	public Pause pause(priority_t priority);
	/**
	 * Pauses the execution of actions. To terminate this state, call resume() or pop().
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports POPPED,FORGOTTEN,REPLACED
	 * @data Does not have an associated data response.
	 * @return Pause command object that represents this CQM.
	 */
	public Pause pause(priority_t priority, long period);
	
	/**
	 * Pauses the execution of actions. To terminate this state, call resume() or pop().
	 * @reports POPPED,FORGOTTEN,REPLACED
	 * @data Does not have an associated data response.
	 * @return Pause command object that represents this CQM.
	 */
	public Pause pause();
	/**
	 * Pauses the execution of actions. To terminate this state, call resume() or pop().
	 * @param period The period at which to repeat the action.
	 * @reports POPPED,FORGOTTEN,REPLACED
	 * @data Does not have an associated data response.
	 * @return Pause command object that represents this CQM.
	 */
	public Pause pause(long period);
	
	
	//------------pop-------------
	/**
	 * Deletes the next command to be executed from the server command queue. 
	 * @param priority The command queue priority.
	 * @reports POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have an associated data response.
	 * @return Pop command object that represents this CQM.
	 */
	public Pop pop(priority_t priority);
	/**
	 * Deletes the next command to be executed from the server command queue. 
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have an associated data response.
	 * @return Pop command object that represents this CQM.
	 */
	public Pop pop(priority_t priority, long period);
	
	/**
	 * Deletes the next command to be executed from the server command queue. 
	 * @param priority The command queue priority.
	 * @reports POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have an associated data response.
	 * @return Pop command object that represents this CQM.
	 */
	public Pop pop();
	/**
	 * Deletes the next command to be executed from the server command queue. 
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have an associated data response.
	 * @return Pop command object that represents this CQM.
	 */
	public Pop pop(long period);
	
	
	//----------skip-------------
	/**
	 * Does nothing for a single AI frame. This is useful if you want to wait the minimum possible positive time between actions.
	 * @param priority The command queue priority.
	 * @reports POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have an associated data response.
	 * @return Skip command object that represents this CQM.
	 */
	public Skip skip(priority_t priority);
	/**
	 * Does nothing for a single AI frame. This is useful if you want to wait the minimum possible positive time between actions.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have an associated data response.
	 * @return Skip command object that represents this CQM.
	 */
	public Skip skip(priority_t priority, long period);
	
	/**
	 * Does nothing for a single AI frame. This is useful if you want to wait the minimum possible positive time between actions.
	 * @param priority The command queue priority.
	 * @reports POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have an associated data response.
	 * @return Skip command object that represents this CQM.
	 */
	public Skip skip();
	/**
	 * Does nothing for a single AI frame. This is useful if you want to wait the minimum possible positive time between actions.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports POPPED,FORGOTTEN,REPLACED,DONE
	 * @data Does not have an associated data response.
	 * @return Skip command object that represents this CQM.
	 */
	public Skip skip(long period);
	
	
	//----------------------peek----------------------------
	/**
	 * Fetches information on a command to be executed.
	 * @param depth The number of commands down the queue to look. For instance,
	 * 0 looks at the next command after peek, -1 looks at this peek command, and n looks
	 * at a command n+1 commands after peek.
	 * @param priority The command queue priority.
	 * @reports DONE,REPLACED,FORGOTTEN,POPPED
	 * @data The opcode and command id number of the command at the specified depth.
	 * @return Peek command object that represents this CQM. 
	 */
	public Peek peek(int depth, priority_t priority);
	/**
	 * Fetches information on a command to be executed.
	 * @param depth The number of commands down the queue to look. For instance,
	 * 0 looks at the next command after peek, -1 looks at this peek command, and n looks
	 * at a command n+1 commands after peek.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,REPLACED,FORGOTTEN,POPPED
	 * @data The opcode and command id number of the command at the specified depth.
	 * @return Peek command object that represents this CQM. 
	 */
	public Peek peek(int depth, priority_t priority, long period);
	
	/**
	 * Fetches information on a command to be executed.
	 * @param depth The number of commands down the queue to look. For instance,
	 * 0 looks at the next command after peek, -1 looks at this peek command, and n looks
	 * at a command n+1 commands after peek.
	 * @reports DONE,REPLACED,FORGOTTEN,POPPED
	 * @data The opcode and command id number of the command at the specified depth.
	 * @return Peek command object that represents this CQM. 
	 */
	public Peek peek(int depth);
	/**
	 * Fetches information on a command to be executed.
	 * @param depth The number of commands down the queue to look. For instance,
	 * 0 looks at the next command after peek, -1 looks at this peek command, and n looks
	 * at a command n+1 commands after peek.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,REPLACED,FORGOTTEN,POPPED
	 * @data The opcode and command id number of the command at the specified depth.
	 * @return Peek command object that represents this CQM. 
	 */
	public Peek peek(int depth, long period);
	
	/**
	 * Fetches information on the next command to be executed after this.
	 * @param priority The command queue priority.
	 * @reports DONE,REPLACED,FORGOTTEN,POPPED
	 * @data The opcode and command id number of the subsequent command..
	 * @return Peek command object that represents this CQM. 
	 */
	public Peek peek(priority_t priority);
	/**
	 * Fetches information on the next command to be executed after this.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,REPLACED,FORGOTTEN,POPPED
	 * @data The opcode and command id number of the subsequent command..
	 * @return Peek command object that represents this CQM. 
	 */
	public Peek peek(priority_t priority, long period);
	
	/**
	 * Fetches information on the next command to be executed after this.
	 * @reports DONE,REPLACED,FORGOTTEN,POPPED
	 * @data The opcode and command id number of the subsequent command..
	 * @return Peek command object that represents this CQM. 
	 */
	public Peek peek();
	/**
	 * Fetches information on the next command to be executed after this.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,REPLACED,FORGOTTEN,POPPED
	 * @data The opcode and command id number of the subsequent command..
	 * @return Peek command object that represents this CQM. 
	 */
	public Peek peek(long period);
	
	 /* -------------------------------------------------------------------- *
	  * ----------------------- Queries ------------------------------------ *
	  * -------------------------------------------------------------------- */
	
	//------------------currentLocation-------------------------
	/**
	 * Fetches the current location of the agent.
	 * @param priority The command queue priority.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The three dimensional Cartesian coordinates of the agent.
	 * @return CurrentLocation command object that represents this query.
	 */
    public CurrentLocation currentLocation(priority_t priority);
    /**
     * Fetches the current location of the agent.
     * @param priority The command queue priority.
     * @param period The period at which to repeat the action.
     * @reports DONE,POPPED,FORGOTTEN,REPLACED
     * @data The three dimensional Cartesian coordinates of the agent.
     * @return CurrentLocation command object that represents this query.
     */
    public CurrentLocation currentLocation(priority_t priority, long period);
    
	/**
	 * Fetches the current location of the agent.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The three dimensional Cartesian coordinates of the agent.
	 * @return CurrentLocation command object that represents this query.
	 */
    public CurrentLocation currentLocation();
    /**
     * Fetches the current location of the agent.
     * @param period The period at which to repeat the action.
     * @reports DONE,POPPED,FORGOTTEN,REPLACED
     * @data The three dimensional Cartesian coordinates of the agent.
     * @return CurrentLocation command object that represents this query.
     */
    public CurrentLocation currentLocation(long period);
    
    
    //-------------------currentHealth------------------------
	/**
	 * Fetches the current health of the agent.
	 * @param priority The command queue priority.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The current health of the agent.
	 * @return CurrentHealth command object that represents this query.
	 */
	public CurrentHealth currentHealth(priority_t priority);
	/**
	 * Fetches the current health of the agent.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The current health of the agent.
	 * @return CurrentHealth command object that represents this query.
	 */
	public CurrentHealth currentHealth(priority_t priority, long period);
	
	/**
	 * Fetches the current health of the agent.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The current health of the agent.
	 * @return CurrentHealth command object that represents this query.
	 */
	public CurrentHealth currentHealth();
	/**
	 * Fetches the current health of the agent.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The current health of the agent.
	 * @return CurrentHealth command object that represents this query.
	 */
	public CurrentHealth currentHealth(long period);
	
	
	//----------------------depthViewpoint--------------------
	/**
	 * Takes a depthmap image of the Quagent's current viewpoint.
	 * This is transported over the socket and automatically saved at
	 * client/API/screenshots.
	 * @param priority The command queue priority.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The encoded image.
	 * @return DepthViewpoint command object that represents this query.
	 * This object currently isn't too useful...
	 */
	public DepthViewpoint depthViewpoint(priority_t priority);
	/**
	 * Takes a depthmap image of the Quagent's current viewpoint.
	 * This is transported over the socket and automatically saved at
	 * client/API/screenshots.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the command.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The encoded image.
	 * @return DepthViewpoint command object that represents this query.
	 * This object currently isn't too useful...
	 */
	public DepthViewpoint depthViewpoint(long period, priority_t priority);
	/**
	 * Takes a depthmap image of the Quagent's current viewpoint.
	 * This is transported over the socket and automatically saved at
	 * client/API/screenshots.
	 * @param period The period at which to repeat the command.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The encoded image.
	 * @return DepthViewpoint command object that represents this query.
	 * This object currently isn't too useful...
	 */
	public DepthViewpoint depthViewpoint(long period);
	/**
	 * Takes a depthmap image of the Quagent's current viewpoint.
	 * This is transported over the socket and automatically saved at
	 * client/API/screenshots.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The encoded image.
	 * @return DepthViewpoint command object that represents this query.
	 * This object currently isn't too useful...
	 */
	public DepthViewpoint depthViewpoint();
	
	
	//----------------------imageViewpoint
	/**
	 * Takes a color image of the Quagent's current viewpoint.
	 * This is transported over the socket and automatically saved at
	 * client/API/screenshots.
	 * @param priority The command queue priority.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The encoded image.
	 * @return ImageViewpoint command object that represents this query.
	 * This object currently isn't too useful...
	 */
	public ImageViewpoint imageViewpoint(priority_t priority);
	/**
	 * Takes a color image of the Quagent's current viewpoint.
	 * This is transported over the socket and automatically saved at
	 * client/API/screenshots.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The encoded image.
	 * @return ImageViewpoint command object that represents this query.
	 * This object currently isn't too useful...
	 */
	public ImageViewpoint imageViewpoint();
	
	/**
	 * Takes a color image of the Quagent's current viewpoint.
	 * This is transported over the socket and automatically saved at
	 * client/API/screenshots.
	 * @param period The period at which to repeat the command.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The encoded image.
	 * @return ImageViewpoint command object that represents this query.
	 * This object currently isn't too useful...
	 */
	public ImageViewpoint imageViewpoint(long period);
	/**
	 * Takes a color image of the Quagent's current viewpoint.
	 * This is transported over the socket and automatically saved at
	 * client/API/screenshots.
	 * @param period The period at which to repeat the command.
	 * @param priority The command queue priority.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The encoded image.
	 * @return ImageViewpoint command object that represents this query.
	 * This object currently isn't too useful...
	 */
	public ImageViewpoint imageViewpoint(long period, priority_t priority);
	
	//----------------------pickUp----------------------------
	/**
	 * Picks up any items within the Quagent's bounding box.
	 * @param period The period at which to repeat the command.
	 * @reports DONE, NOTHING_THERE, POPPED, FORGOTTEN, REPLACED
	 * @data Boolean if picked up, details on item picked up.
	 * @return PickUp command object representing the command.
	 */
	public PickUp pickUp(long period);
	/**
	 * Picks up any items within the Quagent's bounding box.
	 * @reports DONE, NOTHING_THERE, POPPED, FORGOTTEN, REPLACED
	 * @data Boolean if picked up, details on item picked up.
	 * @return PickUp command object representing the command.
	 */
	public PickUp pickUp();
	/**
	 * Picks up any items within the Quagent's bounding box.
	 * @param period The period at which to repeat the command.
	 * @param priority The command queue priority.
	 * @reports DONE, NOTHING_THERE, POPPED, FORGOTTEN, REPLACED
	 * @data Boolean if picked up, details on item picked up.
	 * @return PickUp command object representing the command.
	 */
	public PickUp pickUp(long period, priority_t priority);
	/**
	 * Picks up any items within the Quagent's bounding box.
	 * @param priority The command queue priority.
	 * @reports DONE, NOTHING_THERE, POPPED, FORGOTTEN, REPLACED
	 * @data Boolean if picked up, details on item picked up.
	 * @return PickUp command object representing the command.
	 */
	public PickUp pickUp(priority_t priority);
	
	
	//----------------------putDown---------------------------
	/**
	 * Put downs the specified item (see static definitions in PutDown).
	 * @param item Item to put down.
	 * @param quantity Amount to put down. If the quantity is greater than current amount, is rounded down to current amount.
	 * @reports DONE,INVALID_ITEM,NONE_IN_INVENTORY,INVALID_QUANTITY,POPPED,FORGOTTEN,REPLACED
	 * @data No associated data response.
	 * @return PutDown command object representing the command.
	 */
	public PutDown putDown(int item, int quantity);
	/**
	 * Put downs the specified item (see static definitions in PutDown).
	 * @param item Item to put down.
	 * @param quantity Amount to put down. If the quantity is greater than current amount, is rounded down to current amount.
	 * @param priority The command queue priority.
	 * @reports DONE,INVALID_ITEM,NONE_IN_INVENTORY,INVALID_QUANTITY,POPPED,FORGOTTEN,REPLACED
	 * @data No associated data response.
	 * @return PutDown command object representing the command.
	 */
	public PutDown putDown(int item, int quantity, priority_t priority);
	/**
	 * Put downs the specified item (see static definitions in PutDown).
	 * @param item Item to put down.
	 * @param quantity Amount to put down. If the quantity is greater than current amount, is rounded down to current amount.
	 * @param period The period at which to execute the command.
	 * @reports DONE,INVALID_ITEM,NONE_IN_INVENTORY,INVALID_QUANTITY,POPPED,FORGOTTEN,REPLACED
	 * @data No associated data response.
	 * @return PutDown command object representing the command.
	 */
	public PutDown putDown(int item, int quantity, long period);
	/**
	 * Put downs the specified item (see static definitions in PutDown).
	 * @param item Item to put down.
	 * @param quantity Amount to put down. If the quantity is greater than current amount, is rounded down to current amount.
	 * @param period The period at which to repeat the command.
	 * @param priority The command queue priority. 
	 * @reports DONE,INVALID_ITEM,NONE_IN_INVENTORY,INVALID_QUANTITY,POPPED,FORGOTTEN,REPLACED
	 * @data No associated data response.
	 * @return PutDown command object representing the command.
	 */
	public PutDown putDown(int item, int quantity, long period, priority_t priority);
	
	
	//----------------------checkInventory-------------------
	/**
	 * Returns a list of items currently in the Quagent's inventory.
	 * @param period The period at which to execute the command.
	 * @param priority The command queue priority.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data Sequence of responses represents all items in inventory.
	 * @return CheckInventory command object representing the command.
	 */
	public CheckInventory checkInventory(long period, priority_t priority);
	/**
	 * Returns a list of items currently in the Quagent's inventory.
	 * @param period The period at which to execute the command.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data Sequence of responses represents all items in inventory.
	 * @return CheckInventory command object representing the command.
	 */
	public CheckInventory checkInventory(long period);
	/**
	 * Returns a list of items currently in the Quagent's inventory.
	 * @param priority The command queue priority.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data Sequence of responses represents all items in inventory.
	 * @return CheckInventory command object representing the command.
	 */
	public CheckInventory checkInventory(priority_t priority);
	/**
	 * Returns a list of items currently in the Quagent's inventory.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data Sequence of responses represents all items in inventory.
	 * @return CheckInventory command object representing the command.
	 */
	public CheckInventory checkInventory();
	

	
	//----------------------maxHealth-------------------------
	/**
	 * Fetches the maximum health of the agent.
	 * @param priority The command queue priority.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The maximum health of the agent.
	 * @return MaxHealth command object that represents this query.
	 */
	public MaxHealth maxHealth(priority_t priority);
	/**
	 * Fetches the maximum health of the agent.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The maximum health of the agent.
	 * @return MaxHealth command object that represents this query.
	 */
	public MaxHealth maxHealth(priority_t priority, long period);
	
	/**
	 * Fetches the maximum possible health of the agent.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The maximum health of the agent.
	 * @return MaxHealth command object that represents this query.
	 */
	public MaxHealth maxHealth();
	/**
	 * Fetches the maximum possible health of the agent.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The maximum health of the agent.
	 * @return MaxHealth command object that represents this query.
	 */
	public MaxHealth maxHealth(long period);
	
	
	//-----------------------currentArmor---------------------
	/**
	 * Fetches the agent's current armor value.
	 * @param priority The command queue priority. 
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The current armor of the agent.
	 * @return CurrentArmor command object that represents this query.
	 */
	public CurrentArmor currentArmor(priority_t priority);
	/**
	 * Fetches the agent's current armor value.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action. 
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The current armor of the agent.
	 * @return CurrentArmor command object that represents this query.
	 */
	public CurrentArmor currentArmor(priority_t priority, long period);
	
	/**
	 * Fetches the agent's current armor value.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The current armor of the agent.
	 * @return CurrentArmor command object that represents this query.
	 */
	public CurrentArmor currentArmor();
	/**
	 * Fetches the agent's current armor value.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The current armor of the agent.
	 * @return CurrentArmor command object that represents this query.
	 */
	public CurrentArmor currentArmor(long period);
	
	
	//-------------------------currentAmmo-----------------------
	/**
	 * Fetches the agent's remaining ammunition for a given weapon.
	 * @param wep Weapon to fetch ammunition remaining for. 
	 * @param priority The command queue priority.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED,INVALID_WEAPON
	 * @data The current ammunition for <i>wep</i>.
	 * @return CurrentAmmo command object that represents this query.
	 */
	public CurrentAmmo currentAmmo(weapon wep, priority_t priority);
	
	/**
	 * Fetches the agent's remaining ammunition for a given weapon.
	 * @param wep Weapon to fetch ammunition remaining for. 
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED,INVALID_WEAPON
	 * @data The current ammunition for <i>wep</i>.
	 * @return CurrentAmmo command object that represents this query.
	 */
	public CurrentAmmo currentAmmo(weapon wep, priority_t priority, long period);
	
	/**
	 * Fetches the agent's remaining ammunition for a given weapon.
	 * @param wep Weapon to fetch ammunition remaining for. 
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED,INVALID_WEAPON
	 * @data The current ammunition for <i>wep</i>.
	 * @return CurrentAmmo command object that represents this query.
	 */
	public CurrentAmmo currentAmmo(weapon wep);
	
	/**
	 * Fetches the agent's remaining ammunition for a given weapon.
	 * @param wep Weapon to fetch ammunition remaining for. 
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED,INVALID_WEAPON
	 * @data The current ammunition for <i>wep</i>.
	 * @return CurrentAmmo command object that represents this query.
	 */
	public CurrentAmmo currentAmmo(weapon wep, long period);
	
	
	//------------------------maxArmor---------------------------
	/**
	 * Fetches the maximum possible armor value of the agent.
	 * @param priority The command queue priority.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The maximum armor of the agent.
	 * @return MaxArmor command object that represents this query.
	 */
	public MaxArmor maxArmor(priority_t priority);
	/**
	 * Fetches the maximum possible armor value of the agent.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The maximum armor of the agent.
	 * @return MaxArmor command object that represents this query.
	 */
	public MaxArmor maxArmor(priority_t priority, long period);
	
	/**
	 * Fetches the maximum possible armor value of the agent.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The maximum armor of the agent.
	 * @return MaxArmor command object that represents this query.
	 */
	public MaxArmor maxArmor();
	/**
	 * Fetches the maximum possible armor value of the agent.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The maximum armor of the agent.
	 * @return MaxArmor command object that represents this query.
	 */
	public MaxArmor maxArmor(long period);
	
	
	//------------------------canSee-----------------------------
	/**
	 * Asks the agent to tell you if it has line of "sight" to another entity (specified by the entityid you give it). 
	 * For Quagents, this entity id is located at client.qid. 
	 * The line of sight ray is drawn from 30 units above the origin of the bot to 30 units above the origin of the target. 
	 * Unlike the Eye of Sauron, its gaze only pierces cloud, shadow, and flesh (not earth). 
	 * In other words, obstructing entities, water, slime, or lava do not block line of sight - only walls, floors, and things like that do. 
	 * It does not bounce off mirrored surfaces or go through portals. Lack of light does not affect this operation.
	 * @param entityid The unique identifier for the entity.
	 * @param priority The command queue identity.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data A boolean which indicates whether the entity is within the agent's line of sight. If true, also responds with the distance, yaw, and pitch to the
	 * target.
	 * @return CanSee command object that represents this query. 
	 */
	public CanSee canSee(int entityid,priority_t priority);
	/**
	 * Asks the agent to tell you if it has line of "sight" to another entity (specified by the entityid you give it). 
	 * For Quagents, this entity id is located at client.qid. 
	 * The line of sight ray is drawn from 30 units above the origin of the bot to 30 units above the origin of the target. 
	 * Unlike the Eye of Sauron, its gaze only pierces cloud, shadow, and flesh (not earth). 
	 * In other words, obstructing entities, water, slime, or lava do not block line of sight - only walls, floors, and things like that do. 
	 * It does not bounce off mirrored surfaces or go through portals. Lack of light does not affect this operation.
	 * @param entityid The unique identifier for the entity.
	 * @param priority The command queue identity.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data A boolean which indicates whether the entity is within the agent's line of sight. If true, also responds with the distance, yaw, and pitch to the
	 * target.
	 * @return CanSee command object that represents this query. 
	 */
	public CanSee canSee(int entityid,priority_t priority, long period);

	/**
	 * Asks the agent to tell you if it has line of "sight" to another entity (specified by the entityid you give it). 
	 * For Quagents, this entity id is located at client.qid. 
	 * The line of sight ray is drawn from 30 units above the origin of the bot to 30 units above the origin of the target. 
	 * Unlike the Eye of Sauron, its gaze only pierces cloud, shadow, and flesh (not earth). 
	 * In other words, obstructing entities, water, slime, or lava do not block line of sight - only walls, floors, and things like that do. 
	 * It does not bounce off mirrored surfaces or go through portals. Lack of light does not affect this operation.
	 * @param entityid The unique identifier for the entity.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data A boolean which indicates whether the entity is within the agent's line of sight. If true, also responds with the distance, yaw, and pitch to the
	 * target.
	 * @return CanSee command object that represents this query. 
	 */
	public CanSee canSee(int entityid);
	/**
	 * Asks the agent to tell you if it has line of "sight" to another entity (specified by the entityid you give it). 
	 * For Quagents, this entity id is located at client.qid. 
	 * The line of sight ray is drawn from 30 units above the origin of the bot to 30 units above the origin of the target. 
	 * Unlike the Eye of Sauron, its gaze only pierces cloud, shadow, and flesh (not earth). 
	 * In other words, obstructing entities, water, slime, or lava do not block line of sight - only walls, floors, and things like that do. 
	 * It does not bounce off mirrored surfaces or go through portals. Lack of light does not affect this operation.
	 * @param entityid The unique identifier for the entity.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data A boolean which indicates whether the entity is within the agent's line of sight. If true, also responds with the distance, yaw, and pitch to the
	 * target.
	 * @return CanSee command object that represents this query. 
	 */
	public CanSee canSee(int entityid, long period);
	
	
	//-------------------facing---------------------
	/**
	 * Fetches the agent's current facing.
	 * @param priority The command queue priority.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data View angles of the bot.
	 * @return Facing command object that represents this query. 
	 */
	public Facing facing(priority_t priority);
	/**
	 * Fetches the agent's current facing.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data View angles of the bot.
	 * @return Facing command object that represents this query. 
	 */
	public Facing facing(priority_t priority, long period);
	
	/**
	 * Fetches the agent's current facing.
	 * @param priority The command queue priority.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data View angles of the bot.
	 * @return Facing command object that represents this query. 
	 */
	public Facing facing();
	/**
	 * Fetches the agent's current facing.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data View angles of the bot.
	 * @return Facing command object that represents this query. 
	 */
	public Facing facing(long period);
	
	
	//-------------------------------------rangefinder-----------------------------------------------
	/**
	 * Asks the agent to tell you what and where the nearest object within a specific distance at a specific angle is.
	 * @param type The type of rangefinder to use. They are as follows: 
	 * <ul><li> 0: An infinitely narrow beam that is only stopped by permanent surfaces that will stop the player (usually just walls, floors, ceilings, and moving platforms, never other quagents or rockets or powerups). </li>
	 * <li> 1: An infinitely narrow beam that is blocked by anything that would block player movement (like mode-0, but plus other things like quagents). </li>
	 * <li> 2: An infinitely narrow beam that is blocked by anything that would block a shot (often equivalent to mode 1, but on some levels (like suspended), there are walls/floors that can be shot through but not walked through). </li>
	 * <li> 3: An infinitely narrow beam that is blocked by anything in the environment (like mode-1, but plus powerups or other things that can be walked through). </li>
	 * <li> 4: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-0 rangefinder. </li>  
	 * <li> 5: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-1 rangefinder. </li>  
	 * <li> 6: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-2 rangefinder. </li>  
	 * <li> 7: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-3 rangefinder. </li>
	 * </ul>
	 * @param range The maximum distance of the rangefinder. So far, the maximum distance we have seen in Quake has been 8120, so a value of 9999 should act as infinite for all intents and purposes.
	 * @param rot The rotational angle (in degrees) left of Quagent's facing to fire the rangefinder.
	 * @param azi The azimuthal angle (in degrees) down from Quagent's facing to fire the rangefinder.
	 * @param priority The command queue priority.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The range at which the object was detected, and the corresponding entity id (-1 if nothing detected).
	 * @return Rangefinder command object that represents this query.   
	 */
	public Rangefinder rangefinder(int type, int range, double rot, double azi, priority_t priority);
	/**
	 * Asks the agent to tell you what and where the nearest object within a specific distance at a specific angle is.
	 * @param type The type of rangefinder to use. They are as follows: 
	 * <ul><li> 0: An infinitely narrow beam that is only stopped by permanent surfaces that will stop the player (usually just walls, floors, ceilings, and moving platforms, never other quagents or rockets or powerups). </li>
	 * <li> 1: An infinitely narrow beam that is blocked by anything that would block player movement (like mode-0, but plus other things like quagents). </li>
	 * <li> 2: An infinitely narrow beam that is blocked by anything that would block a shot (often equivalent to mode 1, but on some levels (like suspended), there are walls/floors that can be shot through but not walked through). </li>
	 * <li> 3: An infinitely narrow beam that is blocked by anything in the environment (like mode-1, but plus powerups or other things that can be walked through). </li>
	 * <li> 4: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-0 rangefinder. </li>  
	 * <li> 5: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-1 rangefinder. </li>  
	 * <li> 6: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-2 rangefinder. </li>  
	 * <li> 7: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-3 rangefinder. </li>
	 * </ul>
	 * @param range The maximum distance of the rangefinder. So far, the maximum distance we have seen in Quake has been 8120, so a value of 9999 should act as infinite for all intents and purposes.
	 * @param rot The rotational angle (in degrees) left of Quagent's facing to fire the rangefinder.
	 * @param azi The azimuthal angle (in degrees) down from Quagent's facing to fire the rangefinder.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The range at which the object was detected, and the corresponding entity id (-1 if nothing detected).
	 * @return Rangefinder command object that represents this query.   
	 */
	public Rangefinder rangefinder(int type, int range, double rot, double azi, priority_t priority, long period);
	
	/**
	 * Asks the agent to tell you what and where the nearest object within a specific distance at a specific angle is.
	 * @param type The type of rangefinder to use. They are as follows: 
	 * <ul><li> 0: An infinitely narrow beam that is only stopped by permanent surfaces that will stop the player (usually just walls, floors, ceilings, and moving platforms, never other quagents or rockets or powerups). </li>
	 * <li> 1: An infinitely narrow beam that is blocked by anything that would block player movement (like mode-0, but plus other things like quagents). </li>
	 * <li> 2: An infinitely narrow beam that is blocked by anything that would block a shot (often equivalent to mode 1, but on some levels (like suspended), there are walls/floors that can be shot through but not walked through). </li>
	 * <li> 3: An infinitely narrow beam that is blocked by anything in the environment (like mode-1, but plus powerups or other things that can be walked through). </li>
	 * <li> 4: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-0 rangefinder. </li>  
	 * <li> 5: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-1 rangefinder. </li>  
	 * <li> 6: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-2 rangefinder. </li>  
	 * <li> 7: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-3 rangefinder. </li>
	 * </ul>
	 * @param range The maximum distance of the rangefinder. So far, the maximum distance we have seen in Quake has been 8120, so a value of 9999 should act as infinite for all intents and purposes.
	 * @param rot The rotational angle (in degrees) left of Quagent's facing to fire the rangefinder.
	 * @param azi The azimuthal angle (in degrees) down from Quagent's facing to fire the rangefinder.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The range at which the object was detected, and the corresponding entity id (-1 if nothing detected).
	 * @return Rangefinder command object that represents this query.   
	 */
	public Rangefinder rangefinder(int type, int range, double rot, double azi);
	/**
	 * Asks the agent to tell you what and where the nearest object within a specific distance at a specific angle is.
	 * @param type The type of rangefinder to use. They are as follows: 
	 * <ul><li> 0: An infinitely narrow beam that is only stopped by permanent surfaces that will stop the player (usually just walls, floors, ceilings, and moving platforms, never other quagents or rockets or powerups). </li>
	 * <li> 1: An infinitely narrow beam that is blocked by anything that would block player movement (like mode-0, but plus other things like quagents). </li>
	 * <li> 2: An infinitely narrow beam that is blocked by anything that would block a shot (often equivalent to mode 1, but on some levels (like suspended), there are walls/floors that can be shot through but not walked through). </li>
	 * <li> 3: An infinitely narrow beam that is blocked by anything in the environment (like mode-1, but plus powerups or other things that can be walked through). </li>
	 * <li> 4: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-0 rangefinder. </li>  
	 * <li> 5: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-1 rangefinder. </li>  
	 * <li> 6: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-2 rangefinder. </li>  
	 * <li> 7: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-3 rangefinder. </li>
	 * </ul>
	 * @param range The maximum distance of the rangefinder. So far, the maximum distance we have seen in Quake has been 8120, so a value of 9999 should act as infinite for all intents and purposes.
	 * @param rot The rotational angle (in degrees) left of Quagent's facing to fire the rangefinder.
	 * @param azi The azimuthal angle (in degrees) down from Quagent's facing to fire the rangefinder.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The range at which the object was detected, and the corresponding entity id (-1 if nothing detected).
	 * @return Rangefinder command object that represents this query.   
	 */
	public Rangefinder rangefinder(int type, int range, double rot, double azi, long period);
	
	/**
	 * Asks the agent to tell you what and where the nearest object at a specific angle is.
	 * @param type The type of rangefinder to use. They are as follows: 
	 * <ul><li> 0: An infinitely narrow beam that is only stopped by permanent surfaces that will stop the player (usually just walls, floors, ceilings, and moving platforms, never other quagents or rockets or powerups). </li>
	 * <li> 1: An infinitely narrow beam that is blocked by anything that would block player movement (like mode-0, but plus other things like quagents). </li>
	 * <li> 2: An infinitely narrow beam that is blocked by anything that would block a shot (often equivalent to mode 1, but on some levels (like suspended), there are walls/floors that can be shot through but not walked through). </li>
	 * <li> 3: An infinitely narrow beam that is blocked by anything in the environment (like mode-1, but plus powerups or other things that can be walked through). </li>
	 * <li> 4: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-0 rangefinder. </li>  
	 * <li> 5: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-1 rangefinder. </li>  
	 * <li> 6: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-2 rangefinder. </li>  
	 * <li> 7: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-3 rangefinder. </li>
	 * </ul>
	 * @param rot The rotational angle (in degrees) left of Quagent's facing to fire the rangefinder.
	 * @param azi The azimuthal angle (in degrees) down from Quagent's facing to fire the rangefinder.
	 * @param priority The command queue priority.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The range at which the object was detected, and the corresponding entity id (-1 if nothing detected).
	 * @return Rangefinder command object that represents this query.   
	 */
	public Rangefinder rangefinder(int type, double rot, double azi, priority_t priority);
	/**
	 * Asks the agent to tell you what and where the nearest object at a specific angle is.
	 * @param type The type of rangefinder to use. They are as follows: 
	 * <ul><li> 0: An infinitely narrow beam that is only stopped by permanent surfaces that will stop the player (usually just walls, floors, ceilings, and moving platforms, never other quagents or rockets or powerups). </li>
	 * <li> 1: An infinitely narrow beam that is blocked by anything that would block player movement (like mode-0, but plus other things like quagents). </li>
	 * <li> 2: An infinitely narrow beam that is blocked by anything that would block a shot (often equivalent to mode 1, but on some levels (like suspended), there are walls/floors that can be shot through but not walked through). </li>
	 * <li> 3: An infinitely narrow beam that is blocked by anything in the environment (like mode-1, but plus powerups or other things that can be walked through). </li>
	 * <li> 4: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-0 rangefinder. </li>  
	 * <li> 5: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-1 rangefinder. </li>  
	 * <li> 6: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-2 rangefinder. </li>  
	 * <li> 7: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-3 rangefinder. </li>
	 * </ul>
	 * @param rot The rotational angle (in degrees) left of Quagent's facing to fire the rangefinder.
	 * @param azi The azimuthal angle (in degrees) down from Quagent's facing to fire the rangefinder.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The range at which the object was detected, and the corresponding entity id (-1 if nothing detected).
	 * @return Rangefinder command object that represents this query.   
	 */
	public Rangefinder rangefinder(int type, double rot, double azi, priority_t priority, long period);
	
	/**
	 * Asks the agent to tell you what and where the nearest object at a specific angle is.
	 * @param type The type of rangefinder to use. They are as follows: 
	 * <ul><li> 0: An infinitely narrow beam that is only stopped by permanent surfaces that will stop the player (usually just walls, floors, ceilings, and moving platforms, never other quagents or rockets or powerups). </li>
	 * <li> 1: An infinitely narrow beam that is blocked by anything that would block player movement (like mode-0, but plus other things like quagents). </li>
	 * <li> 2: An infinitely narrow beam that is blocked by anything that would block a shot (often equivalent to mode 1, but on some levels (like suspended), there are walls/floors that can be shot through but not walked through). </li>
	 * <li> 3: An infinitely narrow beam that is blocked by anything in the environment (like mode-1, but plus powerups or other things that can be walked through). </li>
	 * <li> 4: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-0 rangefinder. </li>  
	 * <li> 5: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-1 rangefinder. </li>  
	 * <li> 6: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-2 rangefinder. </li>  
	 * <li> 7: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-3 rangefinder. </li>
	 * </ul>
	 * @param rot The rotational angle (in degrees) left of Quagent's facing to fire the rangefinder.
	 * @param azi The azimuthal angle (in degrees) down from Quagent's facing to fire the rangefinder.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The range at which the object was detected, and the corresponding entity id (-1 if nothing detected).
	 * @return Rangefinder command object that represents this query.   
	 */
	public Rangefinder rangefinder(int type, double rot, double azi);
	/**
	 * Asks the agent to tell you what and where the nearest object at a specific angle is.
	 * @param type The type of rangefinder to use. They are as follows: 
	 * <ul><li> 0: An infinitely narrow beam that is only stopped by permanent surfaces that will stop the player (usually just walls, floors, ceilings, and moving platforms, never other quagents or rockets or powerups). </li>
	 * <li> 1: An infinitely narrow beam that is blocked by anything that would block player movement (like mode-0, but plus other things like quagents). </li>
	 * <li> 2: An infinitely narrow beam that is blocked by anything that would block a shot (often equivalent to mode 1, but on some levels (like suspended), there are walls/floors that can be shot through but not walked through). </li>
	 * <li> 3: An infinitely narrow beam that is blocked by anything in the environment (like mode-1, but plus powerups or other things that can be walked through). </li>
	 * <li> 4: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-0 rangefinder. </li>  
	 * <li> 5: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-1 rangefinder. </li>  
	 * <li> 6: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-2 rangefinder. </li>  
	 * <li> 7: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-3 rangefinder. </li>
	 * </ul>
	 * @param rot The rotational angle (in degrees) left of Quagent's facing to fire the rangefinder.
	 * @param azi The azimuthal angle (in degrees) down from Quagent's facing to fire the rangefinder.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The range at which the object was detected, and the corresponding entity id (-1 if nothing detected).
	 * @return Rangefinder command object that represents this query.   
	 */
	public Rangefinder rangefinder(int type, double rot, double azi, long period);
	
	/**
	 * Asks the agent to tell you what and where the nearest object at a specific angle is.
	 * @param type The type of rangefinder to use. They are as follows: 
	 * <ul><li> 0: An infinitely narrow beam that is only stopped by permanent surfaces that will stop the player (usually just walls, floors, ceilings, and moving platforms, never other quagents or rockets or powerups). </li>
	 * <li> 1: An infinitely narrow beam that is blocked by anything that would block player movement (like mode-0, but plus other things like quagents). </li>
	 * <li> 2: An infinitely narrow beam that is blocked by anything that would block a shot (often equivalent to mode 1, but on some levels (like suspended), there are walls/floors that can be shot through but not walked through). </li>
	 * <li> 3: An infinitely narrow beam that is blocked by anything in the environment (like mode-1, but plus powerups or other things that can be walked through). </li>
	 * <li> 4: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-0 rangefinder. </li>  
	 * <li> 5: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-1 rangefinder. </li>  
	 * <li> 6: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-2 rangefinder. </li>  
	 * <li> 7: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-3 rangefinder. </li>
	 * </ul>
	 * @param rot The rotational angle (in degrees) left of Quagent's facing to fire the rangefinder.
	 * @param priority The command queue priority.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The range at which the object was detected, and the corresponding entity id (-1 if nothing detected).
	 * @return Rangefinder command object that represents this query.   
	 */
	public Rangefinder rangefinder(int type, double rot, priority_t priority);
	/**
	 * Asks the agent to tell you what and where the nearest object at a specific angle is.
	 * @param type The type of rangefinder to use. They are as follows: 
	 * <ul><li> 0: An infinitely narrow beam that is only stopped by permanent surfaces that will stop the player (usually just walls, floors, ceilings, and moving platforms, never other quagents or rockets or powerups). </li>
	 * <li> 1: An infinitely narrow beam that is blocked by anything that would block player movement (like mode-0, but plus other things like quagents). </li>
	 * <li> 2: An infinitely narrow beam that is blocked by anything that would block a shot (often equivalent to mode 1, but on some levels (like suspended), there are walls/floors that can be shot through but not walked through). </li>
	 * <li> 3: An infinitely narrow beam that is blocked by anything in the environment (like mode-1, but plus powerups or other things that can be walked through). </li>
	 * <li> 4: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-0 rangefinder. </li>  
	 * <li> 5: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-1 rangefinder. </li>  
	 * <li> 6: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-2 rangefinder. </li>  
	 * <li> 7: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-3 rangefinder. </li>
	 * </ul>
	 * @param rot The rotational angle (in degrees) left of Quagent's facing to fire the rangefinder.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The range at which the object was detected, and the corresponding entity id (-1 if nothing detected).
	 * @return Rangefinder command object that represents this query.   
	 */
	public Rangefinder rangefinder(int type, double rot, priority_t priority, long period);
	
	/**
	 * Asks the agent to tell you what and where the nearest object at a specific angle is.
	 * @param type The type of rangefinder to use. They are as follows: 
	 * <ul><li> 0: An infinitely narrow beam that is only stopped by permanent surfaces that will stop the player (usually just walls, floors, ceilings, and moving platforms, never other quagents or rockets or powerups). </li>
	 * <li> 1: An infinitely narrow beam that is blocked by anything that would block player movement (like mode-0, but plus other things like quagents). </li>
	 * <li> 2: An infinitely narrow beam that is blocked by anything that would block a shot (often equivalent to mode 1, but on some levels (like suspended), there are walls/floors that can be shot through but not walked through). </li>
	 * <li> 3: An infinitely narrow beam that is blocked by anything in the environment (like mode-1, but plus powerups or other things that can be walked through). </li>
	 * <li> 4: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-0 rangefinder. </li>  
	 * <li> 5: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-1 rangefinder. </li>  
	 * <li> 6: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-2 rangefinder. </li>  
	 * <li> 7: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-3 rangefinder. </li>
	 * </ul>
	 * @param rot The rotational angle (in degrees) left of Quagent's facing to fire the rangefinder.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The range at which the object was detected, and the corresponding entity id (-1 if nothing detected).
	 * @return Rangefinder command object that represents this query.   
	 */
	public Rangefinder rangefinder(int type, double rot);
	/**
	 * Asks the agent to tell you what and where the nearest object at a specific angle is.
	 * @param type The type of rangefinder to use. They are as follows: 
	 * <ul><li> 0: An infinitely narrow beam that is only stopped by permanent surfaces that will stop the player (usually just walls, floors, ceilings, and moving platforms, never other quagents or rockets or powerups). </li>
	 * <li> 1: An infinitely narrow beam that is blocked by anything that would block player movement (like mode-0, but plus other things like quagents). </li>
	 * <li> 2: An infinitely narrow beam that is blocked by anything that would block a shot (often equivalent to mode 1, but on some levels (like suspended), there are walls/floors that can be shot through but not walked through). </li>
	 * <li> 3: An infinitely narrow beam that is blocked by anything in the environment (like mode-1, but plus powerups or other things that can be walked through). </li>
	 * <li> 4: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-0 rangefinder. </li>  
	 * <li> 5: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-1 rangefinder. </li>  
	 * <li> 6: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-2 rangefinder. </li>  
	 * <li> 7: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-3 rangefinder. </li>
	 * </ul>
	 * @param rot The rotational angle (in degrees) left of Quagent's facing to fire the rangefinder.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The range at which the object was detected, and the corresponding entity id (-1 if nothing detected).
	 * @return Rangefinder command object that represents this query.   
	 */
	public Rangefinder rangefinder(int type, double rot, long period);
	
	/**
	 * Asks the agent to tell you what and where the nearest object at a specific angle is. Uses the type-1 rangefinder.
	 * @param rot The rotational angle (in degrees) left of Quagent's facing to fire the rangefinder.
	 * @param priority The command queue priority.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The range at which the object was detected, and the corresponding entity id (-1 if nothing detected).
	 * @return Rangefinder command object that represents this query.   
	 */
	public Rangefinder rangefinder(double rot, priority_t priority);
	/**
	 * Asks the agent to tell you what and where the nearest object at a specific angle is. Uses the type-1 rangefinder.
	 * @param rot The rotational angle (in degrees) left of Quagent's facing to fire the rangefinder.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The range at which the object was detected, and the corresponding entity id (-1 if nothing detected).
	 * @return Rangefinder command object that represents this query.   
	 */
	public Rangefinder rangefinder(double rot, priority_t priority, long period);
	
	/**
	 * Asks the agent to tell you what and where the nearest object at a specific angle is. Uses the type-1 rangefinder.
	 * @param rot The rotational angle (in degrees) left of Quagent's facing to fire the rangefinder.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The range at which the object was detected, and the corresponding entity id (-1 if nothing detected).
	 * @return Rangefinder command object that represents this query.   
	 */
	public Rangefinder rangefinder(double rot);
	/**
	 * Asks the agent to tell you what and where the nearest object at a specific angle is. Uses the type-1 rangefinder.
	 * @param rot The rotational angle (in degrees) left of Quagent's facing to fire the rangefinder.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The range at which the object was detected, and the corresponding entity id (-1 if nothing detected).
	 * @return Rangefinder command object that represents this query.   
	 */
	public Rangefinder rangefinder(double rot, long period);
	
	/**
	 * Asks the agent to tell you what and where the nearest object located in front of the Quagent is.
	 * @param type The type of rangefinder to use. They are as follows: 
	 * <ul><li> 0: An infinitely narrow beam that is only stopped by permanent surfaces that will stop the player (usually just walls, floors, ceilings, and moving platforms, never other quagents or rockets or powerups). </li>
	 * <li> 1: An infinitely narrow beam that is blocked by anything that would block player movement (like mode-0, but plus other things like quagents). </li>
	 * <li> 2: An infinitely narrow beam that is blocked by anything that would block a shot (often equivalent to mode 1, but on some levels (like suspended), there are walls/floors that can be shot through but not walked through). </li>
	 * <li> 3: An infinitely narrow beam that is blocked by anything in the environment (like mode-1, but plus powerups or other things that can be walked through). </li>
	 * <li> 4: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-0 rangefinder. </li>  
	 * <li> 5: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-1 rangefinder. </li>  
	 * <li> 6: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-2 rangefinder. </li>  
	 * <li> 7: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-3 rangefinder. </li>
	 * </ul>
	 * @param priority The command queue priority.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The range at which the object was detected, and the corresponding entity id (-1 if nothing detected).
	 * @return Rangefinder command object that represents this query.   
	 */
	public Rangefinder rangefinder(int type, priority_t priority);
	/**
	 * Asks the agent to tell you what and where the nearest object located in front of the Quagent is.
	 * @param type The type of rangefinder to use. They are as follows: 
	 * <ul><li> 0: An infinitely narrow beam that is only stopped by permanent surfaces that will stop the player (usually just walls, floors, ceilings, and moving platforms, never other quagents or rockets or powerups). </li>
	 * <li> 1: An infinitely narrow beam that is blocked by anything that would block player movement (like mode-0, but plus other things like quagents). </li>
	 * <li> 2: An infinitely narrow beam that is blocked by anything that would block a shot (often equivalent to mode 1, but on some levels (like suspended), there are walls/floors that can be shot through but not walked through). </li>
	 * <li> 3: An infinitely narrow beam that is blocked by anything in the environment (like mode-1, but plus powerups or other things that can be walked through). </li>
	 * <li> 4: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-0 rangefinder. </li>  
	 * <li> 5: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-1 rangefinder. </li>  
	 * <li> 6: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-2 rangefinder. </li>  
	 * <li> 7: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-3 rangefinder. </li>
	 * </ul>
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The range at which the object was detected, and the corresponding entity id (-1 if nothing detected).
	 * @return Rangefinder command object that represents this query.   
	 */
	public Rangefinder rangefinder(int type, priority_t priority, long period);
	
	/**
	 * Asks the agent to tell you what and where the nearest object in front of the Quagent is.
	 * @param type The type of rangefinder to use. They are as follows: 
	 * <ul><li> 0: An infinitely narrow beam that is only stopped by permanent surfaces that will stop the player (usually just walls, floors, ceilings, and moving platforms, never other quagents or rockets or powerups). </li>
	 * <li> 1: An infinitely narrow beam that is blocked by anything that would block player movement (like mode-0, but plus other things like quagents). </li>
	 * <li> 2: An infinitely narrow beam that is blocked by anything that would block a shot (often equivalent to mode 1, but on some levels (like suspended), there are walls/floors that can be shot through but not walked through). </li>
	 * <li> 3: An infinitely narrow beam that is blocked by anything in the environment (like mode-1, but plus powerups or other things that can be walked through). </li>
	 * <li> 4: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-0 rangefinder. </li>  
	 * <li> 5: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-1 rangefinder. </li>  
	 * <li> 6: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-2 rangefinder. </li>  
	 * <li> 7: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-3 rangefinder. </li>
	 * </ul>
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The range at which the object was detected, and the corresponding entity id (-1 if nothing detected).
	 * @return Rangefinder command object that represents this query.   
	 */
	public Rangefinder rangefinder(int type);
	/**
	 * Asks the agent to tell you what and where the nearest object in front of the Quagent is.
	 * @param type The type of rangefinder to use. They are as follows: 
	 * <ul><li> 0: An infinitely narrow beam that is only stopped by permanent surfaces that will stop the player (usually just walls, floors, ceilings, and moving platforms, never other quagents or rockets or powerups). </li>
	 * <li> 1: An infinitely narrow beam that is blocked by anything that would block player movement (like mode-0, but plus other things like quagents). </li>
	 * <li> 2: An infinitely narrow beam that is blocked by anything that would block a shot (often equivalent to mode 1, but on some levels (like suspended), there are walls/floors that can be shot through but not walked through). </li>
	 * <li> 3: An infinitely narrow beam that is blocked by anything in the environment (like mode-1, but plus powerups or other things that can be walked through). </li>
	 * <li> 4: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-0 rangefinder. </li>  
	 * <li> 5: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-1 rangefinder. </li>  
	 * <li> 6: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-2 rangefinder. </li>  
	 * <li> 7: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-3 rangefinder. </li>
	 * </ul>
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The range at which the object was detected, and the corresponding entity id (-1 if nothing detected).
	 * @return Rangefinder command object that represents this query.   
	 */
	public Rangefinder rangefinder(int type, long period);
	
	/**
	 * Asks the agent to tell you what and where the nearest object in front of the Quagent is. Uses type-1 rangefinder.
	 * @param priority The command queue priority.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The range at which the object was detected, and the corresponding entity id (-1 if nothing detected).
	 * @return Rangefinder command object that represents this query.   
	 */
	public Rangefinder rangefinder(priority_t priority);
	/**
	 * Asks the agent to tell you what and where the nearest object in front of the Quagent is. Uses type-1 rangefinder.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The range at which the object was detected, and the corresponding entity id (-1 if nothing detected).
	 * @return Rangefinder command object that represents this query.   
	 */
	public Rangefinder rangefinder(priority_t priority, long period);
	
	/**
	 * Asks the agent to tell you what and where the nearest object in front of the Quagent is. Uses type-1 rangefinder.
	 * @param priority The command queue priority.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The range at which the object was detected, and the corresponding entity id (-1 if nothing detected).
	 * @return Rangefinder command object that represents this query.   
	 */
	public Rangefinder rangefinder();
	/**
	 * Asks the agent to tell you what and where the nearest object in front of the Quagent is. Uses type-1 rangefinder.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The range at which the object was detected, and the corresponding entity id (-1 if nothing detected).
	 * @return Rangefinder command object that represents this query.   
	 */
	public Rangefinder rangefinder(long period);
	
	
	//---------------batchRangefinder----------------------
	/**
	 * Batch version of rangefinder. Used for intensive rangefinder applications, such as image aquisition. 
	 * @param type The type of rangefinder to use. They are as follows: 
	 * <ul><li> 0: An infinitely narrow beam that is only stopped by permanent surfaces that will stop the player (usually just walls, floors, ceilings, and moving platforms, never other quagents or rockets or powerups). </li>
	 * <li> 1: An infinitely narrow beam that is blocked by anything that would block player movement (like mode-0, but plus other things like quagents). </li>
	 * <li> 2: An infinitely narrow beam that is blocked by anything that would block a shot (often equivalent to mode 1, but on some levels (like suspended), there are walls/floors that can be shot through but not walked through). </li>
	 * <li> 3: An infinitely narrow beam that is blocked by anything in the environment (like mode-1, but plus powerups or other things that can be walked through). </li>
	 * <li> 4: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-0 rangefinder. </li>  
	 * <li> 5: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-1 rangefinder. </li>  
	 * <li> 6: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-2 rangefinder. </li>  
	 * <li> 7: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-3 rangefinder. </li>
	 * </ul>
	 * @param distance The maximum distance of the rangefinder. So far, the maximum distance we have seen in Quake has been 8120, so a value of 9999 should act as infinite for all intents and purposes.
	 * @param rot The rotational angle (in degrees) left of the Quagent's facing to start firing the rangefinder.
	 * @param azi The azimuthal angle (in degrees) down from the Quagent's facing to start firing the rangefinder.
	 * @param width Number of YAW-variant samples.
	 * @param height Number of PITCH-variant samples.
	 * @param sampleDeltaYaw How far apart two YAW-variant samples should be.
	 * @param sampleDeltaPitch How far apart two PITCH-variant samples should be.
	 * @param priority The command queue priority.
	 * @reports DONE,REPLACED,POPPED,FORGOTTEN
	 * @data n X1 E1 X2 E2 X3 E3... Xn En where n is the number of samples (width*height) in the raster. Each Xi is the range at which the rangefinder stopped, and each corresponding Ei is the entityid of the entity that stopped it.
	 * @return BatchRangefinder command object which represents this query.
	 */
	public BatchRangefinder batchRangefinder(int type, int distance, double rot, double azi, int width, int height, double sampleDeltaYaw, double sampleDeltaPitch, priority_t priority);
	/**
	 * Batch version of rangefinder. Used for intensive rangefinder applications, such as image aquisition. 
	 * @param type The type of rangefinder to use. They are as follows: 
	 * <ul><li> 0: An infinitely narrow beam that is only stopped by permanent surfaces that will stop the player (usually just walls, floors, ceilings, and moving platforms, never other quagents or rockets or powerups). </li>
	 * <li> 1: An infinitely narrow beam that is blocked by anything that would block player movement (like mode-0, but plus other things like quagents). </li>
	 * <li> 2: An infinitely narrow beam that is blocked by anything that would block a shot (often equivalent to mode 1, but on some levels (like suspended), there are walls/floors that can be shot through but not walked through). </li>
	 * <li> 3: An infinitely narrow beam that is blocked by anything in the environment (like mode-1, but plus powerups or other things that can be walked through). </li>
	 * <li> 4: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-0 rangefinder. </li>  
	 * <li> 5: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-1 rangefinder. </li>  
	 * <li> 6: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-2 rangefinder. </li>  
	 * <li> 7: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-3 rangefinder. </li>
	 * </ul>
	 * @param distance The maximum distance of the rangefinder. So far, the maximum distance we have seen in Quake has been 8120, so a value of 9999 should act as infinite for all intents and purposes.
	 * @param rot The rotational angle (in degrees) left of the Quagent's facing to start firing the rangefinder.
	 * @param azi The azimuthal angle (in degrees) down from the Quagent's facing to start firing the rangefinder.
	 * @param width Number of YAW-variant samples.
	 * @param height Number of PITCH-variant samples.
	 * @param sampleDeltaYaw How far apart two YAW-variant samples should be.
	 * @param sampleDeltaPitch How far apart two PITCH-variant samples should be.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,REPLACED,POPPED,FORGOTTEN
	 * @data n X1 E1 X2 E2 X3 E3... Xn En where n is the number of samples (width*height) in the raster. Each Xi is the range at which the rangefinder stopped, and each corresponding Ei is the entityid of the entity that stopped it.
	 * @return BatchRangefinder command object which represents this query.
	 */
	public BatchRangefinder batchRangefinder(int type, int distance, double rot, double azi, int width, int height, double sampleDeltaYaw, double sampleDeltaPitch, priority_t priority, long period);
	
	/**
	 * Batch version of rangefinder. Used for intensive rangefinder applications, such as image aquisition. 
	 * @param type The type of rangefinder to use. They are as follows: 
	 * <ul><li> 0: An infinitely narrow beam that is only stopped by permanent surfaces that will stop the player (usually just walls, floors, ceilings, and moving platforms, never other quagents or rockets or powerups). </li>
	 * <li> 1: An infinitely narrow beam that is blocked by anything that would block player movement (like mode-0, but plus other things like quagents). </li>
	 * <li> 2: An infinitely narrow beam that is blocked by anything that would block a shot (often equivalent to mode 1, but on some levels (like suspended), there are walls/floors that can be shot through but not walked through). </li>
	 * <li> 3: An infinitely narrow beam that is blocked by anything in the environment (like mode-1, but plus powerups or other things that can be walked through). </li>
	 * <li> 4: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-0 rangefinder. </li>  
	 * <li> 5: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-1 rangefinder. </li>  
	 * <li> 6: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-2 rangefinder. </li>  
	 * <li> 7: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-3 rangefinder. </li>
	 * </ul>
	 * @param distance The maximum distance of the rangefinder. So far, the maximum distance we have seen in Quake has been 8120, so a value of 9999 should act as infinite for all intents and purposes.
	 * @param rot The rotational angle (in degrees) left of the Quagent's facing to start firing the rangefinder.
	 * @param azi The azimuthal angle (in degrees) down from the Quagent's facing to start firing the rangefinder.
	 * @param width Number of YAW-variant samples.
	 * @param height Number of PITCH-variant samples.
	 * @param sampleDeltaYaw How far apart two YAW-variant samples should be.
	 * @param sampleDeltaPitch How far apart two PITCH-variant samples should be.
	 * @reports DONE,REPLACED,POPPED,FORGOTTEN
	 * @data n X1 E1 X2 E2 X3 E3... Xn En where n is the number of samples (width*height) in the raster. Each Xi is the range at which the rangefinder stopped, and each corresponding Ei is the entityid of the entity that stopped it.
	 * @return BatchRangefinder command object which represents this query.
	 */
	public BatchRangefinder batchRangefinder(int type, int distance, double rot, double azi, int width, int height, double sampleDeltaYaw, double sampleDeltaPitch);
	/**
	 * Batch version of rangefinder. Used for intensive rangefinder applications, such as image aquisition. 
	 * @param type The type of rangefinder to use. They are as follows: 
	 * <ul><li> 0: An infinitely narrow beam that is only stopped by permanent surfaces that will stop the player (usually just walls, floors, ceilings, and moving platforms, never other quagents or rockets or powerups). </li>
	 * <li> 1: An infinitely narrow beam that is blocked by anything that would block player movement (like mode-0, but plus other things like quagents). </li>
	 * <li> 2: An infinitely narrow beam that is blocked by anything that would block a shot (often equivalent to mode 1, but on some levels (like suspended), there are walls/floors that can be shot through but not walked through). </li>
	 * <li> 3: An infinitely narrow beam that is blocked by anything in the environment (like mode-1, but plus powerups or other things that can be walked through). </li>
	 * <li> 4: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-0 rangefinder. </li>  
	 * <li> 5: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-1 rangefinder. </li>  
	 * <li> 6: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-2 rangefinder. </li>  
	 * <li> 7: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-3 rangefinder. </li>
	 * </ul>
	 * @param distance The maximum distance of the rangefinder. So far, the maximum distance we have seen in Quake has been 8120, so a value of 9999 should act as infinite for all intents and purposes.
	 * @param rot The rotational angle (in degrees) left of the Quagent's facing to start firing the rangefinder.
	 * @param azi The azimuthal angle (in degrees) down from the Quagent's facing to start firing the rangefinder.
	 * @param width Number of YAW-variant samples.
	 * @param height Number of PITCH-variant samples.
	 * @param sampleDeltaYaw How far apart two YAW-variant samples should be.
	 * @param sampleDeltaPitch How far apart two PITCH-variant samples should be.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,REPLACED,POPPED,FORGOTTEN
	 * @data n X1 E1 X2 E2 X3 E3... Xn En where n is the number of samples (width*height) in the raster. Each Xi is the range at which the rangefinder stopped, and each corresponding Ei is the entityid of the entity that stopped it.
	 * @return BatchRangefinder command object which represents this query.
	 */
	public BatchRangefinder batchRangefinder(int type, int distance, double rot, double azi, int width, int height, double sampleDeltaYaw, double sampleDeltaPitch, long period);
	
	/**
	 * Batch version of rangefinder. Used for intensive rangefinder applications, such as image aquisition. 
	 * @param type The type of rangefinder to use. They are as follows: 
	 * <ul><li> 0: An infinitely narrow beam that is only stopped by permanent surfaces that will stop the player (usually just walls, floors, ceilings, and moving platforms, never other quagents or rockets or powerups). </li>
	 * <li> 1: An infinitely narrow beam that is blocked by anything that would block player movement (like mode-0, but plus other things like quagents). </li>
	 * <li> 2: An infinitely narrow beam that is blocked by anything that would block a shot (often equivalent to mode 1, but on some levels (like suspended), there are walls/floors that can be shot through but not walked through). </li>
	 * <li> 3: An infinitely narrow beam that is blocked by anything in the environment (like mode-1, but plus powerups or other things that can be walked through). </li>
	 * <li> 4: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-0 rangefinder. </li>  
	 * <li> 5: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-1 rangefinder. </li>  
	 * <li> 6: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-2 rangefinder. </li>  
	 * <li> 7: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-3 rangefinder. </li>
	 * </ul>
	 * @param rot The rotational angle (in degrees) left of the Quagent's facing to start firing the rangefinder.
	 * @param azi The azimuthal angle (in degrees) down from the Quagent's facing to start firing the rangefinder.
	 * @param width Number of YAW-variant samples.
	 * @param height Number of PITCH-variant samples.
	 * @param sampleDeltaYaw How far apart two YAW-variant samples should be.
	 * @param sampleDeltaPitch How far apart two PITCH-variant samples should be.
	 * @param priority The command queue priority.
	 * @reports DONE,REPLACED,POPPED,FORGOTTEN
	 * @data n X1 E1 X2 E2 X3 E3... Xn En where n is the number of samples (width*height) in the raster. Each Xi is the range at which the rangefinder stopped, and each corresponding Ei is the entityid of the entity that stopped it.
	 * @return BatchRangefinder command object which represents this query.
	 */
	public BatchRangefinder batchRangefinder(int type, double rot, double azi, int width, int height, double sampleDeltaYaw, double sampleDeltaPitch, priority_t priority);
	/**
	 * Batch version of rangefinder. Used for intensive rangefinder applications, such as image aquisition. 
	 * @param type The type of rangefinder to use. They are as follows: 
	 * <ul><li> 0: An infinitely narrow beam that is only stopped by permanent surfaces that will stop the player (usually just walls, floors, ceilings, and moving platforms, never other quagents or rockets or powerups). </li>
	 * <li> 1: An infinitely narrow beam that is blocked by anything that would block player movement (like mode-0, but plus other things like quagents). </li>
	 * <li> 2: An infinitely narrow beam that is blocked by anything that would block a shot (often equivalent to mode 1, but on some levels (like suspended), there are walls/floors that can be shot through but not walked through). </li>
	 * <li> 3: An infinitely narrow beam that is blocked by anything in the environment (like mode-1, but plus powerups or other things that can be walked through). </li>
	 * <li> 4: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-0 rangefinder. </li>  
	 * <li> 5: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-1 rangefinder. </li>  
	 * <li> 6: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-2 rangefinder. </li>  
	 * <li> 7: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-3 rangefinder. </li>
	 * </ul>
	 * @param rot The rotational angle (in degrees) left of the Quagent's facing to start firing the rangefinder.
	 * @param azi The azimuthal angle (in degrees) down from the Quagent's facing to start firing the rangefinder.
	 * @param width Number of YAW-variant samples.
	 * @param height Number of PITCH-variant samples.
	 * @param sampleDeltaYaw How far apart two YAW-variant samples should be.
	 * @param sampleDeltaPitch How far apart two PITCH-variant samples should be.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,REPLACED,POPPED,FORGOTTEN
	 * @data n X1 E1 X2 E2 X3 E3... Xn En where n is the number of samples (width*height) in the raster. Each Xi is the range at which the rangefinder stopped, and each corresponding Ei is the entityid of the entity that stopped it.
	 * @return BatchRangefinder command object which represents this query.
	 */
	public BatchRangefinder batchRangefinder(int type, double rot, double azi, int width, int height, double sampleDeltaYaw, double sampleDeltaPitch, priority_t priority, long period);
	
	/**
	 * Batch version of rangefinder. Used for intensive rangefinder applications, such as image aquisition. 
	 * @param type The type of rangefinder to use. They are as follows: 
	 * <ul><li> 0: An infinitely narrow beam that is only stopped by permanent surfaces that will stop the player (usually just walls, floors, ceilings, and moving platforms, never other quagents or rockets or powerups). </li>
	 * <li> 1: An infinitely narrow beam that is blocked by anything that would block player movement (like mode-0, but plus other things like quagents). </li>
	 * <li> 2: An infinitely narrow beam that is blocked by anything that would block a shot (often equivalent to mode 1, but on some levels (like suspended), there are walls/floors that can be shot through but not walked through). </li>
	 * <li> 3: An infinitely narrow beam that is blocked by anything in the environment (like mode-1, but plus powerups or other things that can be walked through). </li>
	 * <li> 4: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-0 rangefinder. </li>  
	 * <li> 5: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-1 rangefinder. </li>  
	 * <li> 6: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-2 rangefinder. </li>  
	 * <li> 7: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-3 rangefinder. </li>
	 * </ul>
	 * @param rot The rotational angle (in degrees) left of the Quagent's facing to start firing the rangefinder.
	 * @param azi The azimuthal angle (in degrees) down from the Quagent's facing to start firing the rangefinder.
	 * @param width Number of YAW-variant samples.
	 * @param height Number of PITCH-variant samples.
	 * @param sampleDeltaYaw How far apart two YAW-variant samples should be.
	 * @param sampleDeltaPitch How far apart two PITCH-variant samples should be.
	 * @reports DONE,REPLACED,POPPED,FORGOTTEN
	 * @data Matrix of entity-distance pairs for all samples taken. NOTE: this may be very large.
	 * @return BatchRangefinder command object which represents this query.
	 */
	public BatchRangefinder batchRangefinder(int type, double rot, double azi, int width, int height, double sampleDeltaYaw, double sampleDeltaPitch);	
	/**
	 * Batch version of rangefinder. Used for intensive rangefinder applications, such as image aquisition. 
	 * @param type The type of rangefinder to use. They are as follows: 
	 * <ul><li> 0: An infinitely narrow beam that is only stopped by permanent surfaces that will stop the player (usually just walls, floors, ceilings, and moving platforms, never other quagents or rockets or powerups). </li>
	 * <li> 1: An infinitely narrow beam that is blocked by anything that would block player movement (like mode-0, but plus other things like quagents). </li>
	 * <li> 2: An infinitely narrow beam that is blocked by anything that would block a shot (often equivalent to mode 1, but on some levels (like suspended), there are walls/floors that can be shot through but not walked through). </li>
	 * <li> 3: An infinitely narrow beam that is blocked by anything in the environment (like mode-1, but plus powerups or other things that can be walked through). </li>
	 * <li> 4: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-0 rangefinder. </li>  
	 * <li> 5: A wide beam the size of a Quagent that is blocked if any part of it is blocked by something that would block a mode-1 rangefinder. </li>  
	 * <li> 6: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-2 rangefinder. </li>  
	 * <li> 7: a wide beam the size of a quagent that is blocked if any part of it is blocked by something that would block a mode-3 rangefinder. </li>
	 * </ul>
	 * @param rot The rotational angle (in degrees) left of the Quagent's facing to start firing the rangefinder.
	 * @param azi The azimuthal angle (in degrees) down from the Quagent's facing to start firing the rangefinder.
	 * @param width Number of YAW-variant samples.
	 * @param height Number of PITCH-variant samples.
	 * @param sampleDeltaYaw How far apart two YAW-variant samples should be.
	 * @param sampleDeltaPitch How far apart two PITCH-variant samples should be.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,REPLACED,POPPED,FORGOTTEN
	 * @data Matrix of entity-distance pairs for all samples taken. NOTE: this may be very large.
	 * @return BatchRangefinder command object which represents this query.
	 */
	public BatchRangefinder batchRangefinder(int type, double rot, double azi, int width, int height, double sampleDeltaYaw, double sampleDeltaPitch, long period);	

	
	//---------------------radar---------------------------
	/**
	 * This command responds with information on all entities in sight within a specified range.
	 * @param range The maximum distance to look for entities.
	 * @param priority The command queue priority. 
	 * @reports DONE,POPPED,FORGOTTED,REPLACED
	 * @data Entity ID, corresponding type, corresponding distance, relative yaw and pitch to entity from Quagent's current facing.
	 * @return Radar command object which represents this query. 
	 */
	public Radar radar(double range, priority_t priority);
	/**
	 * This command responds with information on all entities in sight within a specified range.
	 * @param range The maximum distance to look for entities.
	 * @param priority The command queue priority. 
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTED,REPLACED
	 * @data Entity ID, corresponding type, corresponding distance, relative yaw and pitch to entity from Quagent's current facing.
	 * @return Radar command object which represents this query. 
	 */
	public Radar radar(double range, priority_t priority, long period);
	
	/**
	 * This command responds with information on all entities in sight within a specified range.
	 * @param range The maximum distance to look for entities.
	 * @reports DONE,POPPED,FORGOTTED,REPLACED
	 * @data Entity ID, corresponding type, corresponding distance, relative yaw and pitch to entity from Quagent's current facing.
	 * @return Radar command object which represents this query. 
	 */
	public Radar radar(double range);	
	/**
	 * This command responds with information on all entities in sight within a specified range.
	 * @param range The maximum distance to look for entities.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTED,REPLACED
	 * @data Entity ID, corresponding type, corresponding distance, relative yaw and pitch to entity from Quagent's current facing.
	 * @return Radar command object which represents this query. 
	 */
	public Radar radar(double range, long period);
	
	/**
	 * This command responds with information on all entities in sight within an infinite range.
	 * @param priority The command queue priority. 
	 * @reports DONE,POPPED,FORGOTTED,REPLACED
	 * @data Entity ID, corresponding type, corresponding distance, relative yaw and pitch to entity from Quagent's current facing.
	 * @return Radar command object which represents this query. 
	 */
	public Radar radar(priority_t priority);
	/**
	 * This command responds with information on all entities in sight within an infinite range.
	 * @param priority The command queue priority. 
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTED,REPLACED
	 * @data Entity ID, corresponding type, corresponding distance, relative yaw and pitch to entity from Quagent's current facing.
	 * @return Radar command object which represents this query. 
	 */
	public Radar radar(priority_t priority, long period);
	
	/**
	 * This command responds with information on all entities in sight within a specified range.
	 * @reports DONE,POPPED,FORGOTTED,REPLACED
	 * @data Entity ID, corresponding type, corresponding distance, relative yaw and pitch to entity from Quagent's current facing.
	 * @return Radar command object which represents this query. 
	 */
	public Radar radar() throws IOException;
	/**
	 * This command responds with information on all entities in sight within a specified range.
	 * @param period The period at which to repeat the action.
	 * @reports DONE,POPPED,FORGOTTED,REPLACED
	 * @data Entity ID, corresponding type, corresponding distance, relative yaw and pitch to entity from Quagent's current facing.
	 * @return Radar command object which represents this query. 
	 */
	public Radar radar(long period) throws IOException;
	
	
	//-----------------whatIs-----------------------------
	/**
	 * Queries the server for the classtype of a given entity id.
	 * @param entityid Entity id to be classified.
	 * @param priority The command queue priority. 
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The classname of the entity, else <i>wall</i> or <i>nothing</i>.
	 * @return WhatIs command object which represents this query. 
	 */
	public WhatIs whatIs(int entityid, priority_t priority);
	/**
	 * Queries the server for the classtype of a given entity id.
	 * @param entityid Entity id to be classified.
	 * @param priority The command queue priority.
	 * @param period The period at which to repeat the action. 
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The classname of the entity, else <i>wall</i> or <i>nothing</i>.
	 * @return WhatIs command object which represents this query. 
	 */
	public WhatIs whatIs(int entityid, priority_t priority, long period);
	
	/**
	 * Queries the server for the classtype of a given entity id.
	 * @param entityid Entity id to be classified.
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The classname of the entity, else <i>wall</i> or <i>nothing</i>.
	 * @return WhatIs command object which represents this query. 
	 */
	public WhatIs whatIs(int entityid);
	/**
	 * Queries the server for the classtype of a given entity id.	
	 * @param entityid Entity id to be classified.
	 * @param period The period at which to repeat the action. 
	 * @reports DONE,POPPED,FORGOTTEN,REPLACED
	 * @data The classname of the entity, else <i>wall</i> or <i>nothing</i>.
	 * @return WhatIs command object which represents this query. 
	 */
	public WhatIs whatIs(int entityid, long period);
	
	
	
	//P1 functions =====================================================================================================================================


	
	//---------findEntityDirection----------------
	/**
	 * Returns the direction to the specified entity.
	 * @param entityid The entity to locate.
	 * @param priority The command queue priority.
	 * @return {Yaw to target, Pitch to target}. Both quantities are zero if entity cannot be seen.
	 */
//	public double[] findEntityDirection(int entityid, priority_t priority);
	/**
	 * Returns the direction to the specified entity.
	 * @param entityid The entity to locate.
	 * @return {Yaw to target, Pitch to target}. Both quantities are zero if entity is not in line of sight.
	 */
//	public double[] findEntityDirection(int entityid);
	
	
	//---------findEntityDistance------------------
	/**
	 * Returns the distance to the specified entity.
	 * @param entityid The entity to locate.
	 * @param priority The command queue priority.
	 * @return Distance to target. Returns 0 if entity is not in line of sight. 
	 */
//	public double findEntityDistance(int entityid, priority_t priority);
	/**
	 * Returns the distance to the specified entity.
	 * @param entityid The entity to locate.
	 * @return Distance to target. Returns 0 if entity is not in line of sight. 
	 */
//	public double findEntityDistance(int entityid);
	
	
	//---------------faceEntityDirection----------------
	/**
	 * Locates and rotates to face the specified entity.
	 * @param entityid The entity to face.
	 * @param priority The command queue priority.
	 */
	//public void faceEntityDirection(int entityid, priority_t priority);
	/**
	 * Locates and rotates to face the specified entity.
	 * @param entityid The entity to face.
	 */
	//public void faceEntityDirection(int entityid);
	//public void gotoEntity(int entityid, double speed, int obstacles, priority_t priority) throws IOException;
	//public void showImage(priority_t priority) throws IOException;
	
	
	//----------------------follow---------------------------
	/**
	 * Instructs the Quagent to follow the specified entity.
	 * @param entityid Entity to follow.
	 * @param priority The command queue priority.
	 * @return 
	 */
	public Follow follow(int entityid,priority_t priority);
	/**
	 * Instructs the Quagent to follow the specified entity.
	 * @param entityid Entity to follow.
	 */
	public Follow follow(int entityid);
	/**
	 * Instructs the Quagent to follow the specified entity for the specified duration.
	 * @param entityid Entity to follow.
	 * @param duration Duration to follow for.
	 * @param priority The command queue priority.
	 */
	public Follow follow(int entityid,long duration, priority_t priority);
	/**
	 * Instructs the Quagent to follow the specified entity for the specified duration.
	 * @param entityid Entity to follow.
	 * @param duration Duration to follow for.
	 */
	public Follow follow(int entityid, long duration);
	
	/**
	 * Moves to <i>object</i> located within <i>radius</i> and line of sight.
	 * @param object Object to move to.
	 * @param radius Radius to search.
	 * @param priority The command queue priority.
	 */
	public void moveToObject(String object, int radius, priority_t priority);
	/**
	 * Moves to <i>object</i> located within <i>radius</i> and line of sight.
	 * @param object Object to move to.
	 * @param radius Radius to search.
	 */
	public void moveToObject(String object, int radius);
	/**
	 * Moves to <i>object</i> within line of sight.
	 * @param object Object to move to.
	 * @param priority The command queue priority.
	 */
	public void moveToObject(String object, priority_t priority);
	/**
	 * Moves to <i>object</i> within line of sight.
	 * @param object Object to move to.
	 */
	public void moveToObject(String object);
	
	/**
	 * Follow specified entity.
	 * @param entityid Entity to follow.
	 * @param speed Speed to follow at.
	 * @param obstacles 1 if quagent should auto-avoid obstacles, else 0.
	 * @param duration Duration of following.
	 * @param priority Command queue priority.
	 * @deprecated
	 */
//	public void followEntity(int entityid, double speed, int obstacles, long duration, priority_t priority);
//	public void maintainDistanceFrom(int entityid, double distbuffer, double speed, int obstacles, long duration, priority_t priority) throws IOException;
	public void resume();
}
