package edu.rochester.cs.quagents3.client;
import java.io.Writer;

/**
 * Class which represents a Crouch action.
 */
public class Crouch extends ProtocolZero {
	private static final String opcode = "sc";
	
	private final boolean crouch;
	
	public Crouch(boolean crouch, priority_t priority){
		super(false,CommandType.CROUCH,priority);
		this.crouch = crouch;
	}
	public Crouch(boolean crouch){
		super(false,CommandType.CROUCH);
		this.crouch = crouch;
	}
	public Crouch(boolean crouch, priority_t priority,
			long period){
		super(false,CommandType.CROUCH,period,priority);
		this.crouch = crouch;
	}
	public Crouch(boolean crouch,
			long period){
		super(false,CommandType.CROUCH,period);
		this.crouch = crouch;
	}
	public Crouch(boolean crouch, priority_t priority,
			long period,long delay){
		super(false,CommandType.CROUCH,period,delay,priority);
		this.crouch = crouch;
	}
	public Crouch(boolean crouch,
			long period,long delay){
		super(false,CommandType.CROUCH,period,delay);
		this.crouch = crouch;
	}
	
	/**
	 * Returns the effect of this command.
	 * @return Quagent's crouch state.
	 * @usage May be used to determined if Quagent is crouched. For example, search through client.comhist for most recent
	 * Crouch and call this to see if Quagent is crouched. 
	 */
	public boolean isCrouched(){
		return crouch;
	}
	
	public void setData(String params) {
		System.err.println("error: Crouch does not facilitate a data response");
	}

	/**
	 * Protocol helper function. 
	 * @param crouch
	 * @return Data formatted for transmission to server.
	 */
	private static int convertCrouch(boolean crouch){
		if(crouch)return 1;
		else return 0;
	}
	
	protected void setOp(){
		op = Client.convertPriority(priority) + " " + opcode + " " + id + " " + convertCrouch(crouch);
	}
}
