package edu.rochester.cs.quagents3.client;

import java.awt.Dimension;
import java.awt.Panel;
import java.awt.image.*;
import java.io.File;

import javax.swing.JFrame;

public class DepthViewpoint extends ProtocolZero{
	private static final String opcode = "vd";
	public static final int BYTESPERPIXEL = 4;
	private static int pixnum = 0; //so we can have multiple filse
	
	private int width;
	private int height;
	private byte[] bytes;
	
	public DepthViewpoint(priority_t priority){
		super(true, CommandType.DEPTHVIEWPOINT, priority);
	}
	
	public DepthViewpoint(){
		super(true, CommandType.DEPTHVIEWPOINT);
	}
	
	public DepthViewpoint(long period){
		super(true, CommandType.DEPTHVIEWPOINT, period);
	}
	
	public DepthViewpoint(long period, priority_t priority){
		super(true, CommandType.DEPTHVIEWPOINT, period, priority);
	}
	
	protected void setOp(){
		op = Client.convertPriority(priority) + " " + opcode + " " + id;
	}
	
	public int getWidth(){
		return width;
	}
	
	public int getHeight(){
		return height;
	}
	
	public byte[] getEncodedBytestream(){
		return bytes;
	}
	
	public void setData(String params){} //not used - special case
	
	public void setData(int width, int height, byte[] bytes){
		this.width = width;
		this.height = height;
		this.bytes = bytes;
		QView2png();
	}
	
	private void QView2png(){
		String fname;
		while(true){
			fname = "screenshots/depthviewpoint" + pixnum + ".png";
			File f = new File(fname);
			if(!f.exists())
				break;
			else
				pixnum++;
		}
		try{
		BufferedImage b=new BufferedImage(width,height,BufferedImage.TYPE_INT_RGB);
		WritableRaster wr=b.getRaster();
		int min=0x7fffffff;
	    int max=0;
	    for(int i=0;i<height;i++){
	     for(int j=0;j<width;j++){
	      int b0=bytes[(i*width+j)*4+3];
	      int b1=bytes[(i*width+j)*4+2];
	      int b2=bytes[(i*width+j)*4+1];
	      int b3=bytes[(i*width+j)*4+0];
	      b0=(b0<<24)&0x7f000000;
	      b1=(b1<<16)&0xff0000;
	      b2=(b2<<8)&0xff00;
	      int col=b0+b1+b2+b3;
	      if(col>max) max=col;
	      if(col<min) min=col;
	     }
	    }
		System.out.println("Min: "+min);
		System.out.println("Max: "+max);
	    for(int i=0;i<height;i++){
	     for(int j=0;j<width;j++){
	      int b0=bytes[(i*width+j)*4+3];
	      int b1=bytes[(i*width+j)*4+2];
	      int b2=bytes[(i*width+j)*4+1];
	      int b3=bytes[(i*width+j)*4+0];
	      b0=(b0<<24)&0x7f000000;
	      b1=(b1<<16)&0xff0000;
	      b2=(b2<<8)&0xff00;
	      double col=(b0+b1+b2+b3-min)/(double)(max-min);
	      int[] color=new int[3];
	      color[0]=(int)(255*col);
	      color[1]=color[0];
	      color[2]=color[0];
	      wr.setPixel(j,height-i-1,color);
	     }
	    }
	   b.setData(wr);
	   System.out.println("Saving image...");
	   javax.imageio.ImageIO.write(b,"png",new java.io.File(fname));
	   draw(b, width, height);
	   System.out.println("File revision complete.");
	  }catch(Exception e){
	   System.out.println("Fatal Exception:");
	   e.printStackTrace();
	   System.out.println("abort");
	  }
	}
	
	public void draw(BufferedImage b, int width, int height) {
		JFrame frame = new JFrame("Depth Viewpoint");
		Panel panel = new QImage(b);
		panel.setPreferredSize(new Dimension(width, height));
		frame.getContentPane().add(panel);
		frame.pack();
		frame.setVisible(true);
	}
}
