package edu.rochester.cs.quagents3.client;

import java.io.Writer;
import java.util.List;

/**
 * An Entity represents a nontrivial existence in the Quake world. This is the essential interface
 * needed for any Quagent or GOD in Quagents 3.
 */
public interface Entity {
	//entity functions
	/**
	 * Returns the entity's socket writer. This should be used judiciously, as all client-server
	 * interaction is handled automatically internally.
	 * @return Entity's socket writer.
	 */
	public Writer getWriter();
	
	/**
	 * Blocks until <i>com</i> has terminated.
	 * @param com Command to block on.
	 */
	public void waitForTerminate(Command com);
	
	/**
	 * Executes command and blocks until server returns termination response.
	 * This function is equivalent to calling <i>start()</i> and <i>waitForTerminate()</i> in sequence.
	 * Functions <strong>must</strong> be executed through this or <i>start()</i> in some form, as these functions
	 * also contain necessary internal instrumentation.
	 * 
	 * @param com Command to execute and block on.
	 */
	public void execute(Command com);
	
	/**
	 * Sends command to server to execute, and returns. Functions <strong>must</strong> be executed through this or <i>execute()</i> in some form, as these functions
	 * also contain necessary internal instrumentation.
	 * @param com Command to execute.
	 */
	public void start(Command com);
	
	/**
	 * Adds a command termination listener to the Quagent. This determines general reaction to any command returned.
	 * Attaching command termiantion listeners to individual commands may be more useful then this - see Command class.
	 * @param comlist CommandTermiantionListener to attach
	 */
	public void addEventListener(CommandTerminationListener comlist);
	
	//used internally
	public long incrementCommandID();
	public void removeCommandFromRunning(ProtocolZero com);
	public void addCommandToRunning(ProtocolZero com);
	public void notifyCommandTerminationListener(Command com);
	public List<ProtocolZero> getRunningList();
	public boolean isQuagent(); //inelegant...but necessary for event processing
}
