package edu.rochester.cs.quagents3.client;

/**
 * Function which represents a <i>Follow<i> action.
 * This is a compound action which orders the Quagent to follow
 * the specified entity.
 * It might not be wise to block on this...
 */
public class Follow extends ProtocolOne {
	
	private final long duration; //0 implies indef duration
	private final int entityid;
	private final double speed;
	private final int obstacles;
	
	private CanSee detector;
	private CurrentLocation gps;
	private Facing currentfacing;
	private static final long detectorpd = 300;
	
	public Follow(int entityid, double speed, int obstacles, long duration){
		super(CommandType.FOLLOW);
		this.duration = duration;
		this.entityid = entityid;
		this.speed = speed;
		this.obstacles = obstacles;
		detector = new CanSee(entityid,priority_t.NOW,detectorpd);
		currentfacing = new Facing(priority_t.NOW,detectorpd/2);
		gps = new CurrentLocation(priority_t.NOW,detectorpd/2);
		detector.addTerminationListener(new DetectorListener());
	}
	public Follow(int entityid, long duration){
		super(CommandType.FOLLOW);
		this.duration = duration;
		this.entityid = entityid;
		this.speed = Client.DEFAULTSPEED;
		this.obstacles = 0; //default: no auto obstacle completion
		detector = new CanSee(entityid,priority_t.NOW,detectorpd);
		currentfacing = new Facing(priority_t.NOW,detectorpd/2);
		gps = new CurrentLocation(priority_t.NOW,detectorpd/2);
		detector.addTerminationListener(new DetectorListener());
	}
	public Follow(int entityid){
		super(CommandType.FOLLOW);
		this.duration = 0;
		this.entityid = entityid;
		this.speed = Client.DEFAULTSPEED;
		this.obstacles = 0; //default: no auto obstacle completion
		detector = new CanSee(entityid,priority_t.NOW,detectorpd);
		currentfacing = new Facing(priority_t.NOW,detectorpd/2);
		gps = new CurrentLocation(priority_t.NOW,detectorpd/2);
		detector.addTerminationListener(new DetectorListener());
	}
	
	public long getDuration(){
		return duration;
	}

	public int getEntityID(){
		return entityid;
	}
	
	public double getSpeed(){
		return speed;
	}
	
	public int getObstacles(){
		return obstacles;
	}
	
	public void run() {
		if(paused) return;
		running = true;
		executor.execute(gps);
		executor.execute(currentfacing);
		executor.start(detector);
/*			while(running) { //must be ended by separate thread
				detector.get
=======
		if(!verifyClient()) return; //ensure client is connected to command
		if(duration==0){ //follow indef
			while(true) {
	    		CanSee cs = new CanSee(entityid);
	    		executor.start(cs);
	    		cs.waitForTerminate();
	    		Facing fa = new Facing();
	    		executor.start(fa);
	    		CurrentLocation cr = new CurrentLocation();
	    		cr.setClient(executor);
	    		cr.run();
	    		double quagentyaw = fa.getViewDirection()[0];
	    		double quagentpitch = fa.getViewDirection()[1];
	    		if (cs.canSeeTarget()) {
	    			double angle = cs.getYawToTarget();
	    			if (angle < -180)
	    				angle += 360;
	    			Rotate ro = new Rotate(angle, 0, priority_t.NOW);
	    			executor.start(ro);
	    			if (cs.getDistanceToTarget() > 50) {
	    				double location[] = cr.getCurrentLocation();
	    				double distance = cs.getDistanceToTarget();
	    				distance *= Math.cos((quagentpitch + cs.getPitchToTarget()) / 360 * 2 * Math.PI);
	    				location[0] += (distance - 50) * Math.cos((quagentyaw + cs.getYawToTarget()) / 360 * 2 * Math.PI);
	    				location[1] += (distance - 50) * Math.sin((quagentyaw + cs.getYawToTarget()) / 360 * 2 * Math.PI);
	    				executor.moveTo(location, Client.DEFAULTSPEED, priority_t.REPLACE);
	    			}
	    		}
>>>>>>> .r31
	    	}
		}
		else{ //use duration
			do {
	    	
	    	} while(System.currentTimeMillis()-`inittime<duration);
		}
		terminate();*/
	}
	
	private class DetectorListener implements CommandTerminationListener{
		public void commandTerminated(Command com) {
			if(!running) return;
			if(!detector.canSeeTarget())
				terminate();
		   // executor.rotate(detector.getYawToTarget(), 0, priority_t.NOW);
		    if (detector.getDistanceToTarget() > 50) {
		    	double location[] = gps.getCurrentLocation();
		    	double distance = detector.getDistanceToTarget();
		    	distance *= Math.cos((currentfacing.getViewDirection()[Facing.PITCH] + detector.getPitchToTarget()) / 360 * 2 * Math.PI);
		    	location[0] += (distance - 50) * Math.cos((currentfacing.getViewDirection()[Facing.YAW] + detector.getYawToTarget()) / 360 * 2 * Math.PI);
		    	location[1] += (distance - 50) * Math.sin((currentfacing.getViewDirection()[Facing.YAW] + detector.getYawToTarget()) / 360 * 2 * Math.PI);
		    	((Client)executor).moveTo(location, Client.DEFAULTSPEED,priority_t.REPLACE);
		    }
		}	
	}
	
	@Override
	protected void addChildren() {
		children.add(detector);
		children.add(gps);
		children.add(currentfacing);
	}
	@Override
	public void update() {
		// TODO Auto-generated method stub
		
	}

}
