package edu.rochester.cs.quagents3.client;

import java.net.*;
import java.util.*;
import java.io.*;


public class GOD extends Thread implements GODInf{
	private Socket sock;
	private Writer writer;
	public static int GODPORT = 6002;
	public long commandid = 0;
	public Stack<Command> comhist;
	private ReaderThread rt;
	private CommandTerminationListener comlist;
	private GODEventListener evlist;
	private volatile List<ProtocolZero> running;
	private Updater update;
	
	public static boolean VERBOSE = false;
	
	//GOD has no initial phase, so functions are live after instantiation
	//if connection is validated
	public GOD(String ip) throws UnknownHostException, IOException{
		sock = new Socket(ip, GODPORT);
		writer = new OutputStreamWriter(sock.getOutputStream());
		rt = new ReaderThread(this,sock);
		running = new LinkedList<ProtocolZero>();
		comhist = new Stack<Command>();
		
		addEventListener(new CommandTerminationListener(){
			public void commandTerminated(Command com) {if(VERBOSE)System.out.println(com.toString());}});
		addEventListener(new DefaultGODEventListener());
		rt.start();
	}
	
	public void execute(Command com){
		start(com);
		waitForTerminate(com);	
	}
	
	public void start(Command com){
		com.setExecutor(this);
		comhist.push(com);
		if(!com.periodic)
			com.run();
		else
			update.insert(com);
	}
	
	public void waitForTerminate(Command com){
		synchronized(com){
			while(!com.hasTerminated()){
				try{com.wait();}
				catch(InterruptedException e){}
			}
		}
	}
	
	public boolean isQuagent(){
		return false;
	}
	
	public List<ProtocolZero> getRunningList(){
		return running;
	}

	public Writer getWriter() {
		return writer;
	}
	
	public long incrementCommandID(){
		return ++commandid;
	}
	
	public void removeCommandFromRunning(ProtocolZero com){
		running.remove(com);
	}

	public void addCommandToRunning(ProtocolZero com){
		running.add(com);
	}
	
	public void notifyCommandTerminationListener(Command com){
		comlist.commandTerminated(com);
	}

	public void addEventListener(CommandTerminationListener comlist) {
		this.comlist = comlist;
	}
	
	public void addEventListener(GODEventListener godlist){
		this.evlist = godlist;
	}
	
	public GODEventListener getEventListener(){
		return evlist;
	}
	
	private class DefaultGODEventListener implements GODEventListener{

		public void playerDied(GODEvent event) {
			System.out.println("GOD EVENT: Entity " + event.getID() + " (" + event.getName()
				+ ") died."); System.out.flush();
		}

		public void playerDisconnected(GODEvent event) {
			System.out.println("GOD EVENT: Entity " + event.getID() + " (" + event.getName()
					+ ") disconnected."); System.out.flush();			
		}

		public void playerJoined(GODEvent event) {
			System.out.println("GOD EVENT: Entity " + event.getID() + " (" + event.getName()
					+ ") joined the game.");	System.out.flush();	
		}
		
	}
}
