package edu.rochester.cs.quagents3.client;

import java.util.*;

public class GetPosition extends ProtocolZero{
	private static final String opcode = "gp";
	
	public static final int CLIENTPOSITIONS = -1;
	public static final int ENTITYPOSITIONS = -2;
	public static final int X = 0;
	public static final int Y = 1;
	public static final int Z = 2;
	
	private final int entityid;
	
	//store returns as lists
	private List<Integer> entityids;
	private List<String> types;
	private List<Double[]> coords;
	
	public GetPosition(int entityid){
		super(true,CommandType.GETPOSITION);
		this.entityid = entityid;
		entityids = Collections.synchronizedList(new LinkedList<Integer>());
		types = Collections.synchronizedList(new LinkedList<String>());
		coords = Collections.synchronizedList(new LinkedList<Double[]>());
	}
	
	/**
	 * Returns the first entity in the list's location.
	 * Useful mainly for single entity requests
	 * @return Location of first entity.
	 */
	public Double[] getLocation(){
		return coords.get(0);
	}
	
	/**
	 * Returns the first entity in the list's type.
	 * Useful mainly for single entity requests.
	 * @return Type of first entity;
	 */
	public String getEntityType(){
		return types.get(0);
	}
	
	private int getIndexOfEntity(int entityid){
		return entityids.indexOf(entityid);
	}
	
	public String getType(int entityid){
		return types.get(getIndexOfEntity(entityid));
	}
	
	public Double[] getLocation(int entityid){
		return coords.get(getIndexOfEntity(entityid));
	}

	public void setData(String params) {
		boolean exists = false;
		datastring += params;
		paramscan = new Scanner(params);
		
		int eid = paramscan.nextInt();
		if(!entityids.contains(eid))
			entityids.add(eid);
		else //don't add, we already are tracking this entity
			exists = true;
		
		String type = paramscan.next();
		if(!exists)
			types.add(type);
		else{ //we are already tracking this entity
			  //so just replace old value with new
			int index = entityids.indexOf(eid);
			types.remove(index);
			types.add(index, type);
		}
		
		Double anon[] = {paramscan.nextDouble(), paramscan.nextDouble(),
				paramscan.nextDouble()};
		if(!exists)
			coords.add(anon);
		else{
			int index = entityids.indexOf(eid);
			coords.remove(index);
			coords.add(index,anon);
		}
	}

	protected void setOp() {
		op = opcode + " " + id + " " + entityid;
	}
}
