package edu.rochester.cs.quagents3.client;
import java.io.Writer;
import java.util.List;

/**
 * Class which represents a MoveBy command.
 */
public class MoveBy extends ProtocolZero{
	private static final String opcode = "mb";
	
	//parameters
	private final double dir, dist, speed;
	private final int obstacles;
	
	public MoveBy(double dir, double dist, double speed, int obstacles, priority_t priority){
		super(false,CommandType.MOVEBY,priority);
		this.dir = dir;
		this.dist = dist;
		this.speed = speed;
		this.obstacles = obstacles;
	}
	public MoveBy(double dir, double dist, double speed, int obstacles){
		super(false,CommandType.MOVEBY);
		this.dir = dir;
		this.dist = dist;
		this.speed = speed;
		this.obstacles = obstacles;
	}
	public MoveBy(double dir, double dist, double speed, int obstacles, priority_t priority,long period){
		super(false,CommandType.MOVEBY,period,priority);
		this.dir = dir;
		this.dist = dist;
		this.speed = speed;
		this.obstacles = obstacles;
	}
	public MoveBy(double dir, double dist, double speed, int obstacles,long period){
		super(false,CommandType.MOVEBY,period);
		this.dir = dir;
		this.dist = dist;
		this.speed = speed;
		this.obstacles = obstacles;
	}
	public MoveBy(double dir, double dist, double speed, int obstacles, priority_t priority,long period,long delay){
		super(false,CommandType.MOVEBY,period,delay,priority);
		this.dir = dir;
		this.dist = dist;
		this.speed = speed;
		this.obstacles = obstacles;
	}
	public MoveBy(double dir, double dist, double speed, int obstacles,long period,long delay){
		super(false,CommandType.MOVEBY,period,delay);
		this.dir = dir;
		this.dist = dist;
		this.speed = speed;
		this.obstacles = obstacles;
	}
	
	
	protected void setOp(){
		op = Client.convertPriority(priority)+" " +opcode +" " + id + " "+
		obstacles+" "+dir+" "+speed+" "+dist;
	}
	
	@Override
	public void setData(String params) {
		System.err.println("error: MoveBy has no data response");
	}
	
	/**
	 * Allows access to parameter data.
	 * @return Direction of movement. Degrees left of Quagent's original facing. 
	 */
	public double getDirection(){return dir;}
	
	/**
	 * Allows access to parameter data.
	 * @return Distance of movement requested to server.<br/>
	 * NOTE: This may vary from distance moved. Examine the server exitcode corresponding to this command
	 * (using getExitCode()) to see if this is the case.
	 */
	public double getDistance(){return dist;}
	
	/**
	 * Allows access to parameter data.
	 * @return Speed of movement. 
	 */
	public double getSpeed(){return speed;}
	
	/**
	 * Allows access to parameter data.
	 * @return Obstacles flag. 
	 */
	public int getObstacles(){return obstacles;}
}
