package edu.rochester.cs.quagents3.client;
import java.io.Writer;
import java.util.List;

/**
 * Class that represents a MoveFor command. 
 */
public class MoveFor extends ProtocolZero{
	private static final String opcode = "mf";
	
	private final double dir, time, speed;
	private int obstacles;
	
	public MoveFor(double dir, double time, double speed, int obstacles, priority_t priority){
		super(false,CommandType.MOVEFOR,priority);
		this.dir = dir;
		this.time = time;
		this.speed = speed;
		this.obstacles = obstacles;
	}
	public MoveFor(double dir, double time, double speed, int obstacles){
		super(false,CommandType.MOVEFOR);
		this.dir = dir;
		this.time = time;
		this.speed = speed;
		this.obstacles = obstacles;
	}
	public MoveFor(double dir, double time, double speed, int obstacles, priority_t priority,long period){
		super(false,CommandType.MOVEFOR,period,priority);
		this.dir = dir;
		this.time = time;
		this.speed = speed;
		this.obstacles = obstacles;
	}
	public MoveFor(double dir, double time, double speed, int obstacles,long period){
		super(false,CommandType.MOVEFOR,period);
		this.dir = dir;
		this.time = time;
		this.speed = speed;
		this.obstacles = obstacles;
	}
	public MoveFor(double dir, double time, double speed, int obstacles, priority_t priority,long period,long delay){
		super(false,CommandType.MOVEFOR,period,delay,priority);
		this.dir = dir;
		this.time = time;
		this.speed = speed;
		this.obstacles = obstacles;
	}
	public MoveFor(double dir, double time, double speed, int obstacles,long period,long delay){
		super(false,CommandType.MOVEFOR,period,delay);
		this.dir = dir;
		this.time = time;
		this.speed = speed;
		this.obstacles = obstacles;
	}
	
	
	protected void setOp(){
		op = Client.convertPriority(priority) + " " + opcode + " " + id+
		" " + obstacles + " " + dir +  " " + speed+ " " + time;
	}
	
	@Override
	public void setData(String params) {
		System.err.println("error: MoveFor has no data response");
	}
	
	/**
	 * Allows access to parameter data.
	 * @return Direction of movement. Degrees left of Quagent's original facing. 
	 */
	public double getDirection(){return dir;}
	
	/**
	 * Allows access to parameter data.
	 * @return Duration of movement requested to server.
	 */
	public double getDuration(){return time;}
	
	/**
	 * Allows access to parameter data.
	 * @return Speed of movement. 
	 */
	public double getSpeed(){return speed;}
	
	/**
	 * Allows access to parameter data.
	 * @return Obstacles flag. 
	 */
	public int getObstacles(){return obstacles;}
	
}
