package edu.rochester.cs.quagents3.client;
import java.io.Writer;
import java.util.List;

/**
 * Class which represents a MoveTo command.
 */
public class MoveTo extends ProtocolZero{
	private static final String opcode = "mt";
	
	private final double[] point;
	private final double speed;
	private final int obstacles;
	
	public MoveTo(double[] point, double speed, int obstacles, priority_t priority){
		super(false,CommandType.MOVETO,priority);
		this.point = point;
		this.speed = speed;
		this.obstacles = obstacles;
	}
	public MoveTo(double[] point, double speed, int obstacles){
		super(false,CommandType.MOVETO);
		this.point = point;
		this.speed = speed;
		this.obstacles = obstacles;
	}
	public MoveTo(double[] point, double speed, int obstacles, priority_t priority,long period){
		super(false,CommandType.MOVETO,period,priority);
		this.point = point;
		this.speed = speed;
		this.obstacles = obstacles;
	}
	public MoveTo(double[] point, double speed, int obstacles,long period){
		super(false,CommandType.MOVETO,period);
		this.point = point;
		this.speed = speed;
		this.obstacles = obstacles;
	}
	public MoveTo(double[] point, double speed, int obstacles, priority_t priority,long period,long delay){
		super(false,CommandType.MOVETO,period,delay,priority);
		this.point = point;
		this.speed = speed;
		this.obstacles = obstacles;
	}
	public MoveTo(double[] point, double speed, int obstacles,long period,long delay){
		super(false,CommandType.MOVETO,period,delay);
		this.point = point;
		this.speed = speed;
		this.obstacles = obstacles;
	}
	@Override
	public void setData(String params) {
		System.err.println("error: MoveTo has no data response");
	}
	
	/**
	 * Allows access to parameter data.
	 * @return Specified destination in [x,y] form.
	 */
	public double[] getGotoLoc(){return point;}
	
	/**
	 * Allows access to parameter data.
	 * @return Speed of movement. 
	 */
	public double getSpeed(){return speed;}
	
	/**
	 * Allows access to parameter data.
	 * @return Obstacles flag. 
	 */
	public int getObstacles(){return obstacles;}
	
	protected void setOp(){
		op = Client.convertPriority(priority) + " " + opcode + " " + id+
		" " + obstacles + " " + speed + " " + point[0] + " " + point[1];
	}
}
