package edu.rochester.cs.quagents3.client;
import java.io.Writer;
import java.util.List;
import java.util.Scanner;

/**
 * Class which represents a Peek CQM. 
 */
public class Peek extends ProtocolZero{
	private static final String opcode = "pk";
	
	//for return
	private String opstring;
	private long numqueuecommands; 
	private String pkopcode;
	private long pkid;
	
	//init param
	private final int depth;
	
	public Peek(int depth, priority_t priority){
		super(true,CommandType.PEEK,priority);
		this.depth = depth;
	}
	public Peek(int depth){
		super(true,CommandType.PEEK);
		this.depth = depth;
	}
	public Peek(int depth, priority_t priority,long period){
		super(true,CommandType.PEEK,period,priority);
		this.depth = depth;
	}
	public Peek(int depth,long period){
		super(true,CommandType.PEEK,period);
		this.depth = depth;
	}
	public Peek(int depth, priority_t priority,long period,long delay){
		super(true,CommandType.PEEK,period,delay,priority);
		this.depth = depth;
	}
	public Peek(int depth,long period,long delay){
		super(true,CommandType.PEEK,period,delay);
		this.depth = depth;
	}

	public void setData(String params) {
		datastring = params;
		paramscan = new Scanner(params);
		numqueuecommands = paramscan.nextLong();
		opstring = paramscan.nextLine();
		paramscan = new Scanner(opstring);
		pkopcode = paramscan.next();
		pkid = paramscan.nextLong();
	}
	
	/**
	 * Fetches entire protocol instruction corresponding to examined command. 
	 * @return Protocol instruction for specified command.
	 */
	public String getOpString(){
		return opstring;}
	
	/**
	 * Returns total number of commands in the command queue. 
	 * @return Total number of commands in the server command queue.
	 */
	public long getNumQueueCommands(){
		return numqueuecommands;}
	
	/**
	 * Returns Opcode of examined command.
	 * @return Opcode of examined command.
	 */
	public String getOpcode(){
		return pkopcode;}
	
	/**
	 * Returns command ID of examined command.
	 * @return command ID of examined command.
	 */
	public long getID(){
		return pkid;}
	
	/**
	 * Allows access to parameter data.
	 * @return Command depth.
	 */
	public int getDepth(){return depth;}
	
	protected void setOp(){
		op = Client.convertPriority(priority) + " " + opcode + " " + id + " " + depth;
	}
}
