package edu.rochester.cs.quagents3.client;
import java.io.IOException;
import java.util.Scanner;
/**
 * Superclass for all ProtocolZero commands. These are directly mapped to server commands. <br>
 * <b>NOTE: Currently implemented functions should not be modified unless corresponding server
 * modifications are implemented.</b>
 */
public abstract class ProtocolZero extends Command{
	protected final priority_t priority;
	protected boolean formatted = true; //for complex outputs
	protected Scanner paramscan;

	protected ProtocolZero(boolean isread,CommandType whatami,priority_t priority){
		super(isread,whatami);
		this.priority = priority;
	}
	protected ProtocolZero(boolean isread,CommandType whatami){
		super(isread,whatami);
		this.priority = Client.DEFAULTPRIORITY;
	}
	
	protected ProtocolZero(boolean isread,CommandType whatami, long period,priority_t priority){
		super(isread,whatami,period);
		this.priority = priority;
	}
	protected ProtocolZero(boolean isread,CommandType whatami, long period){
		super(isread,whatami,period);
		this.priority = Client.DEFAULTPRIORITY;
	}
	
	protected ProtocolZero(boolean isread,CommandType whatami, long period, long delay,priority_t priority){
		super(isread,whatami,period,delay);
		this.priority = priority;
	}
	protected ProtocolZero(boolean isread,CommandType whatami, long period, long delay){
		super(isread,whatami,period,delay);
		this.priority = Client.DEFAULTPRIORITY;
	}
	
	//MUST attach action to agent before calling run().
	public void run(){
		if(paused) return;
		if(!verifyClient()) return;
		executor.addCommandToRunning(this);
		datastring = "";
		formatted=false;
		running=true;
		inittime = System.currentTimeMillis();
		if(Client.VERBOSE){
			System.out.println(this.toString());
			System.out.println("Writing: \"" + op + "\"");
		}
		try {executor.getWriter().write(op+"\n");executor.getWriter().flush();} 
		catch (IOException e) {e.printStackTrace();} 
	}
	
	/**
	 * Allows access to command queue priority.
	 * @return This command's command queue priority.
	 */
	public priority_t getPriority(){
		return priority;
	}
	
	abstract protected void setOp();
	
	public void update(){
		if(running) waitForTerminate();
		else{run(); waitForTerminate();}
	}
	
	public void terminate(String exitstring){
		executor.removeCommandFromRunning(this);
		finaltime = System.currentTimeMillis()-inittime;
		running=false;
		ecode=exitstring.toUpperCase();
		comlist.commandTerminated(this);
		executor.notifyCommandTerminationListener(this);
	}
	
	public boolean hasTerminated(){return !running;}

	/**
	 * Sets command response data. This is overloaded for each command type, so the reader
	 * needs only to send the command (which is identified with the command id)
	 * the data string and all parsing and data storage will occur automatically. 
	 * @param params Data string. For all "data" messages it is identified by the form:
	 * <br/>"rs [OPCODE] [COMMANDID] [DATASTRING]".
	 */
	abstract public void setData(String params);
	
	public void setExecutor(Entity c){

		waitForTerminate(); //client switch is sync
							//guarantee that only returned commands are ones explicitly called by client
		executor = c;
		id = executor.incrementCommandID();
		setOp();
	}
	
	
	
	public void pause(){
		if(periodic) //can't pause atomic commands
			paused = true;
		else
			System.err.println("Can't pause atomic command " + this.toString());
	}
	
	public void resume(){
		if(periodic)
			paused = false;
		else
			System.err.println("Can't resume atomic command " + this.toString());
	}
}
