package edu.rochester.cs.quagents3.client;
import java.util.Date;
import java.util.EventObject;

/**
 * A QEvent represents an event occurrence in the Quake world.
 * Currently implemented events include <ul>
 * <li> Death: If a Quagent dies, an event is sent to that Quagent (and only that Quagent) notifying the client of the occurrence. 
 * <li> Listen: If a Quagent is within range of another Quagent's <i>say()</i> command, this message will be transmitted back to the Quagent's client as an event.
 * </ul>
 */
public class QEvent{

	public static int LISTEN = 0;
	public static int DIED = 1;
	private int eventcode;
	private int originid;
	private Date timestamp;
	private String event;
	
	public QEvent(int eventcode, int originid, String source){
		this.eventcode = eventcode;
		timestamp = new Date();
		event = source;
		this.originid = originid;
	}
	
	public Date getTimestamp(){
		return timestamp;
	}
	
	//returns time since 1970
	public long getTime(){
		return timestamp.getTime();
	}
	
	/**
	 * Returns the type of event.
	 * The static variables LISTEN and DIED
	 * may be used to case these.
	 * @deprecated
	 * @return The type of QEvent.
	 */
	public int getQCode(){
		return eventcode;
	}
	
	public String getEvent(){
		return event;
	}
	
	public int getOrigin(){
		return originid;
	}
}
