package edu.rochester.cs.quagents3.client;

import java.io.IOException;

/**
 * This class provides a skeleton Quagent.
 */
public interface QuagentEntity extends Entity{
	/**
	 * Adds an event listener to the Quagent. This determines event responses.
	 * @param list QEventListener to add.
	 */
	public void addEventListener(QEventListener list);
	
	/**
	 * Returns attached event listener. Used by ReaderThread. External code probably won't need this.
	 * @return Quagent's attached event listener
	 */
	public QEventListener getEventListener();
	
	/**
	 * Sets the Quagent's ID number, as assigned by game engine. Used by ReaderThread.
	 * EXTERNAL CODE SHOULD NOT TOUCH THIS!!
	 * @param id Quagent's Quake entity id.
	 */
	public void setQuagentID(int id);
	
	/**
	 * Returns Quagent's assigned entity ID.
	 * @return Quagent's entity id.
	 */
	public int getQuagentID();
	
	//we need the init commands here too, they aren't covered by start and execute
	/**
	 * This function indicates that that the user has finished setting all
	 * initial parameters and is ready to start the main phase of Quagents.
	 * Calling this function will spawn the Quagent and unlock live commands
	 * @category Initial Commands
	 * @throws IOException
	 */
	public void ready() throws IOException;
	
	/**
	 * This function specifies the Quagent's name. This name will appear
	 * in-game as the bot's name.
	 * @param name The Quagent's desired name
	 * @throws IOException
	 */
	public void name(String name) throws IOException;
	
	/**
	 * This function specifies the Quagent's team.
	 * @see team enumerated type
	 * @param t The Quagent's desired team.
	 * @throws IOException
	 */
	public void team(team t) throws IOException;
	
	/**
	 * This function specifies the botfile for the Quagent. Among other things, 
	 * this determines the bot's in-game model.
	 * @param filename The filename for the botfile. 
	 * @throws IOException
	 */
	public void botfile(String filename) throws IOException;
	
	/**
	 * Sets the skill level of the bot, which selects subsections from the botfiles. 
	 * @param skill The bot's skill level.
	 * @throws IOException
	 */
	public void skill(int skill) throws IOException;
	
	/**
	 * Sets the password of the bot's datastream socket. If this command is not issued, the password is the empty string.
	 * @param password Password to be used.
	 * @throws IOException
	 */
	public void password(String password) throws IOException;
	
	/**
	 * Sets the initial spawn position of the bot. 
	 * @param x The x coordinate of the initial position.
	 * @param y The y coordinate of the initial position.
	 * @param z The z coordinate of the initial position.
	 * @throws IOException
	 */
	public void initialPosition(double x, double y, double z) throws IOException;
	
}
