package edu.rochester.cs.quagents3.client;
import java.util.Scanner;

/**
 * Class which represents a rangefinder query.
 */
public class Rangefinder extends ProtocolZero{
	private static final String opcode = "rf";
	
	private final int type,range;
	private final double rot,azi;
	private double entityrange;
	private int entityid;
	
	public Rangefinder(int type, int range, double rot, double azi, priority_t priority){
		super(true,CommandType.RANGEFINDER,priority);
		this.type = type;
		this.range = range;
		this.rot = rot;
		this.azi = azi;
	}
	public Rangefinder(int type, int range, double rot, double azi){
		super(true,CommandType.RANGEFINDER);
		this.type = type;
		this.range = range;
		this.rot = rot;
		this.azi = azi;
	}
	public Rangefinder(double rot){
		super(true,CommandType.RANGEFINDER);
		this.type = Client.DEFAULT_RF_TYPE;
		this.range = 9999;
		this.rot = rot;
		this.azi = 0;
	}
	public Rangefinder(double rot,priority_t priority){
		super(true,CommandType.RANGEFINDER,priority);
		this.type = Client.DEFAULT_RF_TYPE;
		this.range = 9999;
		this.rot = rot;
		this.azi = 0;
	}
	public Rangefinder(){
		super(true,CommandType.RANGEFINDER);
		this.type = Client.DEFAULT_RF_TYPE;
		this.range = 9999;
		this.rot = 0;
		this.azi = 0;
	}
	public Rangefinder(priority_t priority){
		super(true,CommandType.RANGEFINDER,priority);
		this.type = Client.DEFAULT_RF_TYPE;
		this.range = 9999;
		this.rot = 0;
		this.azi = 0;
	}
	public Rangefinder(double rot, double azi){
		super(true,CommandType.RANGEFINDER);
		this.type = Client.DEFAULT_RF_TYPE;
		this.range = 9999;
		this.rot = rot;
		this.azi = azi;
	}
	public Rangefinder(int type, int range, double rot, double azi, priority_t priority,long period){
		super(true,CommandType.RANGEFINDER,period,priority);
		this.type = type;
		this.range = range;
		this.rot = rot;
		this.azi = azi;
	}
	public Rangefinder(int type, int range, double rot, double azi,long period){
		super(true,CommandType.RANGEFINDER,period);
		this.type = type;
		this.range = range;
		this.rot = rot;
		this.azi = azi;
	}
	public Rangefinder(int type, int range, double rot, double azi, priority_t priority,long period,long delay){
		super(true,CommandType.RANGEFINDER,period,delay,priority);
		this.type = type;
		this.range = range;
		this.rot = rot;
		this.azi = azi;
	}
	public Rangefinder(int type, int range, double rot, double azi,long period,long delay){
		super(true,CommandType.RANGEFINDER,period,delay);
		this.type = type;
		this.range = range;
		this.rot = rot;
		this.azi = azi;
	}
	
	protected void setOp(){
		op = Client.convertPriority(priority) + " " + opcode + " " + id+" "
		+ type +" "+range + " " + rot + " " + azi;	
	}
	
	/**
	 * Allows access to parameter data.
	 * @return Rangefinder type requested.
	 */
	public int getType(){return type;}
	
	/**
	 * Allows access to parameter data.
	 * @return Rangefinder range requested to server.
	 */
	public int getSearchRange(){return range;}
	
	/**
	 * Allows access to parameter data.
	 * @return Rotational angle requested.
	 */
	public double getRotationalAngle(){return rot;}
	
	/**
	 * Allows access to parameter data.
	 * @return Azimuthal angle requested.
	 */
	public double getAzimuthalAngle(){return azi;}
	
	/**
	 * Distance to entity detected by rangefinder. Note that this is still valid
	 * even if the "entity" is a wall (entityid=-2) or there is nothing to detect within range (entityid=-1).
	 * @return Distance to entity detected by rangefinder. 
	 */
	public double getEntityRange(){
		return entityrange;}
	
	/**
	 * Entity detected by rangefinder. Note that this is still valid
	 * even if the "entity" is a wall (entityid=-2) or there is nothing to detect within range (entityid=-1)
	 * @return Entity ID detected by rangefinder.
	 */
	public int getEntityID(){
		return entityid;}
	
	@Override
	public void setData(String params) {
		datastring = params;
		paramscan = new Scanner(params);
		entityrange = paramscan.nextDouble();
		entityid = paramscan.nextInt();
	}
}
