package edu.rochester.cs.quagents3.client;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.net.Socket;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.Scanner;
import edu.rochester.cs.quagents3.util.StoppableThread;
/**
 * Class which handles a non-blocking socket thread. <br/>
 * This handler reads any incoming server responses and updates relevent structures accordingly.
 * TODO: split this into two classes? one for client, one for GOD. Or maybe make it more general,
 * using top level entity inf
 */
public class ReaderThread extends StoppableThread {
	
	private BufferedReader reader;
	private Scanner scan;
	private InputStream sockin;
	private String exit;
	private Entity ent;
	
	public ReaderThread(Entity e, Socket sock) throws IOException{
    ent = e;
    setDaemon(true);
	sockin = sock.getInputStream();
	reader = new BufferedReader(new InputStreamReader(sockin));
	}

	
	public void run(){
		boolean data=false; boolean viewpoint = false;
		try {
		while(!this.isStopped()){
		data = false;
		//if(!client.validateSocket()) return; //we don't want to read from closed connection
		if(viewpoint){
			readViewpoint();
			viewpoint = false;
			continue;
		}
		exit = reader.readLine();
		if(exit==null || exit.equals(""))continue; //don't want to parse null string
		if(Client.VERBOSE || GOD.VERBOSE)
			System.out.println("Read: " + exit); System.out.flush();
		scan = new Scanner(exit);
		String code = scan.next();
		if(code.equals("id")){
			((QuagentEntity)ent).setQuagentID(scan.nextInt());continue;}
			//else if(god!=null){god.gid=scan.nextInt();continue;}}
		else if(code.equals("ready")) continue;
		else if(code.equals("rs")){
			String opcode = scan.next();
			data=true;
			if(opcode.equals("vc") || opcode.equals("vd")){
				viewpoint = true;
				continue;}}
		else if(code.equals("ev")) {handleEvent(scan.next(), scan.nextLine()); continue;}
		else 
			scan.next(); //we don't need opcode
		int id = scan.nextInt();
		if(data){
			updateDataID(id,scan.nextLine(),exit);
		}
		else{
		if(scan.hasNext()){
		String exitstring = scan.next();
		terminateID(id,exitstring,exit);
		}}}} 
	catch (IOException e) {e.printStackTrace();}
	}

	//rs vc id width height
	private void readViewpoint() {
		try{
		reader.read();reader.read();reader.read();reader.read(); //implicit "rs v" from function call
		char type = (char) reader.read(); //determine viewpoint type
		reader.read();
		//reader.read(); //burn space
		//get command id
		String id = "";
		char temp;
		while((temp=(char)reader.read())!=' '){
			id+=temp;
		}
		int cmdid = Integer.parseInt(id);
		//reader.read(); //burn space
		//get width
		id = "";
		while((temp=(char)reader.read())!=' ')
			id+=temp;
		int width = Integer.parseInt(id);
		//reader.read(); //burn space
		//get height
		id = "";
		while((temp=(char)reader.read())!=' ')
			id+=temp;
		int height = Integer.parseInt(id);
		//reader.read(); //burn last space
		//now we have command id and type, so we can get image bytestream
		byte[] bytes = null;
		if(type=='c')
			bytes = new byte[width*height*ImageViewpoint.BYTESPERPIXEL+1];
		else if(type=='d')
			bytes = new byte[width*height*DepthViewpoint.BYTESPERPIXEL+1];
		else{
			System.err.println("bad type in readViewpoint()");
			return;
		}
		for(int i=0; i<bytes.length; i++){
			bytes[i] = (byte) sockin.read();
//			if(Client.VERBOSE)
//				System.out.println("Read: " + bytes[i]); System.out.flush();
		}
		updateViewpointData(type, cmdid, width, height, bytes);} 
		catch(IOException e1){e1.printStackTrace();}
	}


	private void updateViewpointData(char type, int id, int width, int height, byte[] bytes) {
		if(id==0) return; //don't care about 0 id
		ImageViewpoint vc; DepthViewpoint vd;
		for(int i=0; i<ent.getRunningList().size(); i++){
			if(ent.getRunningList().get(i).getID() == id){
				if(type=='c'){
					vc = (ImageViewpoint) ent.getRunningList().get(i);
					vc.setData(width, height, bytes);
				}
				else if(type=='d'){
					vd = (DepthViewpoint) ent.getRunningList().get(i);
					vd.setData(width,height,bytes);
				}
			}}}


	private void updateDataID(int id, String data, String exit){
		if(id==0) return; //don't care about 0 id
		ProtocolZero temp;
		for(int i=0; i<ent.getRunningList().size(); i++){
			if((temp = ent.getRunningList().get(i)).getID()==id){
				temp.setData(data);
				return;
			}
			}
		System.err.println("error: malformed id1 " + exit);
	}
	
	private void terminateID(int id, String exitstring, String exit) throws IOException{
		if(id==0) return; //don't care about 0 ids
		ProtocolZero temp;
		for(int i=0; i<ent.getRunningList().size(); i++){
			if((temp = ent.getRunningList().get(i)).getID()==id){
				synchronized(temp){
					temp.terminate(exitstring);
					temp.notify();
				}
				return;
			}	
		}
		System.err.println("error: malformed id2: " + exit);
	}
	

	/**
	 * Handles events read from the <i>EventSocket</i>
	 * @param evcode The two character code which specifies the event type.
	 * @param evstring String containing all other data - this is <i>evcode</i> specific so we
	 * parse data here. 
	 */
	private void handleEvent(String evcode, String evstring){
		Scanner scan = new Scanner(evstring);
		if(ent.isQuagent()){
			if(evcode.equals("sy")){ //speak event
				int qid = scan.nextInt(); String msg = scan.nextLine(); msg = msg.substring(1);
				((QuagentEntity) ent).getEventListener().messageHeard(new QEvent(QEvent.LISTEN,qid,msg));
				//Runtime.getRuntime().exec("") //speech synth here
			}
			else if(evcode.equals("dt")){ //death event
				((QuagentEntity) ent).getEventListener().hasDied(new QEvent(QEvent.DIED,((QuagentEntity)ent).getQuagentID(),null));
			}
			else {
				System.err.println("event error: event type: " + evcode +" not handled.");
				System.err.flush();
			}
		}
		else{ //ent is GOD
			if(evcode.equals("dt"))
				((GODEntity) ent).getEventListener().playerDied(new GODEvent(GODEvent.PLAYERDIED, scan.nextInt(), scan.next()));
			else if(evcode.equals("jo"))
				((GODEntity) ent).getEventListener().playerJoined(new GODEvent(GODEvent.PLAYERJOIN, scan.nextInt(), scan.next()));
			else if(evcode.equals("dc"))
				((GODEntity) ent).getEventListener().playerDisconnected(new GODEvent(GODEvent.PLAYERDROP, scan.nextInt(), scan.next()));
			else {
				System.err.println("event error: event type: " + evcode +" not handled.");
				System.err.flush();
			}
		}
	}
}
