package edu.rochester.cs.quagents3.client;
import java.io.Writer;

/**
 * Class which represents a Rotate command.
 */
public class Rotate extends ProtocolZero{
	private static final String opcode = "ro";
	
	private final double rot,azi;
	
	public Rotate(double rot, double azi, priority_t priority){
		super(false,CommandType.ROTATE,priority);
		this.rot = rot;
		this.azi = azi;
	}
	public Rotate(double rot){
		super(false,CommandType.ROTATE);
		this.rot = rot;
		this.azi = 0;
	}
	public Rotate(double rot, priority_t priority){
		super(false, CommandType.ROTATE,priority);
		this.rot = rot;
		this.azi = 0;
	}
	public Rotate(double rot, double azi){
		super(false,CommandType.ROTATE);
		this.rot = rot;
		this.azi = azi;
	}
	public Rotate(double rot, double azi, priority_t priority,long period){
		super(false,CommandType.ROTATE,period,priority);
		this.rot = rot;
		this.azi = azi;
	}
	public Rotate(double rot, double azi,long period){
		super(false,CommandType.ROTATE,period);
		this.rot = rot;
		this.azi = azi;
	}
	public Rotate(double rot, double azi, priority_t priority,long period,long delay){
		super(false,CommandType.ROTATE,period,delay,priority);
		this.rot = rot;
		this.azi = azi;
	}
	public Rotate(double rot, double azi,long period,long delay){
		super(false,CommandType.ROTATE,period,delay);
		this.rot = rot;
		this.azi = azi;
	}
	
	protected void setOp(){
		op = Client.convertPriority(priority) + " " + opcode + " " + id + 
		" " + rot + " " + azi;
	}
	
	/**
	 * Allows access to parameter data.
	 * @return Rotational angle to rotate by.
	 */
	public double getRotationalAngle(){return rot;}
	
	/**
	 * Allows access to parameter data.
	 * @return Azimuthal angle to rotate by.
	 */
	public double getAzimuthalAngle(){return azi;}
	
	@Override
	public void setData(String params) {
		System.err.println("error: Rotate does not facilitate data response");
	}
}
