package edu.rochester.cs.quagents3.client;
import java.io.Writer;
import java.util.Scanner;

/**
 * Class which represents a SwitchWeapon command.
 */
public class SwitchWeapon extends ProtocolZero{
	private static final String opcode = "sw";
	
	private final weapon newwep;
	private int ammo=-1; //default is fail in case of invalid selection
	
	public SwitchWeapon(weapon newwep, priority_t priority){
		super(true,CommandType.SWITCHWEAPON,priority);
		this.newwep=newwep;
	}
	public SwitchWeapon(weapon newwep){
		super(true,CommandType.SWITCHWEAPON);
		this.newwep=newwep;
	}
	public SwitchWeapon(weapon newwep, priority_t priority,long period){
		super(true,CommandType.SWITCHWEAPON,period,priority);
		this.newwep=newwep;
	}
	public SwitchWeapon(weapon newwep,long period){
		super(true,CommandType.SWITCHWEAPON,period);
		this.newwep=newwep;
	}
	public SwitchWeapon(weapon newwep, priority_t priority,long period,long delay){
		super(true,CommandType.SWITCHWEAPON,period,delay,priority);
		this.newwep=newwep;
	}
	public SwitchWeapon(weapon newwep,long period,long delay){
		super(true,CommandType.SWITCHWEAPON,period,delay);
		this.newwep=newwep;
	}
	
	protected void setOp(){
		op = Client.convertPriority(priority) + " " + opcode + " " + id + " " + convertWeapon(newwep);
	}
	
	/**
	 * Gets equipped weapon (as far as this command's visibility knows).
	 * @return Currently equipped weapon.
	 */
	public weapon getWeapon(){
		return newwep;
		}
	
	/**
	 * Gets ammunition for newly switched-in weapon.
	 * @return Newly switched-in weapon ammunition.
	 */
	public int getAmmo(){
		return ammo;
	}
	
	/**
	 * Used to format weapon request for server protocol.
	 * @param wep
	 * @return
	 */
	public static int convertWeapon(weapon wep){
		switch(wep){
		case GAUNTLET: 
			return 1;
		case MACHINEGUN:
			return 2;
		case SHOTGUN:
			return 3;
		case GRENADE_LAUNCHER:
			return 4;
		case ROCKET_LAUNCHER:
			return 5;
		case LIGHTING_GUN:
			return 6;
		case RAILGUN:
			return 7;
		case PLASMA_PISTOL:
			return 8;
		case BFG:
			return 9;
		default:
			System.err.println("error: invalid weapon selection");
			return -1; //exit with error
		}
	}

	public void setData(String params) {
		datastring = params;
		paramscan = new Scanner(params);
		ammo = paramscan.nextInt();
	}
}
