package edu.rochester.cs.quagents3.client;
// Class that keeps track of the state of the Wumpus World using boolean arrays

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.TextArea;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.net.UnknownHostException;
import java.util.*;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JPanel;

public class WumpusSolver extends AbstractWumpusSolver implements QuagentExecution{
	// Global variables
	public Client agent;
	public boolean[][] breeze;
	public boolean[][] stench;
	public boolean[][] glitter;
	public boolean[][] pits;
	public boolean[][] wumpus;
	public boolean[][] explored;
	public boolean[][] safe;
	public boolean found_wumpus, killed_wumpus;
	//public int dim_x, dim_y, pos_x, pos_y
	public int wump_x, wump_y;
	//public int facing;
	public int wumpus_count;
	//public final int right = 0;
	//public final int up = 90;
	//public final int left = 180;
	//public final int down = 270;
	//public final int roomSize = 192;
	GOD director;
	
	public GUI gui;
	private final int guiroomsize;
	private final int width;
	private final int height;
	
	// Constructor - takes an m x n board and a client
	public WumpusSolver(int x, int y, GOD director) {
		Client.synchronous = true;
		//Client.DEFAULTSPEED = .2;
		this.director = director;
		try {agent = new Client("127.0.0.1");} 
		catch (UnknownHostException e) {e.printStackTrace();} 
		catch (IOException e) {e.printStackTrace();}
		dim_x = x;
		dim_y = y;
		guiroomsize = roomSize;//dim_x
		//needs to be const size...fix this
		width = x*guiroomsize;
		height = y*guiroomsize;
		found_wumpus = killed_wumpus = false;
		wumpus_count = 0;
		breeze = new boolean[dim_x][dim_y];
		stench = new boolean[dim_x][dim_y];
		glitter = new boolean[dim_x][dim_y];
		pits = new boolean[dim_x][dim_y];
		wumpus = new boolean[dim_x][dim_y];
		explored = new boolean[dim_x][dim_y];
		safe = new boolean[dim_x][dim_y];
		pos_x = pos_y = 0;
		wump_x = wump_y = -1;
		facing = right;
		for (int i = 0; i < dim_y; i++) {
			for (int j = 0; j < dim_x; j++) {
				breeze[i][j] = false;
				stench[i][j] = false;
				glitter[i][j] = false;
				pits[i][j] = false;
				wumpus[i][j] = false;
				explored[i][j] = false;
				safe[i][j] = false;
			}
		}
		gui = new GUI();
		try{
		agent.name("Wumpus Explorer");
		agent.ready();
		director.execute(new FollowViewpoint(agent.getQuagentID()));
		}
		catch(Exception e1){e1.printStackTrace();}
	}
	
	// Uses the radar to get the senses in a room
	public void getSensor() {
		// nothing new if this room is already explored
		if (getExplored(pos_x, pos_y))
			return;
		
		// you only explore safe rooms 
		boolean br = false;
		boolean st = false;
		setExplored(pos_x, pos_y, true);
		setSafe(pos_x, pos_y, true);
		//Radar ra = agent.radar(96.0);
		//ra.waitForTerminate();
		Radar ra = sense(agent);
		int max = ra.getNumberEntitiesFound();
		// Each sense is actually an item in quake. Loops through all found entities for senses
		for (int i = 0; i < max; i++) {
			if (ra.getClasstype(i).equals("quagent_sense_breeze")) {
				gui.writeToScreen("You feel a breeze in room (" + pos_x + ", " + pos_y + ").");
				setBreeze(pos_x, pos_y, true);
				br = true;
			}
			if (ra.getClasstype(i).equals("quagent_sense_stench")) {
				gui.writeToScreen("You smell a stench in room (" + pos_x + ", " + pos_y + ").");
				setStench(pos_x, pos_y, true);
				st = true;
			}
			if (ra.getClasstype(i).equals("quagent_sense_glitter")) {
				gui.writeToScreen("You see glitter in room (" + pos_x + ", " + pos_y + ").");
				setGlitter(pos_x, pos_y, true);
			}
		}

		// Update the booleans arrays for adjacent rooms
		if (pos_x != 0 && !getSafe(pos_x - 1, pos_y)) {
			if (st && !found_wumpus) {
				setWumpus(pos_x - 1, pos_y, true);
				wumpus_count++;
			}
			if (br)
				setPit(pos_x - 1, pos_y, true);
			update(pos_x - 1, pos_y);
		}
		if (pos_x != dim_x - 1 && !getSafe(pos_x + 1, pos_y)) {
			if (st && !found_wumpus) {
				setWumpus(pos_x + 1, pos_y, true);
				wumpus_count++;
			}
			if (br)
				setPit(pos_x + 1, pos_y, true);
			update(pos_x + 1, pos_y);
		}
		if (pos_y != 0 && !getSafe(pos_x, pos_y - 1)) {
			if (st && !found_wumpus) {
				setWumpus(pos_x, pos_y - 1, true);
				wumpus_count++;
			}
			if (br)
				setPit(pos_x, pos_y - 1, true);
			update(pos_x, pos_y - 1);
		}
		if (pos_y != dim_y - 1 && !getSafe(pos_x, pos_y + 1)) {
			if (st && !found_wumpus) {
				setWumpus(pos_x, pos_y + 1, true);
				wumpus_count++;
			}
			if (br)
				setPit(pos_x, pos_y + 1, true);
			update(pos_x, pos_y + 1);
		}
		// There is only 1 wumpus in the map
		if (wumpus_count == 1) {
			found_wumpus = true;
			for (int i = 0; i < dim_x; i++)
				for (int j = 0; j < dim_y; j++)
					if (getWumpus(i, j)) {
						wump_x = i;
						wump_y = j;
					}
		}
		gui.repaint();
	}
	
	// Moves the agent by one room - does not move into walls
	/*public boolean move(Client agent) {
		switch (facing) {
		case up: 
			if (pos_y == dim_y - 1) {
				System.out.println("Bump wall");
				return false;
			}
			pos_y++; break;
    	case down: 
    		if (pos_y == 0) {
				System.out.println("Bump wall");
				return false;
			}
    		pos_y--; break;
    	case left: 
    		if (pos_x == 0) {
				System.out.println("Bump wall");
				return false;
			}
    		pos_x--; break;
    	case right: 
    		if (pos_x == dim_x - 1) {
				System.out.println("Bump wall");
				return false;
			}
    		pos_x++; break;
		}
		MoveBy mb = agent.moveBy(roomSize);
		mb.waitForTerminate();
		gui.repaint();
		return true;
	}*/
	
	// Moves to a specified room from current location traveling through safe rooms - uses breadth first search
	public boolean moveTo(int x, int y) {
		boolean found = false;
		// keeps track of the shortest path
		int path[] = new int[dim_x * dim_y];
		for (int i = 0; i < path.length; i++)
			path[i] = -1;
		path[dim_x * y + x] = -2;
		WumpusNode end = new WumpusNode(x, y);
		Queue<WumpusNode> q = new LinkedList<WumpusNode>();
		q.add(end);
		// Use a queue to perform BFS
		while(!q.isEmpty()) {
			WumpusNode temp = q.remove();
			if (temp.x == pos_x && temp.y == pos_y) {
				found = true;
				break;
			}
			// Add adjacent safe nodes
			if (temp.x > 0 && getSafe(temp.x - 1, temp.y) && path[dim_x * temp.y + (temp.x - 1)] == -1) {
				q.add(new WumpusNode(temp.x - 1, temp.y));
				path[dim_x * temp.y + (temp.x - 1)] = dim_x * temp.y + temp.x;
			}
			if (temp.x < dim_x - 1 && getSafe(temp.x + 1, temp.y) && path[dim_x * temp.y + (temp.x + 1)] == -1) {
				q.add(new WumpusNode(temp.x + 1, temp.y));
				path[dim_x * temp.y + (temp.x + 1)] = dim_x * temp.y + temp.x;
			}
			if (temp.y > 0 && getSafe(temp.x, temp.y - 1) && path[dim_x * (temp.y - 1) + temp.x] == -1) {
				q.add(new WumpusNode(temp.x, temp.y - 1));
				path[dim_x * (temp.y - 1) + temp.x] = dim_x * temp.y + temp.x;
			}
			if (temp.y < dim_y - 1 && getSafe(temp.x, temp.y + 1) && path[dim_x * (temp.y + 1) + temp.x] == -1) {
				q.add(new WumpusNode(temp.x, temp.y + 1));
				path[dim_x * (temp.y + 1) + temp.x] = dim_x * temp.y + temp.x;
			}
		}
		// if there is a path, move to that room
		if (found) {
			while(pos_x != x || pos_y != y) {
				int current = dim_x * pos_y + pos_x;
				int next_x = path[current] % dim_x;
				int next_y = path[current] / dim_x;
				System.out.println(next_x + " " + next_y);
				if (next_x == pos_x + 1)
					faceDirection(agent, 'r');
				else if (next_x == pos_x - 1)
					faceDirection(agent, 'l');
				else if (next_y == pos_y + 1)
					faceDirection(agent, 'u');
				else if (next_y == pos_y - 1)
					faceDirection(agent, 'd');
				move(agent);
				gui.repaint();
			}
		}
			
		return found;
	}
	
	// Face a direction
	/*public void faceDirection(char dir) {
		switch (dir) {
		case 'u':
			facing = up;
			Rotate ro = agent.rotate(up - agent.facing().getViewDirection()[0]); 
			ro.waitForTerminate();
			break;
		case 'd':
			facing = down;
			ro = agent.rotate(down - agent.facing().getViewDirection()[0]);
			ro.waitForTerminate();
			break;
		case 'l': 
			facing = left;
			ro = agent.rotate(left - agent.facing().getViewDirection()[0]); 
			ro.waitForTerminate();
			break;
		case 'r':
			facing = right;
			ro = agent.rotate(right - agent.facing().getViewDirection()[0]); 
			ro.waitForTerminate();
			break;
		}
	}*/
	
	// Uses depth first search to explore the wumpus world. The agent will only make safe moves.
	public void explore() {
		WumpusNode current = new WumpusNode(pos_x, pos_y);
		Stack<WumpusNode> s = new Stack<WumpusNode>();
		s.push(current);
		// Uses a stack to perform dfs
		while (!s.isEmpty()) {
			WumpusNode temp = s.pop();
			if (getExplored(temp.x, temp.y)) {
				continue;
			}
			// if the gold is found, pick it up and go back to the beginning
			moveTo(temp.x, temp.y);
			getSensor();
			if (getGlitter(pos_x, pos_y)) {
				agent.execute(new PickUp());
				moveTo(0, 0);
				break;
			}
			// if wumpus is found try to kill it if you can from current room
			if (found_wumpus && !killed_wumpus) {
				if (pos_x == wump_x) {
					if (pos_y > wump_y) {
						faceDirection(agent, 'd');
						while(agent.currentAmmo(weapon.MACHINEGUN).getAmmo() > 0) {
							FireWeapon fw = agent.fireWeapon();
							fw.waitForTerminate();
						}
					}
					else {
						faceDirection(agent, 'u');
						while(agent.currentAmmo(weapon.MACHINEGUN).getAmmo() > 0) {
							FireWeapon fw = agent.fireWeapon();
							fw.waitForTerminate();
						}
					}
					killed_wumpus = true;
					setWumpus(wump_x, wump_y, false);
					setSafe(wump_x, wump_y, true);
				}
				else if (pos_y == wump_y) {
					if (pos_x > wump_x) {
						faceDirection(agent, 'l');
						for (int a = 0; a < 100; a++) {
							FireWeapon fw = agent.fireWeapon();
							fw.waitForTerminate();
						}
					}
					else {
						faceDirection(agent, 'r');
						while(agent.currentAmmo(weapon.MACHINEGUN).getAmmo() > 0) {
							FireWeapon fw = agent.fireWeapon();
							fw.waitForTerminate();
						}
					}
					killed_wumpus = true;
					setWumpus(wump_x, wump_y, false);
					setSafe(wump_x, wump_y, true);
				}
			}
			// push adjacent rooms onto stack
			if (temp.x > 0 && getSafe(temp.x - 1, temp.y) && !getExplored(temp.x - 1, temp.y)) {
				s.push(new WumpusNode(temp.x - 1, temp.y));
			}
			if (temp.x < dim_x - 1 && getSafe(temp.x + 1, temp.y) && !getExplored(temp.x + 1, temp.y)) {
				s.push(new WumpusNode(temp.x + 1, temp.y));
			}
			if (temp.y > 0 && getSafe(temp.x, temp.y - 1) && !getExplored(temp.x, temp.y - 1)) {
				s.push(new WumpusNode(temp.x, temp.y - 1));
			}
			if (temp.y < dim_y - 1 && getSafe(temp.x, temp.y + 1) && !getExplored(temp.x, temp.y + 1)) {
				s.push(new WumpusNode(temp.x, temp.y + 1));
			}
		}
	}
	
	// Updates the boolean arrays
	public void update(int x, int y) {
		int stench_count = 0;
		// There is only a wumpus if there is stench on all four sides, only one wumpus per map.
		if (getWumpus(x, y) && !found_wumpus) {
			if (x != 0 && getExplored(x - 1, y)) {
				if (!getStench(x - 1, y)) {
					setWumpus(x, y, false);
					wumpus_count--;
				}
				else
					stench_count++;
			}
			if (x != dim_x - 1 && getExplored(x + 1, y)) {
				if (!getStench(x + 1, y)) {
					setWumpus(x, y, false);
					wumpus_count--;
				}
				else
					stench_count++;
			}
			if (y != 0 && getExplored(x, y - 1)) {
				if (!getStench(x, y - 1)) {
					setWumpus(x, y, false);
					wumpus_count--;
				}
				else
					stench_count++;
			}
			if (y != dim_y - 1 && getExplored(x, y + 1)) {
				if (!getStench(x, y + 1)) {
					setWumpus(x, y, false);
					wumpus_count--;
				}
				else
					stench_count++;
			}
			if (stench_count >= 2) {
				found_wumpus = true;
				wump_x = x;
				wump_y = y;
				
				for (int i = 0; i < dim_y; i++) {
					for (int j = 0; j < dim_x; j++) {
						if (i != y || j != x) {
							setWumpus(j, i, false);
						}
					}
				}
			}
		}
		// There is only a pit if there is breeze on all four sides
		if (getPit(x, y)) {
			if (x != 0 && getExplored(x - 1, y) && !getBreeze(x - 1, y)) {
				setPit(x, y, false);
			}
			if (x != dim_x - 1 && getExplored(x + 1, y) && !getBreeze(x + 1, y)) {
				setPit(x, y, false);
			}
			if (y != 0 && getExplored(x, y - 1) && !getBreeze(x, y - 1)) {
				setPit(x, y, false);
			}
			if (y != dim_y - 1 && getExplored(x, y + 1) && !getBreeze(x, y + 1)) {
				setPit(x, y, false);
			}
		}
		if (!getPit(x, y) && !getWumpus(x, y))
			setSafe(x, y, true);
	}
	
	// Prints out the current boolean arrays
	public void test() {
		System.out.println("Pits");
		for (int i = dim_y - 1; i >= 0; i--) {
			for (int j = 0; j < dim_x; j++) {
				System.out.print(getPit(j, i) + " ");
			}
			System.out.println();
		}
		System.out.println("Wumpus");
		for (int i = dim_y - 1; i >= 0; i--) {
			for (int j = 0; j < dim_x; j++) {
				System.out.print(getWumpus(j, i) + " ");
			}
			System.out.println();
		}
		System.out.println("Explored");
		for (int i = dim_y - 1; i >= 0; i--) {
			for (int j = 0; j < dim_x; j++) {
				System.out.print(getExplored(j, i) + " ");
			}
			System.out.println();
		}
		System.out.println("Safe");
		for (int i = dim_y - 1; i >= 0; i--) {
			for (int j = 0; j < dim_x; j++) {
				System.out.print(getSafe(j, i) + " ");
			}
			System.out.println();
		}
		System.out.println("found_wumpus = " + found_wumpus + " (" + wump_x + ", " + wump_y + ")");
		System.out.println("killed_wumpus = " + killed_wumpus);
	}
	
	// Setter and getter methods for the arrays
	public void setBreeze(int x, int y, boolean val) {
		breeze[x][y] = val;
	}
	
	public boolean getBreeze(int x, int y) {
		return breeze[x][y];
	}
	
	public void setStench(int x, int y, boolean val) {
		stench[x][y] = val;
	}
	
	public boolean getStench(int x, int y) {
		return stench[x][y];
	}
	
	public void setGlitter(int x, int y, boolean val) {
		glitter[x][y] = val;
	}
	
	public boolean getGlitter(int x, int y) {
		return glitter[x][y];
	}
	
	public void setExplored(int x, int y, boolean val) {
		explored[x][y] = val;
	}
	
	public boolean getExplored(int x, int y) {
		return explored[x][y];
	}
	
	public void setWumpus(int x, int y, boolean val) {
		wumpus[x][y] = val;
	}
	
	public boolean getWumpus(int x, int y) {
		return wumpus[x][y];
	}
	
	public void setPit(int x, int y, boolean val) {
		pits[x][y] = val;
	}
	
	public boolean getPit(int x, int y) {
		return pits[x][y];
	}
	
	public void setSafe(int x, int y, boolean val) {
		safe[x][y] = val;
	}
	
	public boolean getSafe(int x, int y) {
		return safe[x][y];
	}

	public void executeCode() {
		synchronized(agent){
			try {agent.wait();}
			catch (InterruptedException e) {e.printStackTrace();}
		}
		explore();
		test();
	}
	
	private class GUI extends JPanel{
		private ImageIcon wumpusimg, archerimg;
		private JButton start,pause,resume;
		private TextArea area;
		
		public GUI(){
			wumpusimg = new ImageIcon("wumpus.gif");
			archerimg = new ImageIcon("agentr.gif");
			area = new TextArea();
			start = new JButton("Start");
			start.addActionListener(new StartListener());
			pause = new JButton("Pause");
			pause.addActionListener(new PauseListener());
			resume = new JButton("Resume");
			resume.addActionListener(new ResumeListener());
			setPreferredSize(new Dimension(width,height+100));
			setBackground(Color.black);
			area.setBackground(Color.black);
			area.setForeground(Color.white);
			area.setEditable(false);
			this.add(start);
			this.add(pause);
			this.add(resume);
			this.add(area);
			start.setVisible(true);
			pause.setVisible(false);
			resume.setVisible(false);
			repaint();
		}
		
		//writes new line to screen textarea
		public void writeToScreen(String s){
			if(s.endsWith("\n"))
				area.append(s);
			else{
				s += "\n";
				area.append(s);
			}
			repaint();
		}
		
		public void paintComponent(Graphics g){
			super.paintComponent(g);
			g.setColor(Color.white);
			for(int i=0; i<dim_x; i++)
				g.drawLine(i*guiroomsize, 0, i*guiroomsize, height);
			for(int j=0; j<dim_y; j++)
				g.drawLine(0, j*guiroomsize, width, j*guiroomsize);
			
			for(int i=0; i<dim_x; i++){
				for(int j=0; j<dim_y; j++){
					if(pits[i][dim_y-j-1]){
						g.setColor(Color.white);
						g.fillRect(i*guiroomsize, j*guiroomsize, guiroomsize, guiroomsize);
					}
					if(breeze[i][dim_y-j-1]){
						g.setColor(Color.blue);
						g.fillRect(i*guiroomsize, j*guiroomsize, guiroomsize, guiroomsize);
					}
					if(stench[i][dim_y-j-1]){
						g.setColor(Color.green);
						g.fillRect(i*guiroomsize, j*guiroomsize, guiroomsize, guiroomsize);
					}
					if(glitter[i][dim_y-j-1]){
						g.setColor(Color.yellow);
						g.fillRect(i*guiroomsize, j*guiroomsize, guiroomsize, guiroomsize);
					}
					if(wumpus[i][dim_y-j-1])
						wumpusimg.paintIcon(this, g, i*guiroomsize, j*guiroomsize);
				}
			}
			archerimg.paintIcon(this, g, pos_x*guiroomsize, (dim_y-pos_y-1)*guiroomsize);
			
			pause.setLocation((dim_x)*guiroomsize-70, 10);
			start.setLocation((dim_x)*guiroomsize-70, 10);
			resume.setLocation((dim_x)*guiroomsize-70, 10);
			area.setSize(new Dimension(width,100));
			area.setLocation(0, dim_y*guiroomsize+1);
		}
		
		private class StartListener implements ActionListener{
			public void actionPerformed(ActionEvent e){
				start.setVisible(false);
				pause.setVisible(true);
				synchronized(agent){
					agent.notify();
				}
			}
		}
		
		private class PauseListener implements ActionListener{
			public void actionPerformed(ActionEvent e){
				pause.setVisible(false);
				agent.start(new Pause());
				resume.setVisible(true);
			}
		}
		
		private class ResumeListener implements ActionListener{
			public void actionPerformed(ActionEvent e){
				resume.setVisible(false);
				agent.pop(priority_t.NOW);
				pause.setVisible(true);
			}
		}
	}

    public static void main(String[] argv) throws UnknownHostException, IOException {
	// Expects to be on the Wumpus2 map
	WumpusSolver agent = new WumpusSolver(4,4, new GOD("127.0.0.1"));
	// Setup gui
	javax.swing.JFrame frame = new javax.swing.JFrame("Quagents3 Wumpus World");
	frame.getContentPane().add(agent.gui);
	frame.pack();
	frame.setVisible(true);
	// Run agent
	agent.executeCode();
    }
}
