package edu.rochester.cs.quagents3.main;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import edu.rochester.cs.quagents3.client.*;

public class CaveExplorationGUI extends JPanel{
	private static CaveExplorer explorers[];
	private static TrappedSurvivor survivors[];
	private static JComboBox[] behaviors; // || to explorers
	private static Color[] explorerchatcolors; // || to explorers
	private static JLabel[] explorernames;// ""
	private static JComboBox[] followboxes; // ""
	private static JButton[] viewpoints; //|| to explorers
	//private JPanel companel;
	private static GOD director;
	
	private static String chatbuffer=""; //to ensure no repeats
	private static TextArea chatwindow;
	private static  final int width = 600;
	private static final int height = 500;
	
	boolean follow = false;
	boolean rendez = false;
	
	public CaveExplorationGUI(CaveExplorer explorers[], TrappedSurvivor survivors[], GOD director){
		this.director = director;
		CaveExplorationGUI.explorers = explorers;
		CaveExplorationGUI.survivors = survivors;
		behaviors = new JComboBox[explorers.length];
		explorerchatcolors = new Color[explorers.length];
		explorernames = new JLabel[explorers.length];
		followboxes = new JComboBox[explorers.length];
		viewpoints = new JButton[explorers.length];
		chatwindow = new TextArea("",14,73,TextArea.SCROLLBARS_VERTICAL_ONLY);
		for(int i=0; i<behaviors.length; i++){
			behaviors[i] = new JComboBox(CaveExplorer.behaviors);
			behaviors[i].setToolTipText("Select " + explorers[i].getExplorerName() + "'s behavior");
			behaviors[i].addActionListener(new BehaviorListener(i));
		}
		for(int i=0; i<explorerchatcolors.length; i++)
			//explorerchatcolors[i] = new Color(255-255/explorers.length * i, 255-255/explorers.length * i, 255-255/explorers.length * i);
			explorerchatcolors[i] = Color.white;
		for(int i=0; i<explorernames.length; i++)
			explorernames[i] = new JLabel(explorers[i].getExplorerName());
		for(int i=0; i<followboxes.length; i++){
			followboxes[i] = new JComboBox();
			for(int j=0; j<followboxes.length; j++){
				if(i==j) continue;
				followboxes[i].addItem(explorers[j].getExplorerName());
			}
			followboxes[i].setToolTipText("Select which explorer to follow.");
			followboxes[i].setVisible(false);
		}
		for(int i=0; i<viewpoints.length; i++){
			viewpoints[i] = new JButton("Look");
			viewpoints[i].addActionListener(new ViewpointListener(i));
			viewpoints[i].setEnabled(true);
			viewpoints[i].setVisible(true);
		}
			
		this.setBackground(Color.black);
		this.setPreferredSize(new Dimension(width,height));
		chatwindow.setBackground(Color.white);
		chatwindow.setEditable(false);
		JPanel behaviorpanel = new JPanel();
		behaviorpanel.setLayout(new GridLayout(4,explorers.length,50,10));
		behaviorpanel.setPreferredSize(new Dimension(width-(width-100)/explorers.length,120));
		behaviorpanel.setOpaque(false);
		for(int i=0; i<explorernames.length; i++){
			explorernames[i].setForeground(explorerchatcolors[i]);
			behaviorpanel.add(explorernames[i]);
		}
		for(int i=0; i<viewpoints.length; i++)
			behaviorpanel.add(viewpoints[i]);
		for(int i=0; i<behaviors.length; i++)
			behaviorpanel.add(behaviors[i]);
		for(int i=0; i<followboxes.length; i++){
			followboxes[i].addActionListener(new FollowBoxListener(i));
			behaviorpanel.add(followboxes[i]);
		}
		this.add(behaviorpanel);
		this.add(chatwindow);
		viewpoints[0].doClick();
		repaint();
	}
	
	public static void explorerDied(int id){
		behaviors[id].setEnabled(false);
		explorernames[id].setEnabled(false);
		chatwindow.append(explorers[id].getExplorerName()+" has died.\n");
	}
	
	public static void explorerHeardMessage(int id, String message){
		String msg;
		if(id>explorers.length-1)
			msg = survivors[id-explorers.length].getSurvivorName() + ": " + message.toUpperCase() + "\n";
		else
			msg = explorers[id].getExplorerName()+": " +message+"\n";
		synchronized(chatbuffer){
			if(!msg.equals(chatbuffer)){
				chatbuffer = msg;
				chatwindow.append(msg);
			}
		}
	}
	
	public void paintComponent(Graphics g){
		super.paintComponent(g);
		chatwindow.setLocation(0, 275);
	}
	
	private class BehaviorListener implements ActionListener{
		int index;
		public BehaviorListener(int index){
			this.index = index;
		}
		public void actionPerformed(ActionEvent e) {
			String behaviortype = (String) ((JComboBox) e.getSource()).getSelectedItem();
			if(behaviortype.equals("Explore")){
				if(!explorers[index].isExploring())
					explorers[index].explore();
			}
			else if(behaviortype.equals("Wait")){
				if(!explorers[index].isWaiting())
					explorers[index].pause();	
			}
			else if(behaviortype.equals("Rendezvous")){
				behaviors[index].setEnabled(false);
				followboxes[index].setVisible(true);
				((FollowBoxListener) followboxes[index].getActionListeners()[0]).setBoxType(behaviortype);
			}
			else if(behaviortype.equals("Follow")){
				behaviors[index].setEnabled(false);
				followboxes[index].setVisible(true);
				((FollowBoxListener) followboxes[index].getActionListeners()[0]).setBoxType(behaviortype);
			}
			else if(behaviortype.equals("Rescue")){
				if(!explorers[index].isRescuing())
					explorers[index].rescue();
			}
		}
	}
	
	private class FollowBoxListener implements ActionListener{
		int index;
		String type;
		
		public FollowBoxListener(int index){
			this.index = index;
		}
		
		public void actionPerformed(ActionEvent e){
			String name = (String)((JComboBox)e.getSource()).getSelectedItem();
			int id = getIndexGivenName(name);
			if(type.equals("Follow")){
				if(!explorers[index].isFollowing())
					explorers[index].follow(id+1, name);
			}
			else if(type.equals("Rendezvous")){
				System.out.println("test");
				if(!explorers[index].isRendezvousing())
					explorers[index].rendezvous(id);
				followboxes[index].removeItem("All Explorers");
			}
			followboxes[index].setVisible(false);
			behaviors[index].setEnabled(true);
		}
		
		public void setBoxType(String type){
			this.type = type;
			if(type.equals("Rendezvous")){
				followboxes[index].addItem("All Explorers");
			}
		}
	}
	
	private int getIndexGivenName(String name){
		for(int i=0; i<explorers.length; i++){
			if(explorers[i].getExplorerName().equals(name))
				return i;
		}
		return -1; //name not found
	}
	
	private class ViewpointListener implements ActionListener{
		int index;
		
		public ViewpointListener(int index){
			this.index = index;
		}
		
		public void actionPerformed(ActionEvent e1){
			for(int i=0; i<viewpoints.length; i++)
				viewpoints[i].setEnabled(true);
			viewpoints[index].setEnabled(false);
			director.execute(new FollowViewpoint(explorers[index].getEntityID()));
		}
	}
}


