package edu.rochester.cs.quagents3.main;

import edu.rochester.cs.quagents3.client.*;

import java.io.*;
import java.util.Scanner;

/**
 * A CaveExplorer represents a single Quagent member of a larger explorer team.
 * To maintain collaborative goals, each CaveExplorer is completely independent of the other members of its team.
 * The only way that CaveExplorers may exchange information is through verbal communication (<i>say</i> and <i>messageHeard</i>).
 */
public class CaveExplorer implements QuagentExecution{
	public static final String[] behaviors = {"Wait", "Explore", "Rendezvous", "Follow", "Rescue"}; //TODO: add follow, rendezvous, otro
	public static final long sensorperiod = 250;
	private Client explorer;
	public final int id;
	private volatile CartesianSensor2D sonar;
	private Facing currentfacing;
	private Follow follow;
	private volatile boolean isexploring, iswaiting, isrendez, isfollowing, isrescue;
	public static final int ROOMSIZE = 256;
	
	private int[] location; //{x,y}
	
	public CaveExplorer(int id){
		this.id = id;
		location = new int[2];
		sonar = new CartesianSensor2D(sensorperiod);
		sonar.addTerminationListener(new SonarTerminationListener());
		currentfacing = new Facing(sensorperiod/3*2);
		try{
		explorer = new Client("127.0.0.1");
		explorer.addEventListener(new ExplorerEventListener());
		explorer.name("Explorer" + id);
		explorer.ready();
		} catch(Exception e1){System.err.println("Please specify a valid Quagents3 server address");}
		reset();
		explorer.start(sonar);
	}
	
	private void reset(){
		if(isfollowing){
			follow.pause();
			follow.terminate();
		}
		if(isexploring){
			sonar.pause();
			currentfacing.pause();
		}
		isexploring=iswaiting=isrendez=isfollowing=isrescue=false;
		explorer.forgetAllTasks(priority_t.NOW);
	}
	
	public String getExplorerName(){
		return explorer.name;
	}
	
	public void executeCode() {
		explorer.say("Awaiting orders");
	}
	
	
	/* This is now a director command
	public void look(){
		explorer.start(new FollowViewpoint(priority_t.NOW));
	}
	*/
	
	public int getEntityID(){
		return explorer.getQuagentID();
	}
	
	public void explore(){
		reset(); isexploring=true;
		sonar.resume(); currentfacing.resume();
		explorer.say("Understood, I will begin exploring.",priority_t.NOW);
		explorer.moveForever();
	}
	
	public void follow(int id, String name){
		follow = new Follow(id); 
		reset(); isfollowing=true;
		explorer.say("Understood, I will follow " + name + ".",priority_t.NOW);
		explorer.start(follow);
	}
	
	public void pause(){
		reset(); iswaiting=true;
		explorer.say("Understood, I will wait here.", priority_t.NOW);
		explorer.pause();
	}
	
	//id<0: all rendezvous
	public void rendezvous(int id){
		reset(); isrendez=true;
		explorer.say("Understood, I will begin the rendezvous process.",priority_t.NOW);
		if(id>0)
			explorer.say("Explorer " + id + " location", priority_t.NOW);
		else
			explorer.say("All where is your current location?", priority_t.NOW);
		
	}
	
	public void rescue(){
		reset(); isrescue=true;
		explorer.say("Understood, I will search for trapped survivors.");
		explorer.say("IS THERE ANYONE TRAPPED IN HERE? PLEASE RESPOND!");
	}
	
	private void parseMessage(String message, int origin){
		Scanner scan = new Scanner(message);
		String quantifier = scan.next();
		if(quantifier.equals("Explorer")){ //implicit existential
			if(scan.nextInt()==this.id){ //if addressed to us
				//we can parse desired operation
				String operation = scan.next();
				if(operation.equals("location")){
					explorer.say("My current location is ");
				}
				else
					System.err.println("Undefined operation-- \""+message +"\" from entity " + origin);
			}
		}
		else if(quantifier.equals("All")){ //universal
			String operation = scan.nextLine();
			if(operation.equals("where is your current location?")){
				explorer.say("My current location is ");
			}
		}
		else //otherwise, 
			return;
	}
	
	public boolean isExploring(){return isexploring;}
	public boolean isWaiting(){return iswaiting;}
	public boolean isRendezvousing(){return isrendez;}
	public boolean isFollowing(){return isfollowing;}
	public boolean isRescuing(){return isrescue;}
	
	private class SonarTerminationListener implements CommandTerminationListener{
		private Rotate rotateleft, rotateright, turnaround;
		
		public SonarTerminationListener(){
			rotateleft = new Rotate(90,priority_t.NOW);
			rotateright = new Rotate(270, priority_t.NOW);
			turnaround = new Rotate(180, priority_t.NOW);
		}
		public void commandTerminated(Command com) {
			if(Client.VERBOSE)
				System.out.println(sonar.toString());
			if(sonar.getForwardDistance()< sonar.getRightDistance() && isexploring){
					explorer.execute(rotateright);
					explorer.moveForever();
				}
			}
		}
	

	private class ExplorerEventListener implements QEventListener{
		public void hasDied(QEvent event) {
			CaveExplorationGUI.explorerDied(event.getOrigin()-1);
			explorer.pause();
		}
		public void messageHeard(QEvent event) {
			CaveExplorationGUI.explorerHeardMessage(event.getOrigin()-1, event.getEvent());
			parseMessage(event.getEvent(), event.getOrigin());
		}	
	}
}
