package edu.rochester.cs.quagents3.main;


import java.awt.*;
import java.io.IOException;
import java.net.UnknownHostException;
import java.util.*;
import java.util.List;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.GroupLayout;
import edu.rochester.cs.quagents3.client.*;

public class ControlPanel extends JPanel{
	protected List<QuagentState> quagents; //we don't deal with actual connections - only director
									   //thus we only need data structures to store most recent director data
	protected static GOD director;
	protected QuagentPanel quagpane;
	protected WorldPanel worldpane;
	protected JTabbedPane tabpane;
	protected TextArea evwindow;
	protected final static int WIDTH = 600;
	protected final static int HEIGHT = 400;
	
	/**
	 * Control panel takes abitrary # of statically created
	 * Quagents but can also handle dynamically added Quagents with events. 
	 */
	public ControlPanel(GOD director){
		ControlPanel.director = director;
		this.quagents = new LinkedList<QuagentState>();
		director.addEventListener(new DefaultGODEventListener());
		quagpane = new QuagentPanel();
		worldpane = new WorldPanel();
		evwindow = new TextArea("",14,73,TextArea.SCROLLBARS_VERTICAL_ONLY);
		evwindow.setEditable(false);
		evwindow.setBackground(Color.white);
		tabpane = new JTabbedPane();
		tabpane.setSize(new Dimension(worldpane.getWidth(),worldpane.getHeight()));
		tabpane.addTab("Quagents", quagpane);
		tabpane.addTab("World", worldpane);
		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		this.add(tabpane);
		this.add(evwindow);
		this.setBackground(Color.white);
		addQuagent(new QuagentState("User",0)); //represents user player, should remove this eventually...
	}
	
	protected void addQuagent(QuagentState quagent){
		quagents.add(quagent);
		quagpane.addQuagent(quagent);
	}
	
	private class DefaultGODEventListener implements GODEventListener{
		public void playerDied(GODEvent event) {
			evwindow.append("Entity " + event.getID() + " (" + event.getName()
				+ ") died.\n");
		}
		public void playerDisconnected(GODEvent event) {
			evwindow.append("Entity " + event.getID() + " (" + event.getName()
					+ ") disconnected.\n");
			quagents.remove(findIndexFromEntityID(event.getID()));
		}
		public void playerJoined(GODEvent event) {
			evwindow.append("Entity " + event.getID() + " (" + event.getName()
					+ ") joined the game.\n");
			addQuagent(new QuagentState(event.getName(), event.getID()));
		}
	}
	
	//because quagents are not necessarily 1-1 with indexes
	protected int findIndexFromEntityID(int entid){
		for(int i=0; i<quagents.size(); i++){
			if(quagents.get(i).entityid == entid)
				return i;
		}
		return -1; //not good...
	}
	
	//todo - make take generic list of commands
	protected class QuagentPanel extends JPanel{
		ArrayList<String> names; //tracks quagent names, not too useful - used for jcombobox
		ArrayList<Integer> ids; //necessary because ids are not always sequential 
		JComboBox quagentselect;
		TextField health,armor,location; 
		JLabel healthlb, armorlb, locationlb, quagentselectlb;
		JButton healthref, armorref, locationref, fullref;
		volatile boolean refreshing = false;
		ImageIcon refreshicon = new ImageIcon("refresh.JPG");
		GetHealth healthcmd;
		GetArmor armorcmd;
		GetPosition locationcmd;
		SetHealth sethealth;
		SetArmor setarmor;
		
		private volatile int selectedquagent; //==entityid
		
		public QuagentPanel(){
			names = new ArrayList<String>();
			ids = new ArrayList<Integer>();
			for(int i=0; i<quagents.size(); i++){
				names.add(quagents.get(i).name);
				ids.add(quagents.get(i).entityid);
			}
			quagentselect = new JComboBox(names.toArray());
			quagentselect.addActionListener(new QuagentSelectListener());
			health = new TextField();
			health.addActionListener(new HealthEntryListener());
			armor = new TextField();
			armor.addActionListener(new ArmorEntryListener());
			location = new TextField();
			location.setEditable(false);
			healthlb = new JLabel("Health: ");
			healthref = new JButton("Refresh");
			healthref.addActionListener(new RefreshHealthListener());
			armorlb = new JLabel("Armor: ");
			armorref = new JButton("Refresh");
			armorref.addActionListener(new RefreshArmorListener());
			locationlb = new JLabel("Location: ");
			locationref = new JButton("Refresh");
			locationref.addActionListener(new RefreshLocationListener());
			fullref = new JButton("Refresh All");
			fullref.addActionListener(new RefreshAllListener());
			quagentselectlb = new JLabel();
			this.setBackground(Color.white);
			this.add(quagentselect);
			this.add(quagentselectlb);
			this.add(health); this.add(armor); this.add(location);
			this.add(healthlb); this.add(armorlb); this.add(locationlb);
			this.add(healthref); this.add(armorref); this.add(locationref);
			this.setPreferredSize(new Dimension(WIDTH,HEIGHT));
			GroupLayout layout = new GroupLayout(this);
			this.setLayout(layout);
			layout.setAutoCreateGaps(true);
			layout.setAutoCreateContainerGaps(true);
			layout.setHorizontalGroup(layout.createSequentialGroup()
					.addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
							.addComponent(quagentselect).addComponent(quagentselectlb).addComponent(healthlb).addComponent(armorlb)
							.addComponent(locationlb)).addGroup(layout.createParallelGroup()
							.addComponent(healthref).addComponent(armorref).addComponent(locationref)		
							).addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
							.addComponent(fullref).addComponent(health).addComponent(armor).addComponent(location)));
			layout.setVerticalGroup(layout.createSequentialGroup()
					.addComponent(quagentselect)
					.addGroup(layout.createParallelGroup()
					.addComponent(quagentselectlb).addComponent(fullref))
							
					.addGroup(layout.createParallelGroup()
					.addComponent(healthlb).addComponent(healthref).addComponent(health))
					
					.addGroup(layout.createParallelGroup()
					.addComponent(armorlb).addComponent(armorref).addComponent(armor))
					
					.addGroup(layout.createParallelGroup()
					.addComponent(locationlb).addComponent(locationref).addComponent(location)));
			repaint();
		}
		
		public void paintComponent(Graphics g){
			super.paintComponent(g);
			try{quagentselectlb.setText(quagents.get(findIndexFromEntityID(selectedquagent)).name);}
			catch(IndexOutOfBoundsException e1){System.err.println("ControlPanel error: quagent with entid: " + selectedquagent + " not found");}
			if(refreshing)
				refreshicon.paintIcon(quagpane, g, 560, 0);
		}
		
		private void switchQuagent(int entid){
			if(selectedquagent==entid) //we don't need to do anything
				return;
			selectedquagent = entid;
			refreshAll();
			repaint();
		}
		
		public void addQuagent(QuagentState quagent){
			//update data structures
			names.add(quagent.name);
			ids.add(quagent.entityid);
			//add to combo box
			quagentselect.addItem(quagent.name);			
			repaint();
		}
		
		//refreshes all, equivalent to pressing all "refresh" buttons simultaneously
		private void refreshAll(){
			refreshing = true;
			refreshHealth();
			refreshArmor();
			refreshLocation();
			repaint();
		}
		
		private void refreshHealth(){
			refreshing = true;
			healthcmd = new GetHealth(selectedquagent);
			healthcmd.addTerminationListener(new HealthTerminationListener());
			director.start(healthcmd);
			repaint();
		}
		
		private void refreshArmor(){
			refreshing = true;
			armorcmd = new GetArmor(selectedquagent);
			armorcmd.addTerminationListener(new ArmorTerminationListener());
			director.start(armorcmd);
			repaint();
		}
		
		private void refreshLocation(){
			refreshing = true;
			locationcmd = new GetPosition(selectedquagent);
			locationcmd.addTerminationListener(new LocationTerminationListener());
			director.start(locationcmd);
			repaint();
		}
		
		private void setHealth(int newhealth){
			sethealth = new SetHealth(selectedquagent,newhealth);
			director.start(sethealth);
		}
		
		private void setArmor(int newarmor){
			setarmor = new SetArmor(selectedquagent,newarmor);
			director.start(setarmor);
		}
		
		private class QuagentSelectListener implements ActionListener{
			public void actionPerformed(ActionEvent e) {
				switchQuagent(ids.get(((JComboBox)e.getSource()).getSelectedIndex()));
			}			
		}
		
		private class HealthEntryListener implements ActionListener{
			public void actionPerformed(ActionEvent e){
				try{setHealth(Integer.parseInt(health.getText()));}
				catch(NumberFormatException e1){
					health.setText("Invalid entry");
				}
			}
		}
		
		private class ArmorEntryListener implements ActionListener{
			public void actionPerformed(ActionEvent e){
				try{setArmor(Integer.parseInt(armor.getText()));}
				catch(NumberFormatException e1){
					armor.setText("Invalid entry");
				}
			}
		}
		
		private class RefreshAllListener implements ActionListener{
			public void actionPerformed(ActionEvent e){
				healthref.setEnabled(false); armorref.setEnabled(false); locationref.setEnabled(false);
				refreshAll();
				repaint();
			}
		}
		
		private class RefreshHealthListener implements ActionListener{
			public void actionPerformed(ActionEvent e){
				((JButton)e.getSource()).setEnabled(false);
				refreshHealth();
				repaint();
			}
		}
		
		
		private class RefreshArmorListener implements ActionListener{
			public void actionPerformed(ActionEvent e){
				((JButton)e.getSource()).setEnabled(false);
				refreshArmor();
				repaint();
			}
		}

		private class RefreshLocationListener implements ActionListener{
			public void actionPerformed(ActionEvent e){
				((JButton)e.getSource()).setEnabled(false);
				refreshLocation();
				repaint();
			}
		}
		
		private class HealthTerminationListener implements CommandTerminationListener{
			public void commandTerminated(Command com) {
				health.setText(""+healthcmd.getHealth());
				healthref.setEnabled(true);
				repaint();
			}
		}
		
		private class ArmorTerminationListener implements CommandTerminationListener{
			public void commandTerminated(Command com){
				armor.setText("" + armorcmd.getArmor());
				armorref.setEnabled(true);
				repaint();
			}
		}
		
		private class LocationTerminationListener implements CommandTerminationListener{
			public void commandTerminated(Command com){
				Double[] loc = locationcmd.getLocation();
				location.setText("X: " + loc[GetPosition.X] + "\tY: " + loc[GetPosition.Y] + "\tZ: " + loc[GetPosition.Z]);
				locationref.setEnabled(true);
				repaint();
			}
		}
	}
	
	private class HealthEntryListener implements ActionListener{
		public void actionPerformed(ActionEvent e) {
			
		}
	}
	
	protected static class WorldPanel extends JPanel{
		private static JTextField gravity, speedFactor, quantity, x, y, z, entityid, x1, y1, z1, x2, y2, z2;
		private static JLabel gravityLabel, speedLabel, speed, quantityLabel, locationLabel, xLabel, yLabel, zLabel, entityLabel, point1Label, point2Label,
						x1Label, y1Label, z1Label, x2Label, y2Label, z2Label;
		private static JButton gravityButton, speedButton, dropButton, pauseButton, screenshotButton, positionButton, distanceButton, gametypeButton, withinButton;
		private static JPanel gravityPanel, speedPanel, dropPanel, dropSubpanel1, dropSubpanel2, rightPanel, entityPanel, point1Panel, point2Panel, gamemapPanel, top;
		private static JComboBox speedControl, item, gametype, map;
		private static TextArea window;
		private static final String controls[] = {"Speed Up", "Slow Down"};
		private static final String items[] = {"Gold","Treasure"};
		private static final String mapnames[] = {"random map", "delta","firstroom", "hurdles", "largeroom", "medroom", "smallroom", "maze1"};
		private static final String gametypes[] = {"Free for All", "Tournament", "Team Deathmatch", "Capture the Flag"};
		private CommandType command = CommandType.SPEEDUP;
		private int itemNum = 0, gametypeNum = 0;
		//private static  final int width = 500;
		//private static final int height = 500;
		
		public WorldPanel() {			
			// TextFields
			gravity = new JTextField("800", 10);
			speedFactor = new JTextField("", 10);
			quantity = new JTextField("", 10);
			x = new JTextField("", 10);
			y = new JTextField("", 10);
			z = new JTextField("", 10);
			entityid = new JTextField("", 10);
			x1 = new JTextField("", 10);
			y1 = new JTextField("", 10);
			z1 = new JTextField("", 10);
			x2 = new JTextField("", 10);
			y2 = new JTextField("", 10);
			z2 = new JTextField("", 10);
			
			// Labels
			gravityLabel = new JLabel("Gravity:");
			speedLabel = new JLabel("Game Speed:");
			speed = new JLabel("1.0");
			quantityLabel = new JLabel("Quantity:");
			locationLabel = new JLabel("Location:");
			xLabel = new JLabel("x:");
			yLabel = new JLabel("y:");
			zLabel = new JLabel("z:");
			entityLabel = new JLabel("EntityID:");
			point1Label = new JLabel("Point 1:");
			point2Label = new JLabel("Point 2:");
			x1Label = new JLabel("x1:");
			y1Label = new JLabel("y1:");
			z1Label = new JLabel("z1:");
			x2Label = new JLabel("x2:");
			y2Label = new JLabel("y2:");
			z2Label = new JLabel("z2:");
			
			// Buttons
			gravityButton = new JButton("Set");
			gravityButton.setActionCommand("Gravity");
			gravityButton.addActionListener(new ButtonListener());
			speedButton = new JButton("Set");
			speedButton.setActionCommand("Speed");
			speedButton.addActionListener(new ButtonListener());
			dropButton = new JButton("Drop");
			dropButton.setActionCommand("Drop");
			dropButton.addActionListener(new ButtonListener());
			pauseButton = new JButton("Pause");
			pauseButton.setActionCommand("Pause");
			pauseButton.addActionListener(new ButtonListener());
			screenshotButton = new JButton("Screenshot");
			screenshotButton.setActionCommand("Screenshot");
			screenshotButton.addActionListener(new ButtonListener());
			positionButton = new JButton("Get Position");
			positionButton.setActionCommand("Position");
			positionButton.addActionListener(new ButtonListener());
			distanceButton = new JButton("Find Distance");
			distanceButton.setActionCommand("Distance");
			distanceButton.addActionListener(new ButtonListener());
			gametypeButton = new JButton("Set");
			gametypeButton.setActionCommand("GameMap");
			gametypeButton.addActionListener(new ButtonListener());
			withinButton = new JButton("Is Within?");
			withinButton.setActionCommand("Within");
			withinButton.addActionListener(new ButtonListener());
			
			speedControl = new JComboBox(controls);
			speedControl.addActionListener(new BehaviorListener());
			item = new JComboBox(items);
			item.addActionListener(new BehaviorListener());
			gametype = new JComboBox(gametypes);
			gametype.addActionListener(new BehaviorListener());
			map = new JComboBox(mapnames);
			map.addActionListener(new BehaviorListener());
			
			//window = new TextArea("",10,80,TextArea.SCROLLBARS_VERTICAL_ONLY);
			//((Object) window).setLineWrap(true);
			//window.setWrapStyleWord(true);

			
			//Panels
			GroupLayout layout;
			gravityPanel = new JPanel();
			gravityPanel.setLayout(new FlowLayout());
			layout = new GroupLayout(gravityPanel);
			gravityPanel.setLayout(layout);
			layout.setAutoCreateGaps(true);
			layout.setAutoCreateContainerGaps(true);
			layout.setHorizontalGroup(
				layout.createSequentialGroup()
					.addComponent(gravityLabel)
					.addComponent(gravity)
					.addComponent(gravityButton)
			);
			layout.setVerticalGroup(
				layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
					.addComponent(gravityLabel)
					.addComponent(gravity)
					.addComponent(gravityButton)
			);
			//gravityPanel.setBackground(Color.blue);
			
			speedPanel = new JPanel();
			layout = new GroupLayout(speedPanel);
			speedPanel.setLayout(layout);
			layout.setAutoCreateGaps(true);
			layout.setAutoCreateContainerGaps(true);
			layout.setHorizontalGroup(
				layout.createSequentialGroup()
					.addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
						.addComponent(speedLabel)
					   	.addComponent(speedControl))
					.addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING) 
						.addComponent(speed)
						.addComponent(speedFactor)
						.addComponent(speedButton))
			);
			layout.setVerticalGroup(
				layout.createSequentialGroup()
					.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(speedLabel)
						.addComponent(speed))
					.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(speedControl)
						.addComponent(speedFactor))
					.addComponent(speedButton)
			);
			//speedPanel.setBackground(Color.red);
			
			dropSubpanel1 = new JPanel();
			layout = new GroupLayout(dropSubpanel1);
			dropSubpanel1.setLayout(layout);
			layout.setAutoCreateGaps(true);
			layout.setAutoCreateContainerGaps(true);
			layout.setHorizontalGroup(
				layout.createSequentialGroup()
					.addComponent(quantityLabel)
					.addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
						.addComponent(item)
						.addComponent(quantity))
			);
			layout.setVerticalGroup(
				layout.createSequentialGroup()
					.addComponent(item)
					.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(quantityLabel)
						.addComponent(quantity))
			);
			
			dropSubpanel2 = new JPanel();
			layout = new GroupLayout(dropSubpanel2);
			dropSubpanel2.setLayout(layout);
			layout.setAutoCreateGaps(true);
			layout.setAutoCreateContainerGaps(true);
			layout.setHorizontalGroup(
				layout.createSequentialGroup()
					.addComponent(locationLabel)
					.addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
						.addComponent(xLabel)
						.addComponent(yLabel)
						.addComponent(zLabel))
					.addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
						.addComponent(x)
						.addComponent(y)
						.addComponent(z))
			);
			layout.setVerticalGroup(
				layout.createSequentialGroup()
					.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(locationLabel)
						.addComponent(xLabel)
						.addComponent(x))
					.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(yLabel)
						.addComponent(y))
					.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(zLabel)
						.addComponent(z))
			);
			
			dropPanel = new JPanel();
			layout = new GroupLayout(dropPanel);
			dropPanel.setLayout(layout);
			layout.setAutoCreateGaps(true);
			layout.setAutoCreateContainerGaps(true);
			layout.setHorizontalGroup(
				layout.createParallelGroup(GroupLayout.Alignment.CENTER)
					.addComponent(dropSubpanel1)
					.addComponent(dropSubpanel2)
					.addComponent(dropButton)
			);
			layout.setVerticalGroup(
				layout.createSequentialGroup()
					.addComponent(dropSubpanel1)
					.addComponent(dropSubpanel2)
					.addComponent(dropButton)
			);
			//dropPanel.setBackground(Color.green);
			
			entityPanel = new JPanel();
			layout = new GroupLayout(entityPanel);
			entityPanel.setLayout(layout);
			layout.setAutoCreateGaps(true);
			layout.setAutoCreateContainerGaps(true);
			layout.setHorizontalGroup(
				layout.createSequentialGroup()
					.addComponent(entityLabel)
					.addComponent(entityid)
					.addComponent(positionButton)
			);
			layout.setVerticalGroup(
				layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
					.addComponent(entityLabel)
					.addComponent(entityid)
					.addComponent(positionButton)
			);
			
			point1Panel = new JPanel();
			layout = new GroupLayout(point1Panel);
			point1Panel.setLayout(layout);
			layout.setAutoCreateGaps(true);
			layout.setAutoCreateContainerGaps(true);
			layout.setHorizontalGroup(
				layout.createSequentialGroup()
					.addComponent(point1Label)
					.addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
						.addComponent(x1Label)
						.addComponent(y1Label)
						.addComponent(z1Label))
					.addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
						.addComponent(x1)
						.addComponent(y1)
						.addComponent(z1))
			);
			layout.setVerticalGroup(
				layout.createSequentialGroup()
					.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(point1Label)
						.addComponent(x1Label)
						.addComponent(x1))
					.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(y1Label)
						.addComponent(y1))
					.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(z1Label)
						.addComponent(z1))
			);
			
			point2Panel = new JPanel();
			layout = new GroupLayout(point2Panel);
			point2Panel.setLayout(layout);
			layout.setAutoCreateGaps(true);
			layout.setAutoCreateContainerGaps(true);
			layout.setHorizontalGroup(
				layout.createSequentialGroup()
					.addComponent(point2Label)
					.addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
						.addComponent(x2Label)
						.addComponent(y2Label)
						.addComponent(z2Label))
					.addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
						.addComponent(x2)
						.addComponent(y2)
						.addComponent(z2))
			);
			layout.setVerticalGroup(
				layout.createSequentialGroup()
					.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(point2Label)
						.addComponent(x2Label)
						.addComponent(x2))
					.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(y2Label)
						.addComponent(y2))
					.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(z2Label)
						.addComponent(z2))
			);
			
			rightPanel = new JPanel();
			layout = new GroupLayout(rightPanel);
			rightPanel.setLayout(layout);
			layout.setAutoCreateGaps(true);
			layout.setAutoCreateContainerGaps(true);
			layout.setHorizontalGroup(
				layout.createParallelGroup(GroupLayout.Alignment.LEADING)
					.addComponent(entityPanel)
					.addComponent(point1Panel)
					.addComponent(point2Panel)
					.addGroup(layout.createSequentialGroup()
						.addComponent(distanceButton)
						.addComponent(withinButton))
			);
			layout.setVerticalGroup(
				layout.createSequentialGroup()
					.addComponent(entityPanel)
					.addComponent(point1Panel)
					.addComponent(point2Panel)
					.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(distanceButton)
						.addComponent(withinButton))
			);
			//rightPanel.setBackground(Color.orange);
			
			gamemapPanel = new JPanel();
			layout = new GroupLayout(gamemapPanel);
			gamemapPanel.setLayout(layout);
			layout.setAutoCreateGaps(true);
			layout.setAutoCreateContainerGaps(true);
			layout.setHorizontalGroup(
				layout.createSequentialGroup()
					.addGroup(layout.createParallelGroup(GroupLayout.Alignment.TRAILING)
						.addComponent(gametype)
						.addComponent(gametypeButton))
					.addComponent(map)
			);
			layout.setVerticalGroup(
				layout.createSequentialGroup()
					.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(gametype)
						.addComponent(map))
					.addComponent(gametypeButton)
			);
			//gamemapPanel.setBackground(Color.white);
			
			top = new JPanel();
			layout = new GroupLayout(top);
			top.setLayout(layout);
			layout.setAutoCreateGaps(true);
			layout.setAutoCreateContainerGaps(true);
			layout.setHorizontalGroup(
				layout.createSequentialGroup()
					.addGroup(layout.createParallelGroup(GroupLayout.Alignment.CENTER)
						.addComponent(screenshotButton)
						.addComponent(gravityPanel)
					   	.addComponent(speedPanel)
					   	.addComponent(dropPanel))
					.addGroup(layout.createParallelGroup(GroupLayout.Alignment.CENTER)
							.addComponent(pauseButton)
							.addComponent(rightPanel)
							.addComponent(gamemapPanel))
			);
			layout.setVerticalGroup(
				layout.createParallelGroup(GroupLayout.Alignment.CENTER)
					.addGroup(layout.createSequentialGroup()
						.addComponent(screenshotButton)
						.addComponent(gravityPanel)
						.addComponent(speedPanel)
					    .addComponent(dropPanel))
					.addGroup(layout.createSequentialGroup()
						.addComponent(pauseButton)
						.addComponent(rightPanel)
						.addComponent(gamemapPanel))
			);
			top.setBackground(Color.white);
			
			layout = new GroupLayout(this);
			this.setLayout(layout);
			layout.setAutoCreateGaps(true);
			layout.setAutoCreateContainerGaps(true);
			layout.setHorizontalGroup(
				layout.createParallelGroup(GroupLayout.Alignment.LEADING)
					.addComponent(top)
					//.addComponent(window)
			);
			layout.setVerticalGroup(
				layout.createSequentialGroup()
					.addComponent(top)
					//.addComponent(window)
			);


			this.setBackground(Color.white);
			//this.setPreferredSize(new Dimension(width,height));	
			repaint();
		}
		
		public void paintComponent(Graphics g){
			super.paintComponent(g);
		}
		
		private class BehaviorListener implements ActionListener{
			public BehaviorListener(){}
			public void actionPerformed(ActionEvent e) {
				String behaviortype = (String) ((JComboBox) e.getSource()).getSelectedItem();
				if(behaviortype.equals("Speed Up")){
					command = CommandType.SPEEDUP;
				}
				else if(behaviortype.equals("Slow Down")){
					command = CommandType.SLOWMO;
				}
				else if(behaviortype.equals("Gold")){
					itemNum = 0;
				}
				else if(behaviortype.equals("Treasure")){
					itemNum = 1;
				}
				else if(behaviortype.equals("Free for All")){
					gametypeNum = 0;
				}
				else if(behaviortype.equals("Tournament")){
					gametypeNum = 1;
				}
				else if(behaviortype.equals("Team Deathmatch")){
					gametypeNum = 3;
				}
				else if(behaviortype.equals("Capture the Flag")){
					gametypeNum = 4;
				}
			}
		}
		
		private class ButtonListener implements ActionListener{
			public ButtonListener(){}
			public void actionPerformed(ActionEvent e) {
				if ("Gravity".equals(e.getActionCommand())) {
					director.execute(new SetGravity(Integer.parseInt(gravity.getText())));
				}
				else if ("Speed".equals(e.getActionCommand())) {
					if (command == CommandType.SPEEDUP) {
						float factor = Float.parseFloat(speedFactor.getText());
						director.execute(new SpeedUp(factor));
						speed.setText(factor + "");
					}
					else if (command == CommandType.SLOWMO) {
						float factor = Float.parseFloat(speedFactor.getText());
						director.execute(new SlowMo(factor));
						speed.setText(1/factor + "");
					}
				}
				else if ("Position".equals(e.getActionCommand())) {
					director.execute(new GetPosition(Integer.parseInt(entityid.getText())));
				}
				else if ("Pause".equals(e.getActionCommand())) {
					director.execute(new GODPause());
				}
				else if ("Screenshot".equals(e.getActionCommand())) {
					director.execute(new Screenshot());
				}
				else if ("Drop".equals(e.getActionCommand())) {
					director.execute(new Drop(itemNum, Integer.parseInt(quantity.getText()), Float.parseFloat(x.getText()), 
							Float.parseFloat(y.getText()), Float.parseFloat(z.getText())));
				}
				else if ("Distance".equals(e.getActionCommand())) {
					director.execute(new FindDistance(Float.parseFloat(x1.getText()), Float.parseFloat(y1.getText()), 
							Float.parseFloat(z1.getText()), Float.parseFloat(x2.getText()), Float.parseFloat(y2.getText()), 
							Float.parseFloat(z2.getText())));
				}
				else if ("Within".equals(e.getActionCommand())) {
					director.execute(new IsWithin(Integer.parseInt(entityid.getText()), Float.parseFloat(x1.getText()), 
							Float.parseFloat(y1.getText()), Float.parseFloat(z1.getText()), Float.parseFloat(x2.getText()), 
							Float.parseFloat(y2.getText()), Float.parseFloat(z2.getText())));
				}
				else if ("GameMap".equals(e.getActionCommand())) {
					director.execute(new GametypeAndMapControl(gametypeNum, (String) map.getSelectedItem()));
				}
			}
		}
	}
}
