package edu.rochester.cs.quagents3.main;

import java.io.IOException;

import javax.swing.JFrame;
import javax.swing.JPanel;

import edu.rochester.cs.quagents3.client.CTFPlayer;
import edu.rochester.cs.quagents3.client.GOD;
import edu.rochester.cs.quagents3.client.GametypeAndMapControl;
import edu.rochester.cs.quagents3.util.StoppableThread;

//import edu.rochester.cs.quagents3.client;

//what a user needs to specify for a scenario
//this is related to engine config file parameters
abstract public class Scenario extends Thread{
	public static final String configloc = System.getenv("HOME") + ".quaugents3/quagents3/quagents-config.cfg";
	protected final String name; //name of scenario
	public final String[] maps; //possible maps
	protected String selectedmap; //map selected for this scenario instance
	protected int randwidth;
	protected int randheight;
	protected GOD director;
	
	protected Scenario(String name, String[] maps){
		this.name = name;
		this.maps = maps;
	}
	
	public void run(){execute();}
	
	//same stuff as execute, but we assume server is already running.
	public void executeBody(String map){
		if(!validateMap(map)) {System.err.println("error: please specify a valid map for " + name);
		ScenarioStarter.usage(); System.exit(-1);}
		selectedmap = map;
		System.out.println("Starting " + name + "...");
		try{
		System.out.println("Connecting director");
		director = new GOD("127.0.0.1");
		sleep(1000);
		System.out.println("Entering map");
		director.execute(new GametypeAndMapControl(selectedmap));}
		catch(Exception e1){System.err.println("error: please specify a running Quagents3 server"); System.exit(-1);}
		System.out.println("Initializing scenario");
		initialize();
		System.out.println("Intializing GUI");
		initGUI(); //because we don't run Quagents main
		System.out.println("Creating GUI");
		setGUI();
		System.out.println("Executing scenario");
		runQuagentCode();
	}
	
	private void initGUI(){
		if(Quagents.frame==null){
			Quagents.frame = new JFrame("Quagents3");
			Quagents.frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			Quagents.init = new JPanel();
			Quagents.frame.add(Quagents.init);
			Quagents.frame.pack();
			Quagents.frame.setVisible(true);
		}
	}
	private boolean validateMap(String map){
		for(int i=0; i<maps.length; i++)
			if(maps[i].equals(map))
				return true;
		return false;
	}
	
	public void execute(){
		System.out.println("Initializing "+ name + "...");
		Config.config();
		try{configure();}
		catch(IOException e1){System.err.println("Configuration error: "); e1.printStackTrace();}
		System.out.println("Running "+ name +"...");System.out.flush();
		runQuake();
		initialize();
		setGUI();
		runQuagentCode(); //may need to break this into a separate thread
	}
	
	public String getScenarioName(){
		return name;
	}
	
	public String getSelectedMap(){
		return selectedmap;
	}
	
	public void setSelectedMap(String selectedmap){
		this.selectedmap = selectedmap;
	}
	
	public void setRandMapVars(int width, int height){
		randwidth = width;
		randheight = height;
	}
	/**
	 * Config, init scenario. Launch quagent code.
	 */

	abstract protected void configure() throws IOException;
	abstract protected void setGUI();
	abstract protected void runQuagentCode(); 
	abstract protected void initialize();
	
	protected void runQuake(){
		if(selectedmap.equals("random map"))
			new Exec(randwidth,randheight).run();
		else
			new GameStarter(selectedmap).run();
		waitForQuake(); //so we don't try to connect before Quake is ready
	}
	
	//this could be improved...
	protected void waitForQuake(){
		try {
			Thread.sleep(8000);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
