package edu.rochester.cs.quagents3.main;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

public class ScenarioGUI extends JPanel{
	//parallel arrays
	protected Scenario[] scenarios;
	protected JButton[] scenariobuttons;
	protected JComboBox[] scenariomaps;
	
	protected JComboBox[] randwidth;
	protected JComboBox[] randheight;
	protected JButton[] submit;
	public JLabel loading;
	
	protected ImageIcon quagentslogo; //todo
	protected ImageIcon logo;
	
	public ScenarioGUI(Scenario ... scenarios){
		this.scenarios = scenarios;
		scenariobuttons = new JButton[scenarios.length];
		scenariomaps = new JComboBox[scenarios.length];
		randwidth = new JComboBox[scenarios.length];
		randheight = new JComboBox[scenarios.length];
		submit = new JButton[scenarios.length];
		logo = new ImageIcon("quake3-tango.png");
		String[] randsizeoptns = {"1","2","3","4","5","6","7"};
		
		for(int i=0; i<scenarios.length; i++){
			scenariobuttons[i] = new JButton(scenarios[i].name);
			scenariomaps[i] = new JComboBox(scenarios[i].maps);
			randwidth[i] = new JComboBox(randsizeoptns); 
			randheight[i] = new JComboBox(randsizeoptns);
			submit[i] = new JButton("Begin"); 
			scenariobuttons[i].addActionListener(new ScenarioListener(i));
			scenariomaps[i].addActionListener(new MapListener(i));
			scenariobuttons[i].setToolTipText("Click to execute the " + scenarios[i].name + " scenario.");
			scenariomaps[i].setToolTipText("Select map which the scenario will take place on.");
			scenariomaps[i].setVisible(false);
			randwidth[i].setVisible(false); 
			randwidth[i].setSelectedIndex(3);
			randwidth[i].setToolTipText("The number of cells (\"rooms\") wide");
			randheight[i].setVisible(false);
			randheight[i].setSelectedIndex(3);
			randheight[i].setToolTipText("The number of cells (\"rooms\") long");
			submit[i].setVisible(false);
		}
		loading = new JLabel("Loading scenario...");
		loading.setVisible(false);
		
		for(int i=0; i<scenariobuttons.length; i++){
			this.add(scenariobuttons[i]);
			this.add(scenariomaps[i]);
			this.add(randwidth[i]);
			this.add(randheight[i]);
			this.add(submit[i]);
		}
		this.add(loading);
		this.setBackground(Color.white);
		this.setPreferredSize(new Dimension(260,375+scenarios.length * 50));
		repaint();
	}
	
	public void paintComponent(Graphics g){
		super.paintComponent(g);
		
		logo.paintIcon(this, g, 0, 0);
		for(int i=0; i<scenariobuttons.length; i++){
			scenariobuttons[i].setLocation(50, 350 + i*50);
			scenariomaps[i].setLocation(50,350+i*50);
			randwidth[i].setLocation(80, 350+i*50);
			randheight[i].setLocation(130, 350+i*50);
			submit[i].setLocation(90, 385+i*50);
		}
		
		loading.setSize(this.getWidth()-100, 100);
		loading.setLocation(70,375);
	}
	
	private class ScenarioListener implements ActionListener{
		private int index;
		
		public ScenarioListener(int index){
			this.index = index;
		}
		
		//execute, and enter next screen
		public void actionPerformed(ActionEvent e) {
			for(int i=0; i<scenariobuttons.length; i++)
				scenariobuttons[i].setVisible(false);
			scenariomaps[index].setVisible(true);
/*			synchronized(scenariomaps[index]){
				try {scenariomaps[index].wait();} 
				catch (InterruptedException e1) {e1.printStackTrace(); System.exit(-1);}
			}
			scenarios[index].execute((String)scenariomaps[index].getSelectedItem());
 		*/
		}
	}
	
	private class MapListener implements ActionListener{
		private int index;
		
		public MapListener(int index){
			this.index = index;
		}
		
		public void actionPerformed(ActionEvent e) {
			scenariomaps[index].setVisible(false);
			/*
			synchronized(list){
				list.notify();
			}*/
			scenarios[index].setSelectedMap((String)scenariomaps[index].getSelectedItem());
			if(scenarios[index].getSelectedMap().equals("randommap")){
				submit[index].setToolTipText("Click to begin " + scenarios[index].getSelectedMap() + " on a random map");
				submit[index].addActionListener(new SubmitButtonListener(index));
				randwidth[index].setVisible(true);
				randheight[index].setVisible(true);
				submit[index].setVisible(true);
			}
			else{
				loading.setVisible(true);
				scenarios[index].start();
			}
		}
	}
	
	private class SubmitButtonListener implements ActionListener{
		private int index;
		
		public SubmitButtonListener(int index){
			this.index = index;
		}
		
		public void actionPerformed(ActionEvent e) {
			submit[index].setVisible(false); randwidth[index].setVisible(false); randheight[index].setVisible(false);
			scenarios[index].setRandMapVars(randwidth[index].getSelectedIndex()+1, randheight[index].getSelectedIndex()+1);
			loading.setVisible(true);
			scenarios[index].start();
		}
	}
}
