/*
 * File: GUIQuagentController.java
 * Creator: George Ferguson
 * Created: Mon May 21 12:14:18 2012
 * Time-stamp: <Tue Jun  5 14:52:59 EDT 2012 ferguson>
 */

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import javax.swing.*;

/**
 * Simple demo of connecting to the Quagents3 server and creating a qbot
 * that you can control from the UI. Buttons send protocol messages to the
 * server (for your qbot).
 * <p>
 * To run, first start the server with a map like `firstroom'. Then run this
 * Java class in another terminal window. Your agent should appear in the
 * room. You can have it move with the UI buttons. You should see messages
 * from the server as commands complete.
 */
public class GUIQuagentController extends QuagentController implements ActionListener {

    /**
     * Construct and return a new GUIQuagentController connected to the
     * Quagents3 server.
     */
    public GUIQuagentController() throws IOException {
	super();
	initUI();
    }

    protected JFrame frame;

    /**
     * Initialize this GUIQuagentController's user interface.
     */
    protected void initUI() {
	// Frame
	frame = new JFrame("GUIQuagentController");
	frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	// Contents
	Container contents = frame.getContentPane();
	contents.setLayout(new BoxLayout(contents, BoxLayout.PAGE_AXIS));
	// Buttons
	JPanel buttonPanel = new JPanel();
	buttonPanel.setLayout(new BorderLayout());
	JButton button = new JButton("Forward");
	button.addActionListener(this);
	buttonPanel.add(button, BorderLayout.NORTH);
	button = new JButton("Back");
	button.addActionListener(this);
	buttonPanel.add(button, BorderLayout.SOUTH);
	button = new JButton("Left");
	button.addActionListener(this);
	buttonPanel.add(button, BorderLayout.WEST);
	button = new JButton("Right");
	button.addActionListener(this);
	buttonPanel.add(button, BorderLayout.EAST);
	button = new JButton("STOP");
	button.addActionListener(this);
	buttonPanel.add(button, BorderLayout.CENTER);
	contents.add(buttonPanel);
	// Text area to display messages
	msgTA = new JTextArea(6, 25);
	JScrollPane scroller = new JScrollPane(msgTA);
	scroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
	scroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
	contents.add(scroller);
	// Done
	frame.pack();
	frame.setVisible(true);
    }

    protected JTextArea msgTA;

    /**
     * Called when a UI button is clicked: sends appropriate Quagents3
     * protocol message to the server.
     * Note that this method is called on the Swing event thread.
     */
    public void actionPerformed(ActionEvent e) {
	String action = e.getActionCommand();
	if (action.equals("Forward")) {
	    sendCommand("mi", "0 0 0.25");
	} else if (action.equals("Back")) {
	    sendCommand("mi", "0 0 -0.25");
	} else if (action.equals("Left")) {
	    sendCommand("ro", "90 0");
	} else if (action.equals("Right")) {
	    sendCommand("ro", "-90 0");
	} else if (action.equals("STOP")) {
	    sendCommand("fa");
	}
    }

    /**
     * Overrides QuagentController.perceive() to display received messages
     * in the UI rather than printing to stdout.
     */
    @Override
    protected void perceive() throws IOException {
	final String msg = readMessage();
	// Update UI on Swing event thread
	SwingUtilities.invokeLater(new Runnable() {
		public void run() {
		    // JTextArea.append() is thread-safe...
		    msgTA.append(msg);
		    msgTA.append("\n");
		    // But JTextComponent.setCaretPosition is not
		    msgTA.setCaretPosition(msgTA.getDocument().getLength());
		}
	    });
    }

    /**
     * Main method for GUIQuagentController: construct a GUIQuagentController
     * and run it on the main thread (for reading messages).
     */
    public static void main(String[] argv) throws IOException {
	GUIQuagentController controller = new GUIQuagentController();
	controller.run();
    }

}
